@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <sigc++/functors/mem_fun.h>
+#include <uuid/uuid.h>
 
 #include "GameScenario.h"
 #include "MapGenerator.h"
@@ -65,24 +66,24 @@ using namespace std;
 #define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 //#define debug(x)
 
-GameScenario::GameScenario(std::string name,std::string comment, bool turnmode)
-    :d_name(name),d_comment(comment), d_turnmode(turnmode)
+GameScenario::GameScenario(std::string name,std::string comment, bool turnmode,
+			   GameScenario::PlayMode playmode)
+    :d_name(name),d_comment(comment), d_turnmode(turnmode), d_playmode(playmode)
 {
     Armysetlist::getInstance();
-    Armysetlist::getInstance()->instantiatePixmaps();
     Tilesetlist::getInstance();
-    Tilesetlist::getInstance()->instantiatePixmaps();
     Shieldsetlist::getInstance();
-    Shieldsetlist::getInstance()->instantiatePixmaps();
 
     if (fl_counter == 0)
-        fl_counter = new FL_Counter();
+      fl_counter = new FL_Counter();
+
+    setNewRandomId();
 }
 
 // savegame is a filename with absolute path!
 
 GameScenario::GameScenario(string savegame, bool& broken)
-  :d_turnmode(true)
+  :d_turnmode(true), d_playmode(GameScenario::HOTSEAT)
 {
   XML_Helper helper(savegame, ios::in, Configuration::s_zipfiles);
   broken = loadWithHelper(helper);
@@ -90,7 +91,7 @@ GameScenario::GameScenario(string savegame, bool& broken)
 }
 
 GameScenario::GameScenario(XML_Helper &helper, bool& broken)
-  : d_turnmode(true)
+  : d_turnmode(true), d_playmode(GameScenario::HOTSEAT)
 {
   broken = loadWithHelper(helper);
 }
@@ -241,11 +242,8 @@ void GameScenario::setupDiplomacy(bool diplomacy)
 bool GameScenario::loadWithHelper(XML_Helper& helper)
 {
   Armysetlist::getInstance();
-  Armysetlist::getInstance()->instantiatePixmaps();
   Tilesetlist::getInstance();
-  Tilesetlist::getInstance()->instantiatePixmaps();
   Shieldsetlist::getInstance();
-  Shieldsetlist::getInstance()->instantiatePixmaps();
 
   bool broken = false;
 
@@ -334,7 +332,7 @@ bool GameScenario::saveGame(string filename, string extension) const
     }
   else 
     {
-      debug(_("The Filename lacks the extension --> ") << strtoken->getLastToken())
+      debug(_("The Filename lacks the extension --> ") << extension)
 	goodfilename += "." + extension;
     }
 
@@ -378,6 +376,7 @@ bool GameScenario::saveWithHelper(XML_Helper &helper) const
 
   //save the private GameScenario data last due to dependencies
   retval &= helper.openTag("scenario");
+  retval &= helper.saveData("id", d_id);
   retval &= helper.saveData("name", d_name);
   retval &= helper.saveData("comment", d_comment);
   retval &= helper.saveData("turn", s_round);
@@ -388,13 +387,17 @@ bool GameScenario::saveWithHelper(XML_Helper &helper) const
   retval &= helper.saveData("hidden_map", s_hidden_map);
   retval &= helper.saveData("diplomacy", s_diplomacy);
   retval &= helper.saveData("cusp_of_war", s_cusp_of_war);
-  retval &= helper.saveData("neutral_cities", (int) s_neutral_cities);
-  retval &= helper.saveData("razing_cities", (int) s_razing_cities);
+  std::string neutral_cities_str = Configuration::neutralCitiesToString(GameParameters::NeutralCities(s_neutral_cities));
+  retval &= helper.saveData("neutral_cities", neutral_cities_str);
+  std::string razing_cities_str = Configuration::razingCitiesToString(GameParameters::RazingCities(s_razing_cities));
+  retval &= helper.saveData("razing_cities", razing_cities_str);
   retval &= helper.saveData("intense_combat", s_intense_combat);
   retval &= helper.saveData("military_advisor", s_military_advisor);
   retval &= helper.saveData("random_turns", s_random_turns);
   retval &= helper.saveData("surrender_already_offered", 
 			    s_surrender_already_offered);
+  std::string playmode_str = playModeToString(GameScenario::PlayMode(d_playmode));
+  retval &= helper.saveData("playmode", playmode_str);
 
   retval &= helper.closeTag();
 
@@ -416,7 +419,9 @@ bool GameScenario::load(std::string tag, XML_Helper* helper)
 	}
 
       debug("loading scenario")
-	helper->getData(s_round, "turn");
+
+      helper->getData(d_id, "id");
+      helper->getData(s_round, "turn");
       helper->getData(d_turnmode, "turnmode");
       helper->getData(d_name, "name");
       helper->getData(d_comment, "comment");
@@ -426,17 +431,20 @@ bool GameScenario::load(std::string tag, XML_Helper* helper)
       helper->getData(s_hidden_map, "hidden_map");
       helper->getData(s_diplomacy, "diplomacy");
       helper->getData(s_cusp_of_war, "cusp_of_war");
-      int val = -1;
-      helper->getData(val, "neutral_cities");
-      s_neutral_cities = GameParameters::NeutralCities (val);
-      val = -1;
-      helper->getData(val, "razing_cities");
-      s_razing_cities = GameParameters::RazingCities (val);
+      std::string neutral_cities_str;
+      helper->getData(neutral_cities_str, "neutral_cities");
+      s_neutral_cities = Configuration::neutralCitiesFromString(neutral_cities_str);
+      std::string razing_cities_str;
+      helper->getData(razing_cities_str, "razing_cities");
+      s_razing_cities = Configuration::razingCitiesFromString(razing_cities_str);
       helper->getData(s_intense_combat, "intense_combat");
       helper->getData(s_military_advisor, "military_advisor");
       helper->getData(s_random_turns, "random_turns");
       helper->getData(s_surrender_already_offered, 
 		      "surrender_already_offered");
+      std::string playmode_str;
+      helper->getData(playmode_str, "playmode");
+      d_playmode = GameScenario::playModeFromString(playmode_str);
 
       return true;
     }
@@ -573,3 +581,41 @@ void GameScenario::nextRound()
     }
 }
 
+std::string GameScenario::playModeToString(const GameScenario::PlayMode mode)
+{
+  switch (mode)
+    {
+      case GameScenario::HOTSEAT:
+	return "GameScenario::HOTSEAT";
+	break;
+      case GameScenario::NETWORKED:
+	return "GameScenario::NETWORKED";
+	break;
+      case GameScenario::PLAY_BY_MAIL:
+	return "GameScenario::PLAY_BY_MAIL";
+	break;
+    }
+  return "GameScenario::HOTSEAT";
+}
+
+GameScenario::PlayMode GameScenario::playModeFromString(const std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return GameScenario::PlayMode(atoi(str.c_str()));
+  if (str == "GameScenario::HOTSEAT")
+    return GameScenario::HOTSEAT;
+  else if (str == "GameScenario::NETWORKED")
+    return GameScenario::NETWORKED;
+  else if (str == "GameScenario::PLAY_BY_MAIL")
+    return GameScenario::PLAY_BY_MAIL;
+  return GameScenario::HOTSEAT;
+}
+	
+void GameScenario::setNewRandomId()
+{
+  char buf[40];
+  uuid_t uu;
+  uuid_generate_time(uu);
+  uuid_unparse(uu, buf);
+  d_id = buf;
+}
