@@ -18,10 +18,12 @@
 //  02110-1301, USA.
 
 #include <assert.h>
+#include <SDL_image.h>
 
 #include "rectangle.h"
 
 #include "GraphicsCache.h"
+#include "GraphicsLoader.h"
 #include "armysetlist.h"
 #include "shieldsetlist.h"
 #include "army.h"
@@ -44,7 +46,7 @@ struct ArmyCacheItem
 {
     Uint32 armyset;
     Uint32 index;
-    const Player* player;
+    Uint32 player_id;
     bool medals[3];
     SDL_Surface* surface;
 };
@@ -52,14 +54,14 @@ struct ArmyCacheItem
 //the structure to store ships in
 struct ShipCacheItem
 {
-    const Player* player;
+    Uint32 player_id;
     SDL_Surface* surface;
 };
 
 //the structure to store planted standard in
 struct PlantedStandardCacheItem
 {
-    const Player* player;
+    Uint32 player_id;
     SDL_Surface* surface;
 };
 
@@ -117,14 +119,14 @@ struct CursorCacheItem
 struct CityCacheItem
 {
     int type;
-    const Player* player;
+    Uint32 player_id;
     SDL_Surface* surface;
 };
 
 //the structure to store towers in
 struct TowerCacheItem
 {
-    const Player* player;
+    Uint32 player_id;
     SDL_Surface* surface;
 };
 
@@ -132,7 +134,7 @@ struct TowerCacheItem
 struct FlagCacheItem
 {
     Uint32 size;
-    const Player* player;
+    Uint32 player_id;
     SDL_Surface* surface;
 };
 
@@ -141,7 +143,7 @@ struct SelectorCacheItem
 {
     Uint32 type;
     Uint32 frame;
-    const Player* player;
+    Uint32 player_id;
     SDL_Surface* surface;
 };
 
@@ -202,23 +204,34 @@ GraphicsCache::GraphicsCache()
     loadBridgePics();
     loadCursorPics();
     loadFlags();
+    for (int i = 0; i < d_selector.size(); i++)
+      {
+        d_selector[i] = NULL;
+        d_selectormask[i] = NULL;
+      }
+
+    for (int i = 0; i < d_smallselector.size(); i++)
+      {
+        d_smallselector[i] = NULL;
+        d_smallselectormask[i] = NULL;
+      }
     loadSelectors();
     loadProdShields();
     loadMoveBonusPics();
 
-    d_medalsmask = File::getMiscPicture("medals_mask.png");
-    d_smallruinedcity = File::getMiscPicture("smallruinedcity.png");
-    d_smallhero = File::getMiscPicture("hero.png");
-    d_small_ruin_unexplored = File::getMiscPicture("smallunexploredruin.png");
+    d_medalsmask = GraphicsLoader::getMiscPicture("medals_mask.png");
+    d_smallruinedcity = GraphicsLoader::getMiscPicture("smallruinedcity.png");
+    d_smallhero = GraphicsLoader::getMiscPicture("hero.png");
+    d_small_ruin_unexplored = GraphicsLoader::getMiscPicture("smallunexploredruin.png");
     d_small_stronghold_unexplored = 
-      File::getMiscPicture("smallunexploredstronghold.png");
-    d_small_ruin_explored = File::getMiscPicture("smallexploredruin.png");
-    d_small_temple = File::getMiscPicture("smalltemple.png");
+      GraphicsLoader::getMiscPicture("smallunexploredstronghold.png");
+    d_small_ruin_explored = GraphicsLoader::getMiscPicture("smallexploredruin.png");
+    d_small_temple = GraphicsLoader::getMiscPicture("smalltemple.png");
     std::string tileset = GameMap::getInstance()->getTileset()->getSubDir();
     std::string cityset = GameMap::getInstance()->getCityset()->getSubDir();
-    d_port = File::getCitysetPicture(cityset, "port.png");
-    d_explosion = File::getTilesetPicture(tileset, "misc/explosion.png");
-    d_signpost = File::getCitysetPicture(cityset, "signpost.png");
+    d_port = GraphicsLoader::getCitysetPicture(cityset, "port.png");
+    d_explosion = GraphicsLoader::getTilesetPicture(tileset, "misc/explosion.png");
+    d_signpost = GraphicsLoader::getCitysetPicture(cityset, "signpost.png");
 }
 
 GraphicsCache::~GraphicsCache()
@@ -246,13 +259,13 @@ GraphicsCache::~GraphicsCache()
         SDL_FreeSurface(d_flagmask[i]);
     }
 
-    for (int i = 0; i < SELECTOR_FRAMES; i++)
+    for (int i = 0; i < d_selector.size(); i++)
     {
         SDL_FreeSurface(d_selector[i]);
         SDL_FreeSurface(d_selectormask[i]);
     }
 
-    for (int i = 0; i < SMALL_SELECTOR_FRAMES; i++)
+    for (int i = 0; i < d_smallselector.size(); i++)
     {
         SDL_FreeSurface(d_smallselector[i]);
         SDL_FreeSurface(d_smallselectormask[i]);
@@ -375,7 +388,7 @@ SDL_Surface* GraphicsCache::getShipPic(const Player* p)
     ShipCacheItem* myitem;
     for (it = d_shiplist.begin(); it != d_shiplist.end(); it++)
     {
-        if ((*it)->player == p)
+        if ((*it)->player_id == p->getId())
         {
             myitem = (*it);
             
@@ -400,7 +413,7 @@ SDL_Surface* GraphicsCache::getPlantedStandardPic(const Player* p)
     PlantedStandardCacheItem* myitem;
     for (it = d_plantedstandardlist.begin(); it != d_plantedstandardlist.end(); it++)
     {
-        if ((*it)->player == p)
+        if ((*it)->player_id == p->getId())
         {
             myitem = (*it);
             
@@ -419,6 +432,12 @@ SDL_Surface* GraphicsCache::getPlantedStandardPic(const Player* p)
     return myitem->surface;
 }
 
+SDL_Surface* GraphicsCache::getArmyPic(Army *a)
+{
+  return getArmyPic(a->getOwner()->getArmyset(), a->getTypeId(), 
+		    a->getOwner(), NULL);
+}
+
 SDL_Surface* GraphicsCache::getArmyPic(Uint32 armyset, Uint32 army, const Player* p,
                                        const bool *medals)
 {
@@ -441,7 +460,7 @@ SDL_Surface* GraphicsCache::getArmyPic(Uint32 armyset, Uint32 army, const Player
     for (it =d_armylist.begin(); it != d_armylist.end(); it++)
     {
         if (((*it)->armyset == armyset) && ((*it)->index == army)
-            && ((*it)->player ==p)
+            && ((*it)->player_id == p->getId())
             && ((*it)->medals[0]==my_medals[0])
             && ((*it)->medals[1]==my_medals[1])
             && ((*it)->medals[2]==my_medals[2]))
@@ -732,7 +751,7 @@ SDL_Surface* GraphicsCache::getFlagPic(const Stack* s)
     for (it = d_flaglist.begin(); it != d_flaglist.end(); it++)
     {
         myitem = *it;
-        if (myitem->size == s->size() - 1 && myitem->player == s->getOwner())
+        if (myitem->size == s->size() - 1 && myitem->player_id == s->getOwner()->getId())
         {
             // put the item in last place (last touched)
             d_flaglist.erase(it);
@@ -765,7 +784,7 @@ SDL_Surface* GraphicsCache::getSelectorPic(Uint32 type, Uint32 frame,
     for (it = d_selectorlist.begin(); it != d_selectorlist.end(); it++)
     {
         myitem = *it;
-        if ((myitem->type == type) && (myitem->player == p) 
+        if ((myitem->type == type) && (myitem->player_id == p->getId()) 
 	    && myitem->frame == frame)
         {
             // put the item in last place (last touched)
@@ -852,7 +871,7 @@ SDL_Surface* GraphicsCache::applyMask(SDL_Surface* image, SDL_Surface* mask, SDL
 SDL_Surface* GraphicsCache::applyMask(SDL_Surface* image, SDL_Surface* mask, const Player* p)
 {
   applyMask(image, mask, p->getMaskColor(),
-	    Playerlist::getInstance()->getNeutral() == p);
+	    Playerlist::getInstance()->getNeutral()->getId() == p->getId());
     if (!mask || mask->w != image->w || mask->h != image->h)
     {
         // we are expected to produce some output and a missing/wrong mask is no
@@ -1000,14 +1019,14 @@ ArmyCacheItem* GraphicsCache::addArmyPic(Uint32 armyset, Uint32 army,
     ArmyCacheItem* myitem = new ArmyCacheItem();
   myitem->armyset = armyset;
   myitem->index = army;
-  myitem->player = p;
+  myitem->player_id = p->getId();
   myitem->medals[0] = medalsbonus[0];
   myitem->medals[1] = medalsbonus[1];
   myitem->medals[2] = medalsbonus[2];
 
   //Now the most important part: load the army picture
   //First, copy the army picture and change it to the display format
-  const Army* basearmy = Armysetlist::getInstance()->getArmy(armyset, army);
+  const ArmyProto * basearmy = Armysetlist::getInstance()->getArmy(armyset, army);
 
   // copy the pixmap including player colors
   myitem->surface = applyMask(basearmy->getPixmap(), basearmy->getMask(), p);
@@ -1095,7 +1114,7 @@ ShipCacheItem* GraphicsCache::addShipPic(const Player* p)
   debug("ADD ship pic: " <<p->getName())
 
   ShipCacheItem* myitem = new ShipCacheItem();
-  myitem->player = p;
+  myitem->player_id = p->getId();
 
   //Now the most important part: load the ship picture
   //First, copy the ship picture and change it to the display format
@@ -1126,7 +1145,7 @@ PlantedStandardCacheItem* GraphicsCache::addPlantedStandardPic(const Player* p)
   debug("ADD planted standard pic: " <<p->getName())
 
     PlantedStandardCacheItem* myitem = new PlantedStandardCacheItem();
-  myitem->player = p;
+  myitem->player_id = p->getId();
 
   //Now the most important part: load the planted standard picture
   //First, copy the picture and change it to the display format
@@ -1314,7 +1333,7 @@ CityCacheItem* GraphicsCache::addCityPic(int type, const Player* p)
 {
   //now create the cache item and add the size
   CityCacheItem* myitem = new CityCacheItem();
-  myitem->player = p;
+  myitem->player_id = p->getId();
   myitem->type = type;
   myitem->surface = d_citypic[p->getId()];
 
@@ -1334,7 +1353,7 @@ TowerCacheItem* GraphicsCache::addTowerPic(const Player* p)
 {
   //now create the cache item and add the size
   TowerCacheItem* myitem = new TowerCacheItem();
-  myitem->player = p;
+  myitem->player_id = p->getId();
   myitem->surface = d_towerpic[p->getId()];
 
   d_towerlist.push_back(myitem);
@@ -1360,7 +1379,7 @@ FlagCacheItem* GraphicsCache::addFlagPic(int size, const Player* p)
 
   //now create the cache item and add the size
   FlagCacheItem* myitem = new FlagCacheItem();
-  myitem->player = p;
+  myitem->player_id = p->getId();
   myitem->size = size;
   myitem->surface = mysurf;
 
@@ -1391,7 +1410,7 @@ SelectorCacheItem* GraphicsCache::addSelectorPic(Uint32 type, Uint32 frame, cons
 
   //now create the cache item and add the size
   SelectorCacheItem* myitem = new SelectorCacheItem();
-  myitem->player = p;
+  myitem->player_id = p->getId();
   myitem->type = type;
   myitem->frame = frame;
   myitem->surface = mysurf;
@@ -1808,7 +1827,7 @@ void GraphicsCache::loadTemplePics()
   int ts = GameMap::getInstance()->getCityset()->getTileSize();
 
   // load the temple pictures
-  SDL_Surface* templepics = File::getCitysetPicture(cityset, "temples.png");
+  SDL_Surface* templepics = GraphicsLoader::getCitysetPicture(cityset, "temples.png");
 
   // copy alpha values, don't use them
   SDL_SetAlpha(templepics, 0, 0);
@@ -1844,7 +1863,7 @@ void GraphicsCache::loadRuinPics()
   int ts = GameMap::getInstance()->getCityset()->getTileSize();
 
   // load the ruin pictures
-  SDL_Surface* ruinpics = File::getCitysetPicture(cityset, "ruin.png");
+  SDL_Surface* ruinpics = GraphicsLoader::getCitysetPicture(cityset, "ruin.png");
 
   // copy alpha values, don't use them
   SDL_SetAlpha(ruinpics, 0, 0);
@@ -1876,7 +1895,7 @@ void GraphicsCache::loadRuinPics()
 void GraphicsCache::loadDiplomacyPics()
 {
   // load the diplomacy pictures
-  SDL_Surface* diplomacypics = File::getMiscPicture("diplomacy-small.png");
+  SDL_Surface* diplomacypics = GraphicsLoader::getMiscPicture("diplomacy-small.png");
   Uint32 ts = 30;
 
   // copy alpha values, don't use them
@@ -1905,7 +1924,7 @@ void GraphicsCache::loadDiplomacyPics()
 
   SDL_FreeSurface(diplomacypics);
   ts = 50;
-  diplomacypics = File::getMiscPicture("diplomacy-large.png");
+  diplomacypics = GraphicsLoader::getMiscPicture("diplomacy-large.png");
   // copy alpha values, don't use them
   SDL_SetAlpha(diplomacypics, 0, 0);
 
@@ -1940,7 +1959,7 @@ void GraphicsCache::loadRoadPics()
   int ts = GameMap::getInstance()->getTileset()->getTileSize();
 
   // load the road pictures
-  SDL_Surface* roadpics = File::getTilesetPicture(tileset, "misc/roads.png");
+  SDL_Surface* roadpics = GraphicsLoader::getTilesetPicture(tileset, "misc/roads.png");
 
   // copy alpha values, don't use them
   SDL_SetAlpha(roadpics, 0, 0);
@@ -1975,7 +1994,7 @@ void GraphicsCache::loadFogPics()
   std::string tileset = GameMap::getInstance()->getTileset()->getSubDir();
 
   // load the fog pictures
-  SDL_Surface* fogpics = File::getTilesetPicture(tileset, "misc/fog.png");
+  SDL_Surface* fogpics = GraphicsLoader::getTilesetPicture(tileset, "misc/fog.png");
 
   // copy alpha values, don't use them
   SDL_SetAlpha(fogpics, 0, 0);
@@ -2010,7 +2029,7 @@ void GraphicsCache::loadBridgePics()
   int ts = GameMap::getInstance()->getTileset()->getTileSize();
 
   // load the bridge pictures
-  SDL_Surface* bridgepics = File::getTilesetPicture(tileset, "misc/bridges.png");
+  SDL_Surface* bridgepics = GraphicsLoader::getTilesetPicture(tileset, "misc/bridges.png");
 
   // copy alpha values, don't use them
   SDL_SetAlpha(bridgepics, 0, 0);
@@ -2043,7 +2062,7 @@ void GraphicsCache::loadCursorPics()
   int ts = 16;
 
   // load the cursor pictures
-  SDL_Surface* cursorpics = File::getMiscPicture ("cursors.png");
+  SDL_Surface* cursorpics = GraphicsLoader::getMiscPicture ("cursors.png");
 
   SDL_PixelFormat* fmt = cursorpics->format;
 
@@ -2076,12 +2095,12 @@ void GraphicsCache::loadCityPics()
 {
   SDL_Surface *tmp;
   SDL_PixelFormat *fmt;
-  // GameMap has the actual tileset stored
-  std::string tileset = GameMap::getInstance()->getTileset()->getSubDir();
+  // GameMap has the actual cityset stored
+  std::string cityset = GameMap::getInstance()->getCityset()->getSubDir();
   int size = GameMap::getInstance()->getTileset()->getTileSize() * 2;
 
   // load the image for the razed city
-  SDL_Surface* razedpics = File::getCitysetPicture(tileset, "castle_razed.png");
+  SDL_Surface* razedpics = GraphicsLoader::getCitysetPicture(cityset, "castle_razed.png");
   // copy alpha values, don't use them
   SDL_SetAlpha(razedpics, 0, 0);
 
@@ -2107,7 +2126,7 @@ void GraphicsCache::loadCityPics()
   SDL_FreeSurface(razedpics);
 
   // load the city pictures
-  SDL_Surface* citypics = File::getCitysetPicture(tileset, "castles.png");
+  SDL_Surface* citypics = GraphicsLoader::getCitysetPicture(cityset, "castles.png");
 
   // copy alpha values, don't use them
   SDL_SetAlpha(citypics, 0, 0);
@@ -2152,7 +2171,7 @@ void GraphicsCache::loadTowerPics()
   int size = GameMap::getInstance()->getCityset()->getTileSize();
 
   // load the image for the towers
-  SDL_Surface* towerpics = File::getCitysetPicture(cityset, "towers.png");
+  SDL_Surface* towerpics = GraphicsLoader::getCitysetPicture(cityset, "towers.png");
   // copy alpha values, don't use them
   SDL_SetAlpha(towerpics, 0, 0);
 
@@ -2179,20 +2198,33 @@ void GraphicsCache::loadTowerPics()
 
 }
 
-void GraphicsCache::loadSelectors()
+bool GraphicsCache::loadSelectorImages(std::string tileset, std::string filename, Uint32 size, std::vector<SDL_Surface*> &images, std::vector<SDL_Surface *> &masks)
 {
-  //load the big selector pictures
-  int i;
-  std::string tileset = GameMap::getInstance()->getTileset()->getSubDir();
-
+  int num_frames;
   // to build flags, we need these three images as basic blocks
-  SDL_Surface* selpics = File::getTilesetPicture(tileset, "misc/selector.png");
+  SDL_Surface* selpics = GraphicsLoader::getTilesetPicture(tileset, filename);
+
+  if (!selpics)
+    return false;
+
+  num_frames = selpics->w / size;
+  if (selpics->w % size != 0)
+    {
+      SDL_FreeSurface (selpics);
+      return false;
+    }
+
+  if (selpics->h != size * 2)
+    {
+      SDL_FreeSurface (selpics);
+      return false;
+    }
+
   SDL_PixelFormat* fmt = selpics->format;
-  int size = GameMap::getInstance()->getTileset()->getTileSize();
 
   // copy alpha values, don't use them
   SDL_SetAlpha(selpics, 0, 0);
-  for (i = 0; i < SELECTOR_FRAMES; i++)
+  for (int i = 0; i < num_frames; i++)
     {
       SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 
 					      fmt->BitsPerPixel, fmt->Rmask, 
@@ -2203,49 +2235,56 @@ void GraphicsCache::loadSelectors()
       selrect.y = 0;
       selrect.w = selrect.h = size;
       SDL_BlitSurface(selpics, &selrect, tmp, 0);
-      d_selector[i] = SDL_DisplayFormatAlpha(tmp);
+      images.push_back (SDL_DisplayFormatAlpha(tmp));
       SDL_FreeSurface(tmp);
 
-      d_selectormask[i] = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 32,
+      SDL_Surface *mask = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 32,
 					       0xFF000000, 0xFF0000, 0xFF00, 
 					       0xFF);
       selrect.y = size;
-      SDL_BlitSurface(selpics, &selrect, d_selectormask[i], 0);
+      SDL_BlitSurface(selpics, &selrect, mask, 0);
+      masks.push_back(mask);
 
     }
   SDL_FreeSurface(selpics);
+  return true;
+}
 
-  //load the small selector pictures
-  selpics = File::getTilesetPicture(tileset, "misc/small_selector.png");
-  fmt = selpics->format;
-  // copy alpha values, don't use them
-  SDL_SetAlpha(selpics, 0, 0);
-  for (i = 0; i < SMALL_SELECTOR_FRAMES; i++)
+void GraphicsCache::loadSelectors()
+{
+  int i;
+  std::string tileset = GameMap::getInstance()->getTileset()->getSubDir();
+  std::string small = GameMap::getInstance()->getTileset()->getSmallSelectorFilename();
+  std::string large = GameMap::getInstance()->getTileset()->getLargeSelectorFilename();
+
+  int size = GameMap::getInstance()->getTileset()->getTileSize();
+  std::vector<SDL_Surface*> images;
+  std::vector<SDL_Surface*> masks;
+  bool success = loadSelectorImages(tileset, large, size, images, masks);
+  if (!success)
     {
-      SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 
-					      fmt->BitsPerPixel, fmt->Rmask, 
-					      fmt->Gmask, fmt->Bmask, 
-					      fmt->Amask);
-      SDL_Rect selrect;
-      selrect.x = i*size;
-      selrect.y = 0;
-      selrect.w = selrect.h = size;
-      SDL_BlitSurface(selpics, &selrect, tmp, 0);
+      fprintf (stderr,"Selector file %s is malformed\n", large.c_str());
+      exit(1);
+    }
+  for (int i = 0; i < images.size(); i++)
+    {
+      d_selector.push_back(images[i]);
+      d_selectormask.push_back(masks[i]);
+    }
 
-      // convert the surface to screen resolution
-      d_smallselector[i] = SDL_DisplayFormatAlpha(tmp);
-
-      // free the temporary surface
-      SDL_FreeSurface(tmp);
-
-      d_smallselectormask[i]=  SDL_CreateRGBSurface(SDL_SWSURFACE, size, size,
-						    32, 0xFF000000, 0xFF0000,
-						    0xFF00, 0xFF);
-      selrect.y = size;
-      SDL_BlitSurface(selpics, &selrect, d_smallselectormask[i], 0);
-
+  images.clear();
+  masks.clear();
+  success = loadSelectorImages(tileset, small, size, images, masks);
+  if (!success)
+    {
+      fprintf (stderr,"Selector file %s is malformed\n", small.c_str());
+      exit(1);
+    }
+  for (int i = 0; i < images.size(); i++)
+    {
+      d_smallselector.push_back(images[i]);
+      d_smallselectormask.push_back(masks[i]);
     }
-  SDL_FreeSurface(selpics);
 }
 
 void GraphicsCache::loadProdShields()
@@ -2253,7 +2292,7 @@ void GraphicsCache::loadProdShields()
   //load the production shieldset
   unsigned int i;
   SDL_Rect prodshieldrect;
-  SDL_Surface* prodshieldpics = File::getMiscPicture("prodshieldset.png");
+  SDL_Surface* prodshieldpics = GraphicsLoader::getMiscPicture("prodshieldset.png");
   // copy alpha values, don't use them
   SDL_SetAlpha(prodshieldpics, 0, 0);
   SDL_PixelFormat* fmt = prodshieldpics->format;
@@ -2282,7 +2321,7 @@ void GraphicsCache::loadMoveBonusPics()
   //load the movement bonus icons
   unsigned int i;
   SDL_Rect movebonusrect;
-  SDL_Surface* movebonuspics = File::getMiscPicture("movebonus.png");
+  SDL_Surface* movebonuspics = GraphicsLoader::getMiscPicture("movebonus.png");
   // copy alpha values, don't use them
   SDL_SetAlpha(movebonuspics, 0, 0);
   SDL_PixelFormat* fmt = movebonuspics->format;
@@ -2312,7 +2351,7 @@ void GraphicsCache::loadFlags()
   std::string tileset = GameMap::getInstance()->getTileset()->getSubDir();
 
   // to build flags, we need these three images as basic blocks
-  SDL_Surface* flag = File::getTilesetPicture(tileset, "misc/flags.png");
+  SDL_Surface* flag = GraphicsLoader::getTilesetPicture(tileset, "misc/flags.png");
   SDL_PixelFormat* fmt = flag->format;
   int tilesize = GameMap::getInstance()->getTileset()->getTileSize();
 
@@ -2350,3 +2389,4 @@ void GraphicsCache::loadFlags()
   // free the temporary surfaces
   SDL_FreeSurface(flag);
 }
+
