@@ -748,8 +748,8 @@ bool MapGenerator::makeRoad(int src_x, int src_y, int dest_x, int dest_y)
 
   Armysetlist *al = Armysetlist::getInstance();
   Uint32 armyset = al->getArmysetId("Default");
-  const Army* basearmy = Armysetlist::getInstance()->getArmy(armyset, 1);
-  Army *a = new Army(*basearmy, NULL);
+  const ArmyProto* basearmy = Armysetlist::getInstance()->getArmy(armyset, 1);
+  Army *a = Army::createNonUniqueArmy(*basearmy);
   s.push_back(a);
   // try to get there with a scout
   Uint32 moves = p->calculate(&s, dest, false);
@@ -802,8 +802,8 @@ bool MapGenerator::isAccessible (int src_x, int src_y, int dest_x, int dest_y)
 
   Armysetlist *al = Armysetlist::getInstance();
   Uint32 armyset = al->getArmysetId("Default");
-  const Army* basearmy = Armysetlist::getInstance()->getArmy(armyset, 1);
-  Army *a = new Army(*basearmy, NULL);
+  const ArmyProto* basearmy = Armysetlist::getInstance()->getArmy(armyset, 1);
+  Army *a = Army::createNonUniqueArmy(*basearmy);
   s.push_back(a);
   // try to get there with a scout
   if (p->calculate(&s, dest, true) == 0)
@@ -830,8 +830,8 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
 
   Armysetlist *al = Armysetlist::getInstance();
   Uint32 armyset = al->getArmysetId("Default");
-  const Army* basearmy = Armysetlist::getInstance()->getArmy(armyset, 16);
-  Army *a = new Army(*basearmy, NULL);
+  const ArmyProto* basearmy = Armysetlist::getInstance()->getArmy(armyset, 16);
+  Army *a = Army::createNonUniqueArmy(*basearmy);
   s.push_back(a);
   // try to get there with a giant bat
   Uint32 moves = p->calculate(&s, dest, false);
