@@ -40,6 +40,8 @@
 #include "armysetlist.h"
 #include "playerlist.h"
 #include "player.h"
+#include "armyprodbase.h"
+#include "heroproto.h"
 
 using namespace std;
 
@@ -58,9 +60,9 @@ Action::Action(const Action &action)
 }
 Action::Action(XML_Helper *helper)
 {
-  Uint32 t;
-  helper->getData(t, "type");
-  d_type = Action::Type(t);
+  std::string type_str;
+  helper->getData(type_str, "type");
+  d_type = actionTypeFromString(type_str);
 }
 
 Action::~Action()
@@ -82,7 +84,8 @@ bool Action::saveContents(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->saveData("type", d_type);
+    std::string type_str = actionTypeToString(Action::Type(d_type));
+    retval &= helper->saveData("type", type_str);
     retval &= doSave(helper);
 
     return retval;
@@ -90,82 +93,86 @@ bool Action::saveContents(XML_Helper* helper) const
 
 Action* Action::handle_load(XML_Helper* helper)
 {
-    Uint32 t;
-    helper->getData(t, "type");
+  std::string type_str;
 
-    switch (t)
+  helper->getData(type_str, "type");
+  Action::Type t = actionTypeFromString(type_str);
+
+  switch (t)
     {
-        case STACK_MOVE:
-            return (new Action_Move(helper));
-        case STACK_SPLIT:
-            return (new Action_Split(helper));
-        case STACK_FIGHT:
-            return (new Action_Fight(helper));
-        case STACK_JOIN:
-            return(new Action_Join(helper));
-        case RUIN_SEARCH:
-            return (new Action_Ruin(helper));
-        case TEMPLE_SEARCH:
-            return (new Action_Temple(helper));
-        case CITY_OCCUPY:
-            return (new Action_Occupy(helper));
-        case CITY_PILLAGE:
-            return (new Action_Pillage(helper));
-        case CITY_SACK:
-            return (new Action_Sack(helper));
-        case CITY_RAZE:
-            return (new Action_Raze(helper));
-        case CITY_UPGRADE:
-            return (new Action_Upgrade(helper));
-        case CITY_BUY:
-            return (new Action_Buy(helper));
-        case CITY_PROD:
-            return (new Action_Production(helper));
-        case REWARD: 
-            return (new Action_Reward(helper));
-        case QUEST:
-            return (new Action_Quest(helper));
-        case HERO_EQUIP:
-            return (new Action_Equip(helper));
-        case UNIT_ADVANCE:
-            return (new Action_Level(helper));
-        case STACK_DISBAND:
-            return (new Action_Disband(helper));
-        case MODIFY_SIGNPOST:
-            return (new Action_ModifySignpost(helper));
-        case CITY_RENAME:
-            return (new Action_RenameCity(helper));
-        case CITY_VECTOR:
-            return (new Action_Vector(helper));
-        case FIGHT_ORDER:
-            return (new Action_FightOrder(helper));
-        case RESIGN:
-            return (new Action_Resign(helper));
-        case ITEM_PLANT:
-            return (new Action_Plant(helper));
-        case PRODUCE_UNIT:
-            return (new Action_Produce(helper));
-        case PRODUCE_VECTORED_UNIT:
-            return (new Action_ProduceVectored(helper));
-        case DIPLOMATIC_STATE:
-            return (new Action_DiplomacyState(helper));
-        case DIPLOMATIC_PROPOSAL:
-            return (new Action_DiplomacyProposal(helper));
-	case DIPLOMATIC_SCORE:
-            return (new Action_DiplomacyScore(helper));
-        case END_TURN:
-            return (new Action_EndTurn(helper));
-        case CITY_CONQUER:
-            return (new Action_ConquerCity(helper));
-        case RECRUIT_HERO:
-            return (new Action_RecruitHero(helper));
-        case PLAYER_RENAME:
-            return (new Action_RenamePlayer(helper));
-        case CITY_DESTITUTE:
-            return (new Action_CityTooPoorToProduce(helper));
+      case STACK_MOVE:
+          return (new Action_Move(helper));
+      case STACK_SPLIT:
+          return (new Action_Split(helper));
+      case STACK_FIGHT:
+          return (new Action_Fight(helper));
+      case STACK_JOIN:
+          return(new Action_Join(helper));
+      case RUIN_SEARCH:
+          return (new Action_Ruin(helper));
+      case TEMPLE_SEARCH:
+          return (new Action_Temple(helper));
+      case CITY_OCCUPY:
+          return (new Action_Occupy(helper));
+      case CITY_PILLAGE:
+          return (new Action_Pillage(helper));
+      case CITY_SACK:
+          return (new Action_Sack(helper));
+      case CITY_RAZE:
+          return (new Action_Raze(helper));
+      case CITY_UPGRADE:
+          return (new Action_Upgrade(helper));
+      case CITY_BUY:
+          return (new Action_Buy(helper));
+      case CITY_PROD:
+          return (new Action_Production(helper));
+      case REWARD: 
+          return (new Action_Reward(helper));
+      case QUEST:
+          return (new Action_Quest(helper));
+      case HERO_EQUIP:
+          return (new Action_Equip(helper));
+      case UNIT_ADVANCE:
+          return (new Action_Level(helper));
+      case STACK_DISBAND:
+          return (new Action_Disband(helper));
+      case MODIFY_SIGNPOST:
+          return (new Action_ModifySignpost(helper));
+      case CITY_RENAME:
+          return (new Action_RenameCity(helper));
+      case CITY_VECTOR:
+          return (new Action_Vector(helper));
+      case FIGHT_ORDER:
+          return (new Action_FightOrder(helper));
+      case RESIGN:
+          return (new Action_Resign(helper));
+      case ITEM_PLANT:
+          return (new Action_Plant(helper));
+      case PRODUCE_UNIT:
+          return (new Action_Produce(helper));
+      case PRODUCE_VECTORED_UNIT:
+          return (new Action_ProduceVectored(helper));
+      case DIPLOMATIC_STATE:
+          return (new Action_DiplomacyState(helper));
+      case DIPLOMATIC_PROPOSAL:
+          return (new Action_DiplomacyProposal(helper));
+      case DIPLOMATIC_SCORE:
+          return (new Action_DiplomacyScore(helper));
+      case END_TURN:
+          return (new Action_EndTurn(helper));
+      case CITY_CONQUER:
+          return (new Action_ConquerCity(helper));
+      case RECRUIT_HERO:
+          return (new Action_RecruitHero(helper));
+      case PLAYER_RENAME:
+          return (new Action_RenamePlayer(helper));
+      case CITY_DESTITUTE:
+          return (new Action_CityTooPoorToProduce(helper));
+      case INIT_TURN:
+          return (new Action_InitTurn(helper));
     }
 
-    return 0;
+  return 0;
 }
 
 
@@ -259,6 +266,10 @@ Action* Action::copy(const Action* a)
             return 
               (new Action_CityTooPoorToProduce
                 (*dynamic_cast<const Action_CityTooPoorToProduce*>(a)));
+        case INIT_TURN:
+            return 
+              (new Action_InitTurn
+                (*dynamic_cast<const Action_InitTurn*>(a)));
     }
 
     return 0;
@@ -463,7 +474,7 @@ std::string Action_Fight::dump() const
     for (uit = d_attackers.begin(); uit != d_attackers.end(); uit++)
         s << (*uit) <<" ";
 
-    s << "\nDefending stacks: ";
+    s << "\n Defending stacks: ";
     for (uit = d_defenders.begin(); uit != d_defenders.end(); uit++)
         s << (*uit) <<" ";
 
@@ -677,7 +688,7 @@ std::string Action_Temple::dump() const
 {
     std::stringstream s;
 
-    s <<"Stack " <<d_stack <<"visited temple " <<d_temple <<".\n";
+    s <<"Stack " <<d_stack <<" visited temple " <<d_temple <<".\n";
 
     return s.str();
 }
@@ -962,8 +973,8 @@ std::string Action_Buy::dump() const
 {
     std::stringstream s;
 
-    s <<"Production " <<d_prod <<"bought in city " <<d_city;
-    s <<"slot: " <<d_slot << "\n";
+    s <<"Production " <<d_prod <<" bought in city " <<d_city;
+    s <<" slot: " <<d_slot << "\n";
 
     return s.str();
 }
@@ -979,11 +990,11 @@ bool Action_Buy::doSave(XML_Helper* helper) const
     return retval;
 }
 
-bool Action_Buy::fillData(City* c, int slot, const Army *prod)
+bool Action_Buy::fillData(City* c, int slot, const ArmyProto *prod)
 {
     d_city = c->getId();
     d_slot = slot;
-    d_prod = prod->getType();
+    d_prod = prod->getTypeId();
 
     return true;
 }
@@ -1052,20 +1063,23 @@ bool Action_Reward::load(std::string tag, XML_Helper *helper)
 {
     if (tag == "reward")
       {
-	Reward *reward = new Reward(helper);
-	if (reward->getType() == Reward::GOLD)
-	  d_reward = new Reward_Gold(helper);
-	else if (reward->getType() == Reward::ALLIES)
-	  d_reward = new Reward_Allies(helper);
-	else if (reward->getType() == Reward::ITEM)
-	  d_reward = new Reward_Item(helper);
-	else if (reward->getType() == Reward::RUIN)
-	  d_reward = new Reward_Ruin(helper);
-	else if (reward->getType() == Reward::MAP)
-	  d_reward = new Reward_Map(helper);
-	  
-	delete reward;
-
+	Uint32 t;
+	std::string type_str;
+	helper->getData(type_str, "type");
+	t = Reward::rewardTypeFromString(type_str);
+	switch (t)
+	  {
+	  case  Reward::GOLD:
+	    d_reward = new Reward_Gold(helper); break;
+	  case  Reward::ALLIES:
+	    d_reward = new Reward_Allies(helper); break;
+	  case Reward::ITEM:
+	    d_reward = new Reward_Item(helper); break;
+	  case Reward::RUIN:
+	    d_reward = new Reward_Ruin(helper); break;
+	  case Reward::MAP:
+	    d_reward = new Reward_Map(helper); break;
+	  }
 	return true;
       }
     return false;
@@ -1074,7 +1088,8 @@ bool Action_Reward::load(std::string tag, XML_Helper *helper)
 Action_Reward::Action_Reward (const Action_Reward &action)
 : Action(action), d_stack(action.d_stack)
 {
-  d_reward = new Reward(*action.d_reward);
+  const Reward *reward = action.d_reward;
+  d_reward = Reward::copy(reward);
 }
 
 Action_Reward::Action_Reward(XML_Helper* helper)
@@ -1110,16 +1125,17 @@ bool Action_Reward::doSave(XML_Helper* helper) const
   bool retval = true;
 
   retval &= helper->saveData("stack", d_stack);
-  if (d_reward->getType() == Reward::GOLD)
-    static_cast<Reward_Gold*>(d_reward)->save(helper);
-  else if (d_reward->getType() == Reward::ALLIES)
-    static_cast<Reward_Allies*>(d_reward)->save(helper);
-  else if (d_reward->getType() == Reward::ITEM)
-    static_cast<Reward_Item*>(d_reward)->save(helper);
-  else if (d_reward->getType() == Reward::RUIN)
-    static_cast<Reward_Ruin*>(d_reward)->save(helper);
-  else if (d_reward->getType() == Reward::MAP)
-    static_cast<Reward_Map*>(d_reward)->save(helper);
+  retval &= d_reward->save(helper);
+  //if (d_reward->getType() == Reward::GOLD)
+    //dynamic_cast<Reward_Gold*>(d_reward)->save(helper);
+  //else if (d_reward->getType() == Reward::ALLIES)
+    //dynamic_cast<Reward_Allies*>(d_reward)->save(helper);
+  //else if (d_reward->getType() == Reward::ITEM)
+    //dynamic_cast<Reward_Item*>(d_reward)->save(helper);
+  //else if (d_reward->getType() == Reward::RUIN)
+    //dynamic_cast<Reward_Ruin*>(d_reward)->save(helper);
+  //else if (d_reward->getType() == Reward::MAP)
+    //dynamic_cast<Reward_Map*>(d_reward)->save(helper);
 
   return retval;
 }
@@ -1213,13 +1229,13 @@ bool Action_Quest::fillData(Quest* q)
 // Action_Equip
 
 Action_Equip::Action_Equip()
-:Action(Action::HERO_EQUIP), d_hero(0), d_item(0)
+:Action(Action::HERO_EQUIP), d_hero(0), d_item(0), d_pos(Vector<int>(-1,-1))
 {
 }
 
 Action_Equip::Action_Equip (const Action_Equip &action)
 : Action(action), d_hero(action.d_hero), d_item(action.d_item),
-    d_slot(action.d_slot)
+    d_slot(action.d_slot), d_pos(action.d_pos)
 {
 }
 
@@ -1229,6 +1245,11 @@ Action_Equip::Action_Equip(XML_Helper* helper)
   helper->getData(d_hero, "hero");
   helper->getData(d_item, "item");
   helper->getData(d_slot, "dest");
+  int i;
+  helper->getData(i, "x");
+  d_pos.x = i;
+  helper->getData(i, "y");
+  d_pos.y = i;
 }
 
 Action_Equip::~Action_Equip()
@@ -1239,7 +1260,8 @@ std::string Action_Equip::dump() const
 {
   std::stringstream ss;
 
-  ss <<"Hero " <<d_hero <<" moved item " <<d_item <<"to slot " <<d_slot;
+  ss <<"Hero " <<d_hero <<" moved item " <<d_item <<" to slot " <<d_slot;
+  ss <<" at tile " << d_pos.x << "," << d_pos.y;
   ss <<std::endl;
 
   return ss.str();
@@ -1252,15 +1274,21 @@ bool Action_Equip::doSave(XML_Helper* helper) const
   retval &= helper->saveData("hero", d_hero);
   retval &= helper->saveData("item", d_item);
   retval &= helper->saveData("dest", d_slot);
+  int i = d_pos.x;
+  retval &= helper->saveData("x", i);
+  i = d_pos.y;
+  retval &= helper->saveData("y", i);
 
   return retval;
 }
 
-bool Action_Equip::fillData(Hero *hero, Item *item, Action_Equip::Slot slot)
+bool Action_Equip::fillData(Hero *hero, Item *item, Action_Equip::Slot slot,
+			    Vector<int> pos)
 {
   d_hero = hero->getId();
   d_item = item->getId();
   d_slot = slot;
+  d_pos = pos;
 
   return true;
 }
@@ -1689,27 +1717,40 @@ Action_Produce::Action_Produce()
 }
 
 Action_Produce::Action_Produce(const Action_Produce &action)
-: Action(action), d_army_type(action.d_army_type), d_city(action.d_city),
-    d_vectored(action.d_vectored)
+: Action(action), d_city(action.d_city), d_vectored(action.d_vectored)
 {
+  d_army = new ArmyProdBase (*action.d_army);
 }
 
 Action_Produce::Action_Produce(XML_Helper* helper)
 :Action(helper)
 {
-  helper->getData(d_army_type, "army_type");
   helper->getData(d_city, "city");
   helper->getData(d_vectored, "vectored");
+  helper->registerTag("armyprodbase", sigc::mem_fun(this, &Action_Produce::load));
+}
+
+bool Action_Produce::load(std::string tag, XML_Helper *helper)
+{
+    if (tag == "armyprodbase")
+      {
+	d_army = new ArmyProdBase(helper);
+
+	return true;
+      }
+    return false;
 }
 
 Action_Produce::~Action_Produce()
 {
+  if (d_army)
+    delete d_army;
 }
 
 std::string Action_Produce::dump() const
 {
   std::stringstream s;
-  s << "armytype " << d_army_type << " shows up at city " << d_city;
+  s << "army of type " << d_army->getTypeId() << " shows up at city " << d_city;
   if (d_vectored)
     s <<" but it is vectored to another city";
   s <<"\n";
@@ -1721,19 +1762,16 @@ bool Action_Produce::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->saveData("army_type", d_army_type);
   retval &= helper->saveData("city", d_city);
   retval &= helper->saveData("vectored", d_vectored);
+  retval &= d_army->save(helper);
 
   return retval;
 }
 
-bool Action_Produce::fillData(const Army *army, City *city, bool vectored)
+bool Action_Produce::fillData(const ArmyProdBase *army, City *city, bool vectored)
 {
-  if (army == NULL)
-    d_army_type = -1;
-  else
-    d_army_type = army->getType();
+  d_army = new ArmyProdBase(*army);
   d_city = city->getId();
   d_vectored = vectored;
   return true;
@@ -1745,33 +1783,58 @@ bool Action_Produce::fillData(const Army *army, City *city, bool vectored)
 Action_ProduceVectored::Action_ProduceVectored()
 :Action(Action::PRODUCE_VECTORED_UNIT)
 {
+  d_army = NULL;
+  d_dest = Vector<int>(-1,-1);
+  d_src = Vector<int>(-1,-1);
 }
 
 Action_ProduceVectored::Action_ProduceVectored(const Action_ProduceVectored &action)
-: Action(action), d_army_type(action.d_army_type), d_dest(action.d_dest)
+: Action(action), d_dest(action.d_dest), d_src(action.d_src)
 {
+  if (action.d_army)
+    d_army = new ArmyProdBase(*action.d_army);
+  else
+    d_army = NULL;
 }
 
 Action_ProduceVectored::Action_ProduceVectored(XML_Helper* helper)
-:Action(helper)
+:Action(helper), d_army(NULL)
 {
-  helper->getData(d_army_type, "army_type");
   int i;
-  helper->getData(i, "x");
+  helper->getData(i, "dest_x");
   d_dest.x = i;
-  helper->getData(i, "y");
+  helper->getData(i, "dest_y");
   d_dest.y = i;
+  helper->getData(i, "src_x");
+  d_src.x = i;
+  helper->getData(i, "src_y");
+  d_src.y = i;
+  helper->registerTag("armyprodbase", sigc::mem_fun(this, &Action_ProduceVectored::load));
+}
+
+bool Action_ProduceVectored::load(std::string tag, XML_Helper *helper)
+{
+    if (tag == "armyprodbase")
+      {
+	d_army = new ArmyProdBase(helper);
+
+	return true;
+      }
+    return false;
 }
 
 Action_ProduceVectored::~Action_ProduceVectored()
 {
+  if (d_army)
+    delete d_army;
 }
 
 std::string Action_ProduceVectored::dump() const
 {
   std::stringstream s;
-  s << "armytype " << d_army_type << " shows up at ";
-  s <<d_dest.x <<"," <<d_dest.y <<")\n";
+  s << "vectored army of type " << d_army->getTypeId() << " shows up at ";
+  s <<d_dest.x <<"," <<d_dest.y << " from " << d_src.x << "," << d_src.y;
+  s <<"\n";
 
   return s.str();
 }
@@ -1780,17 +1843,22 @@ bool Action_ProduceVectored::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->saveData("army_type", d_army_type);
-  retval &= helper->saveData("x", d_dest.x);
-  retval &= helper->saveData("y", d_dest.y);
+  retval &= helper->saveData("dest_x", d_dest.x);
+  retval &= helper->saveData("dest_y", d_dest.y);
+  retval &= helper->saveData("src_x", d_src.x);
+  retval &= helper->saveData("src_y", d_src.y);
+  retval &= d_army->save(helper);
 
   return retval;
 }
 
-bool Action_ProduceVectored::fillData(Uint32 army_type, Vector<int> dest)
+bool Action_ProduceVectored::fillData(ArmyProdBase *army, Vector<int> dest,
+				      Vector<int> src)
 {
-  d_army_type = army_type;
+  if (army)
+    d_army = new ArmyProdBase (*army);
   d_dest = dest;
+  d_src = src;
   return true;
 }
 
@@ -2067,21 +2135,21 @@ Action_RecruitHero::Action_RecruitHero(XML_Helper* helper)
     helper->getData(d_cost, "cost");
     helper->getData(d_allies, "allies");
     helper->getData(d_ally_army_type, "ally_army_type");
-    helper->registerTag("hero", sigc::mem_fun(this, &Action_RecruitHero::load));
+    helper->registerTag("heroproto", sigc::mem_fun(this, &Action_RecruitHero::load));
 }
 
 Action_RecruitHero::Action_RecruitHero(const Action_RecruitHero &action)
 : Action(action), d_city(action.d_city), d_cost(action.d_cost), 
     d_allies(action.d_allies), d_ally_army_type(action.d_ally_army_type)
 {
-  d_hero = new Hero(*action.d_hero);
+  d_hero = new HeroProto(*action.d_hero);
 }
 
 bool Action_RecruitHero::load(std::string tag, XML_Helper *helper)
 {
-    if (tag == "hero")
+    if (tag == "heroproto")
       {
-	d_hero = new Hero(helper);
+	d_hero = new HeroProto(helper);
 
 	return true;
       }
@@ -2096,7 +2164,7 @@ std::string Action_RecruitHero::dump() const
 {
     std::stringstream s;
 
-    s << "Hero " << d_hero->getId() << " recruited with " << d_allies << "allies\n";
+    s << "Hero " << d_hero->getName() << " recruited with " << d_allies << " allies\n";
 
     return s.str();
 }
@@ -2114,14 +2182,14 @@ bool Action_RecruitHero::doSave(XML_Helper* helper) const
     return retval;
 }
 
-bool Action_RecruitHero::fillData(Hero* hero, City *city, int cost, int alliesCount, const Army *ally)
+bool Action_RecruitHero::fillData(HeroProto* hero, City *city, int cost, int alliesCount, const ArmyProto *ally)
 {
     d_hero = hero;
     d_city = city->getId();
     d_cost = cost;
     d_allies = alliesCount;
     if (alliesCount > 0)
-      d_ally_army_type = ally->getType();
+      d_ally_army_type = ally->getTypeId();
     else
       d_ally_army_type = 0;
     return true;
@@ -2218,11 +2286,200 @@ bool Action_CityTooPoorToProduce::doSave(XML_Helper* helper) const
     return retval;
 }
 
-bool Action_CityTooPoorToProduce::fillData(City* c, const Army *army)
+bool Action_CityTooPoorToProduce::fillData(City* c, const ArmyProdBase *army)
 {
     d_city = c->getId();
-    d_army_type = army->getType();
+    d_army_type = army->getTypeId();
 
     return true;
 }
 
+//-----------------------------------------------------------------------------
+//Action_InitTurn
+
+Action_InitTurn::Action_InitTurn()
+:Action(Action::INIT_TURN)
+{
+}
+
+Action_InitTurn::Action_InitTurn(const Action_InitTurn &action)
+: Action(action)
+{
+}
+
+Action_InitTurn::Action_InitTurn(XML_Helper* helper)
+:Action(helper)
+{
+}
+
+Action_InitTurn::~Action_InitTurn()
+{
+}
+
+std::string Action_InitTurn::dump() const
+{
+  return "initializing turn\n";
+}
+
+bool Action_InitTurn::doSave(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  return retval;
+}
+
+std::string Action::actionTypeToString(Action::Type type)
+{
+  switch (type)
+    {
+    case Action::STACK_MOVE:
+      return "Action::STACK_MOVE";
+    case Action::STACK_SPLIT:
+      return "Action::STACK_SPLIT";
+    case Action::STACK_FIGHT:
+      return "Action::STACK_FIGHT";
+    case Action::STACK_JOIN:
+      return "Action::STACK_JOIN";
+    case Action::RUIN_SEARCH:
+      return "Action::RUIN_SEARCH";
+    case Action::TEMPLE_SEARCH:
+      return "Action::TEMPLE_SEARCH";
+    case Action::CITY_OCCUPY:
+      return "Action::CITY_OCCUPY";
+    case Action::CITY_PILLAGE:
+      return "Action::CITY_PILLAGE";
+    case Action::CITY_SACK:
+      return "Action::CITY_SACK";
+    case Action::CITY_RAZE:
+      return "Action::CITY_RAZE";
+    case Action::CITY_UPGRADE:
+      return "Action::CITY_UPGRADE";
+    case Action::CITY_BUY:
+      return "Action::CITY_BUY";
+    case Action::CITY_PROD:
+      return "Action::CITY_PROD";
+    case Action::REWARD: 
+      return "Action::REWARD" ;
+    case Action::QUEST:
+      return "Action::QUEST";
+    case Action::HERO_EQUIP:
+      return "Action::HERO_EQUIP";
+    case Action::UNIT_ADVANCE:
+      return "Action::UNIT_ADVANCE";
+    case Action::STACK_DISBAND:
+      return "Action::STACK_DISBAND";
+    case Action::MODIFY_SIGNPOST:
+      return "Action::MODIFY_SIGNPOST";
+    case Action::CITY_RENAME:
+      return "Action::CITY_RENAME";
+    case Action::CITY_VECTOR:
+      return "Action::CITY_VECTOR";
+    case Action::FIGHT_ORDER:
+      return "Action::FIGHT_ORDER";
+    case Action::RESIGN:
+      return "Action::RESIGN";
+    case Action::ITEM_PLANT:
+      return "Action::ITEM_PLANT";
+    case Action::PRODUCE_UNIT:
+      return "Action::PRODUCE_UNIT";
+    case Action::PRODUCE_VECTORED_UNIT:
+      return "Action::PRODUCE_VECTORED_UNIT";
+    case Action::DIPLOMATIC_STATE:
+      return "Action::DIPLOMATIC_STATE";
+    case Action::DIPLOMATIC_PROPOSAL:
+      return "Action::DIPLOMATIC_PROPOSAL";
+    case Action::DIPLOMATIC_SCORE:
+      return "Action::DIPLOMATIC_SCORE";
+    case Action::END_TURN:
+      return "Action::END_TURN";
+    case Action::CITY_CONQUER:
+      return "Action::CITY_CONQUER";
+    case Action::RECRUIT_HERO:
+      return "Action::RECRUIT_HERO";
+    case Action::PLAYER_RENAME:
+      return "Action::PLAYER_RENAME";
+    case Action::CITY_DESTITUTE:
+      return "Action::CITY_DESTITUTE";
+    case Action::INIT_TURN:
+      return "Action::INIT_TURN";
+    }
+      
+  return "Action::MOVE";
+}
+
+Action::Type Action::actionTypeFromString(std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return Action::Type(atoi(str.c_str()));
+  if (str == "Action::STACK_MOVE")
+    return Action::STACK_MOVE;
+  else if (str == "Action::STACK_SPLIT")
+    return Action::STACK_SPLIT;
+  else if (str == "Action::STACK_FIGHT")
+    return Action::STACK_FIGHT;
+  else if (str == "Action::STACK_JOIN")
+    return Action::STACK_JOIN;
+  else if (str == "Action::RUIN_SEARCH")
+    return Action::RUIN_SEARCH;
+  else if (str == "Action::TEMPLE_SEARCH")
+    return Action::TEMPLE_SEARCH;
+  else if (str == "Action::CITY_OCCUPY")
+    return Action::CITY_OCCUPY;
+  else if (str == "Action::CITY_PILLAGE")
+    return Action::CITY_PILLAGE;
+  else if (str == "Action::CITY_SACK")
+    return Action::CITY_SACK;
+  else if (str == "Action::CITY_RAZE")
+    return Action::CITY_RAZE;
+  else if (str == "Action::CITY_UPGRADE")
+    return Action::CITY_UPGRADE;
+  else if (str == "Action::CITY_BUY")
+    return Action::CITY_BUY;
+  else if (str == "Action::CITY_PROD")
+    return Action::CITY_PROD;
+  else if (str == "Action::REWARD" )
+    return Action::REWARD; 
+  else if (str == "Action::QUEST")
+    return Action::QUEST;
+  else if (str == "Action::HERO_EQUIP")
+    return Action::HERO_EQUIP;
+  else if (str == "Action::UNIT_ADVANCE")
+    return Action::UNIT_ADVANCE;
+  else if (str == "Action::STACK_DISBAND")
+    return Action::STACK_DISBAND;
+  else if (str == "Action::MODIFY_SIGNPOST")
+    return Action::MODIFY_SIGNPOST;
+  else if (str == "Action::CITY_RENAME")
+    return Action::CITY_RENAME;
+  else if (str == "Action::CITY_VECTOR")
+    return Action::CITY_VECTOR;
+  else if (str == "Action::FIGHT_ORDER")
+    return Action::FIGHT_ORDER;
+  else if (str == "Action::RESIGN")
+    return Action::RESIGN;
+  else if (str == "Action::ITEM_PLANT")
+    return Action::ITEM_PLANT;
+  else if (str == "Action::PRODUCE_UNIT")
+    return Action::PRODUCE_UNIT;
+  else if (str == "Action::PRODUCE_VECTORED_UNIT")
+    return Action::PRODUCE_VECTORED_UNIT;
+  else if (str == "Action::DIPLOMATIC_STATE")
+    return Action::DIPLOMATIC_STATE;
+  else if (str == "Action::DIPLOMATIC_PROPOSAL")
+    return Action::DIPLOMATIC_PROPOSAL;
+  else if (str == "Action::DIPLOMATIC_SCORE")
+    return Action::DIPLOMATIC_SCORE;
+  else if (str == "Action::END_TURN")
+    return Action::END_TURN;
+  else if (str == "Action::CITY_CONQUER")
+    return Action::CITY_CONQUER;
+  else if (str == "Action::RECRUIT_HERO")
+    return Action::RECRUIT_HERO;
+  else if (str == "Action::PLAYER_RENAME")
+    return Action::PLAYER_RENAME;
+  else if (str == "Action::CITY_DESTITUTE")
+    return Action::CITY_DESTITUTE;
+  else if (str == "Action::INIT_TURN")
+    return Action::INIT_TURN;
+  return Action::STACK_MOVE;
+}
