@@ -130,10 +130,12 @@ class Action
                 CITY_CONQUER = 31,
                 RECRUIT_HERO = 32,
                 PLAYER_RENAME = 33,
-		CITY_DESTITUTE = 34
+		CITY_DESTITUTE = 34,
+		INIT_TURN = 35,
         };
-                
-        
+	static std::string actionTypeToString(Action::Type type);
+	static Action::Type actionTypeFromString(std::string str);
+
 	//! Default constructor.
         Action(Type type);
 
@@ -680,7 +682,7 @@ class Action_Buy : public Action
 	 * with the new Army unit type being produced.
 	 */
 	//! Populate the action with pertinent data.
-        bool fillData(City* c, int slot, const Army *prod);
+        bool fillData(City* c, int slot, const ArmyProto *prod);
 
 	Uint32 getCityId() const {return d_city;};
 	int getProductionSlot() const {return d_slot;};
@@ -859,16 +861,18 @@ class Action_Equip : public Action
 	 * it's Action_Equip::Slot.
 	 */
 	//! Populate the action with pertinent data.
-        bool fillData(Hero *hero, Item *item, Slot slot);
+        bool fillData(Hero *hero, Item *item, Slot slot, Vector<int> pos);
 
 	Uint32 getHeroId() const {return d_hero;};
 	Uint32 getItemId() const {return d_item;};
 	Uint32 getToBackpackOrToGround() const {return d_slot;};
+	Vector<int> getItemPos() const {return d_pos;};
 
         private:
         Uint32 d_hero;
         Uint32 d_item;
         Uint32 d_slot;
+	Vector<int> d_pos;
 };
 
 //-----------------------------------------------------------------------------
@@ -1170,10 +1174,10 @@ class Action_Produce: public Action
 	 * it is being vectored elsewhere.
 	 */
 	//! Populate the action with pertinent data.
-        bool fillData(const Army *a, City *city, bool vectored);
+        bool fillData(const ArmyProdBase *a, City *city, bool vectored);
 
-	//! Get the Id of the army type of the Army unit that was produced.
-	Uint32 getArmyType() const {return d_army_type;}
+	//! Get the instance of the army that was produced.
+	ArmyProdBase * getArmy() const {return d_army;}
 
 	//! Get the Id of the City that produced the Army unit.
 	Uint32 getCityId() const {return d_city;}
@@ -1181,9 +1185,11 @@ class Action_Produce: public Action
 	//! Get whether or not the Army unit is being vectored elsewhere.
 	bool getVectored() const {return d_vectored;}
     private:
-        Uint32 d_army_type;
+	ArmyProdBase *d_army;
         Uint32 d_city;
         bool d_vectored;
+
+	bool load(std::string tag, XML_Helper *helper);
 };
 
         
@@ -1223,16 +1229,21 @@ class Action_ProduceVectored: public Action
 	 * it has showing up.
 	 */
 	//! Populate the action with pertinent data.
-        bool fillData(Uint32 army_type, Vector <int>dest);
+        bool fillData(ArmyProdBase *army, Vector <int> dest, Vector<int> src);
 
 	//! Get the army type Id that has shown up.
-	Uint32 getArmyType() const {return d_army_type;}
+	ArmyProdBase *getArmy() const {return d_army;};
 
 	//! Get the position on the map where the army showed up.
 	Vector<int> getDestination() const {return d_dest;}
+
+	//! Get the position on the map where the army is coming from.
+	Vector<int> getOrigination() const {return d_src;}
     private:
-        Uint32 d_army_type;
+	ArmyProdBase *d_army;
         Vector<int> d_dest;
+        Vector<int> d_src;
+	bool load(std::string tag, XML_Helper *helper);
 };
 
 //-----------------------------------------------------------------------------
@@ -1446,16 +1457,16 @@ class Action_RecruitHero : public Action
         virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the action.
-        bool fillData(Hero* hero, City *city, int cost, int alliesCount, const Army *ally);
+        bool fillData(HeroProto* hero, City *city, int cost, int alliesCount, const ArmyProto *ally);
     
-	Hero* getHero() const {return d_hero;};
+	HeroProto* getHero() const {return d_hero;};
 	Uint32 getCityId() const {return d_city;};
 	Uint32 getCost() const {return d_cost;};
 	Uint32 getNumAllies() const {return d_allies;};
 	Uint32 getAllyArmyType() const {return d_ally_army_type;};
 
         private:
-        Hero *d_hero;
+        HeroProto *d_hero;
         Uint32 d_city, d_cost, d_allies, d_ally_army_type;
 
         bool load(std::string tag, XML_Helper *helper);
@@ -1511,7 +1522,7 @@ class Action_CityTooPoorToProduce: public Action
         virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the action.
-        bool fillData(City *city, const Army *army);
+        bool fillData(City *city, const ArmyProdBase *army);
     
 	Uint32 getCityId() const {return d_city;}
 	Uint32 getArmyType() const {return d_army_type;}
@@ -1520,4 +1531,26 @@ class Action_CityTooPoorToProduce: public Action
 	Uint32 d_city;
 	Uint32 d_army_type;
 };
+
+//-----------------------------------------------------------------------------
+
+class Action_InitTurn: public Action
+{
+    public:
+	//! Make a new initialize turn action.
+        Action_InitTurn();
+	//! Copy constructor
+	Action_InitTurn(const Action_InitTurn &action);
+	//! Load a new initialize turn action from an opened saved-game file.
+        Action_InitTurn(XML_Helper* helper);
+	//! Destroy a initialize turn action.
+        ~Action_InitTurn();
+
+	//! Return some debug information about this action.
+        std::string dump() const;
+
+	//! Save this action to an opened saved-game file.
+        virtual bool doSave(XML_Helper* helper) const;
+};
+
 #endif //ACTION_H
