@@ -41,6 +41,7 @@
 #include "stack.h"
 #include "GameScenarioOptions.h"
 #include "hero.h"
+#include "vectoredunitlist.h"
 
 using namespace std;
 
@@ -183,7 +184,7 @@ void AI_Fast::invadeCity(City* c)
       debug("Occupying it");
       cityOccupy(c);
     }
-  if (d_maniac)
+  else if (d_maniac)
     {
       debug ("Razing it");
       cityRaze(c);
@@ -276,6 +277,7 @@ bool AI_Fast::computerTurn()
 		   blessed, stack_died);
 		if (stack_died)
 		  return true;
+		s = d_stacklist->getActivestack();
 		if (blessed && stack_moved)
 		  stack_moved = false; //do this so we move it later on
 		else if (stack_moved)
@@ -288,12 +290,13 @@ bool AI_Fast::computerTurn()
 	  {
 	    bool stack_died = false;
 	    bool picked_up = false;
-		
+
 	    stack_moved = AI_maybePickUpItems(s, s->getGroupMoves(), 
 					       s->getGroupMoves() + 7, 
 					       picked_up, stack_died);
 	    if (stack_died)
 	      return true;
+	    s = d_stacklist->getActivestack();
 	    if (picked_up && stack_moved)
 	      stack_moved = false; //do this so we move it later on
 	    else if (stack_moved)
@@ -319,15 +322,6 @@ bool AI_Fast::computerTurn()
             }
         }
         
-        // Maybe the stack vanished upon joining. If so, reiterate;
-        if (!d_stacklist->getActivestack())
-	  {
-	    cerr <<"crapola.  i hit a bad spot.\n";
-	    //remove me if this is never hit
-	    exit(1);
-	  return true;
-	  }
-
         // second step: try to resupply
         if (!d_maniac)
         {
@@ -450,8 +444,8 @@ bool AI_Fast::computerTurn()
 		else
 		  {
 		    cerr << "hit yet another bad spot." << moves << "\n";
-		    printf ("stack is at %d,%d, target is at %d,%d\n",
-			    s->getPos().x, s->getPos().y, target->getPos().x,
+		    printf ("stack %d is at %d,%d, target is at %d,%d\n",
+			    s->getId(), s->getPos().x, s->getPos().y, target->getPos().x,
 			    target->getPos().y);
 		    printf ("this error means the map has an unreachable city on it, which is supposed to be impossible.\n");
 		    printf ("if an enemy city is completely surrouned by other stacks, this error message is possible.\n");
@@ -500,7 +494,7 @@ bool AI_Fast::computerTurn()
 
 	    // now we need to choose. If we found a target, attack it, otherwise
 	    // attack the closest city.
-	    Vector<int> pos;
+	    Vector<int> pos = Vector<int>(-1,-1);
 	    if (target)
 	      {
 		pos = target->getClosestPoint(s->getPos());
@@ -510,37 +504,48 @@ bool AI_Fast::computerTurn()
 	      {
 		Citylist *cl = Citylist::getInstance();
 		City *enemy_city = cl->getNearestForeignCity(s->getPos());
-		pos  = enemy_city->getPos();
-		debug("Maniac, found no targets, attacking city " << enemy_city->getName() << " at (" <<pos.x <<"," <<pos.y <<")")
+		if (enemy_city)
+		  {
+		    pos  = enemy_city->getPos();
+		    debug("Maniac, found no targets, attacking city " << enemy_city->getName() << " at (" <<pos.x <<"," <<pos.y <<")")
+		  }
 	      }
 
 	    if (pos == Vector<int>(-1,-1))
-	      {
-		debug("Maniac, failure to calc point")
-		exit (1);
-		return false;
-	      }
+	      return false;
 
 	    int mp = s->getPath()->calculate(s, pos);
 	    if (mp > 0)
 	      {
-	      printf ("stack %d at %d,%d moving %d with %d moves\n",
-		      s->getId(), s->getPos().x, s->getPos().y,
-		      mp, s->getGroupMoves());
+	      //printf ("stack %d at %d,%d moving %d with %d moves\n",
+		      //s->getId(), s->getPos().x, s->getPos().y,
+		      //mp, s->getGroupMoves());
 		bool moved = stackMove(s);
-		printf("result of move: %d\n", moved);
-	      stack_moved |= moved;
-	      s = d_stacklist->getActivestack();
+		//printf("result of move: %d\n", moved);
+		stack_moved |= moved;
+		s = d_stacklist->getActivestack();
 	      }
 	    else
 	      {
 		printf ("we're going the wrong way (mp is %d)!!\n", mp);
 		printf ("this means we couldn't calculate a path from %d,%d to %d,%d\n", s->getPos().x, s->getPos().y, pos.x, pos.y);
-		sleep (10);
+		//sleep (10);
 		Citylist *cl = Citylist::getInstance();
 		City *friendly_city = cl->getNearestFriendlyCity(s->getPos());
-		s->getPath()->calculate(s, friendly_city->getPos());
-		stack_moved |= stackMove(s);
+		if (friendly_city)
+		  {
+		    mp = s->getPath()->calculate(s, friendly_city->getPos());
+		    if (mp > 0)
+		      stack_moved |= stackMove(s);
+		    else
+		      stack_moved |= false;
+		  }
+		else
+		  {
+		    //we can't find anyplace to move to!
+		    //so we stay put.
+		    stack_moved |= false;
+		  }
 	      }
 
 	    if (!d_stacklist->getActivestack())
