@@ -24,7 +24,7 @@
 #include <sigc++/trackable.h>
 
 #include "xmlhelper.h"
-#include "army.h"
+#include "armyproto.h"
 
 
 //! A collection of Army prototype objects.
@@ -55,7 +55,7 @@
  * The armyset configuration file is a same named XML file inside the Armyset's
  * directory.  E.g. army/${Armyset::d_dir}/${Armyset::d_dir}.xml.
  */
-class Armyset: public std::list<Army *>, public sigc::trackable
+class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 {
     public:
 
@@ -138,21 +138,30 @@ class Armyset: public std::list<Army *>, public sigc::trackable
 	//! Set the subdirectory that the armyset is in.
         void setSubDir(std::string dir) {d_dir = dir;}
 
-	//! Load the graphics associated with the Armyset.
-        void instantiatePixmaps();
-
 	//! Get the image of the stack in a ship (minus the mask).
 	SDL_Surface *getShipPic() const {return d_ship;}
 
+	//! Set the image of the stack in a ship
+	void setShipImage(SDL_Surface *ship) {d_ship = ship;};
+
 	//! Get the mask portion of the image of the stack in a ship.
 	SDL_Surface *getShipMask() const {return d_shipmask;}
 
+	//! Set the mask portion of the image of the stack in a ship.
+	void setShipMask(SDL_Surface *shipmask) {d_shipmask = shipmask;};
+
 	//! Get the image of the planted standard (minus the mask).
 	SDL_Surface *getStandardPic() const {return d_standard;}
 
+	//! Set the image of the planted standard (minus the mask).
+	void setStandardImage(SDL_Surface *s) {d_standard = s;};
+
 	//! Get the mask portion of the image of the planted standard.
 	SDL_Surface *getStandardMask() const {return d_standard_mask;}
 
+	//! Set the mask portion of the image of the planted standard.
+	void setStandardMask(SDL_Surface *s) {d_standard_mask = s;};
+
 	//! Find an army with a type in this armyset.
 	/**
 	 * Scan the Army prototype objects in this Armyset and return it.
@@ -166,14 +175,11 @@ class Armyset: public std::list<Army *>, public sigc::trackable
 	 * @return The Army with the given army type id, or NULL if none
 	 *         could be found.
 	 */
-	Army * lookupArmyByType(Uint32 army_type);
+	ArmyProto * lookupArmyByType(Uint32 army_type);
     private:
 
         //! Callback function for the army tag (see XML_Helper)
-        bool loadArmyTemplate(std::string tag, XML_Helper* helper);
-
-	//! Load the images associated with a particular Army.
-	bool instantiatePixmap(Army *a);
+        bool loadArmyProto(std::string tag, XML_Helper* helper);
 
 	//! Load the image and mask of the stack's ship picture.
 	void loadShipPic();
