@@ -86,7 +86,7 @@ Armysetlist::~Armysetlist()
             delete ((*it).second)[0];
 }
 
-Army* Armysetlist::getArmy(Uint32 id, Uint32 index) const
+ArmyProto* Armysetlist::getArmy(Uint32 id, Uint32 index) const
 {
     // always use ArmyProtoMap::find for searching, else a default entry is 
     // created, which can produce really bad results!!
@@ -103,7 +103,7 @@ Army* Armysetlist::getArmy(Uint32 id, Uint32 index) const
     return ((*it).second)[index];
 }
 
-Army* Armysetlist::getScout(Uint32 id) const
+ArmyProto* Armysetlist::getScout(Uint32 id) const
 {
     // always use ArmyProtoMap::find for searching, else a default entry is 
     // created, which can produce really bad results!!
@@ -187,12 +187,6 @@ bool Armysetlist::loadArmyset(std::string name)
 
   return true;
 }
-        
-void Armysetlist::instantiatePixmaps()
-{
-  for (iterator it = begin(); it != end(); it++)
-    (*it)->instantiatePixmaps();
-}
 	
 SDL_Surface * Armysetlist::getShipPic (Uint32 id)
 {
@@ -244,3 +238,12 @@ SDL_Surface * Armysetlist::getStandardMask (Uint32 id)
   return NULL;
 }
 
+Armyset *Armysetlist::getArmyset(Uint32 id)
+{
+  for (iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->getId() == id)
+	return (*it);
+    }
+  return NULL;
+}
