@@ -27,6 +27,9 @@
 #include "playerlist.h"
 #include <limits.h>
 #include "xmlhelper.h"
+#include "hero.h"
+#include "stack.h"
+#include "armyprodbase.h"
 
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 #define debug(x)
@@ -67,7 +70,7 @@ Citylist::Citylist(XML_Helper* helper)
 {
     // simply ask the helper to inform us when a city tag is opened
     helper->registerTag("city", sigc::mem_fun(this, &Citylist::load));
-    helper->registerTag("army", sigc::mem_fun(this, &Citylist::load));
+    helper->registerTag("armyprodbase", sigc::mem_fun(this, &Citylist::load));
     helper->registerTag("slot", sigc::mem_fun(this, &Citylist::load));
 }
 
@@ -95,6 +98,7 @@ void Citylist::collectTaxes(Player* p)
   for (const_iterator it = begin(); it != end(); it++)
     if ((*it).getOwner() == p && (*it).isBurnt() == false)
       p->addGold((*it).getGold());
+
 }
 
 //calculate the amount of money new armies will cost in the upcoming turn.
@@ -109,7 +113,7 @@ Uint32 Citylist::calculateUpcomingUpkeep(Player *p)
 	    int slot =(*it).getActiveProductionSlot();
 	    if (slot == -1)
 	      continue;
-	    const Army *a = (*it).getProductionBase(slot);
+	    const ArmyProdBase *a = (*it).getProductionBase(slot);
 	    total += a->getUpkeep();
 	  }
       }
@@ -143,7 +147,7 @@ void Citylist::nextTurn(Player* p)
 	    int slot =(*it).getActiveProductionSlot();
 	    if (slot == -1)
 	      continue;
-	    const Army *a = (*it).getProductionBase(slot);
+	    const ArmyProdBase *a = (*it).getProductionBase(slot);
 	    diff -= a->getUpkeep();
     
 	    p->cityTooPoorToProduce(&(*it), slot);
@@ -160,6 +164,7 @@ void Citylist::nextTurn(Player* p)
       }
 
 }
+
 static bool isFogged(void *object)
 {
   return ((City*)object)->isFogged();
@@ -369,14 +374,14 @@ bool Citylist::save(XML_Helper* helper) const
 
 bool Citylist::load(std::string tag, XML_Helper* helper)
 {
-    if (tag == "army")
+    if (tag == "armyprodbase")
       {
 	//add it to the right city
 	//how do i add it to the right slot?
 	Citylist::iterator it = end();
 	it--;
 	City *city = &*it;
-	Army *a = new Army (helper, Army::PRODUCTION_BASE);
+	ArmyProdBase *a = new ArmyProdBase (helper);
 	int slot = city->getMaxNoOfProductionBases() - 1;
 	city->addProductionBase(slot, a);
 	return true;
@@ -416,16 +421,12 @@ void Citylist::stopVectoringTo(City *c)
     {
       if ((*it).isBurnt() == true)
 	continue;
-      if ((*it).getOwner() != c->getOwner())
-	continue;
       if ((*it).getVectoring() == Vector<int>(-1,-1))
 	continue;
       if (c->contains((*it).getVectoring()))
-	{
-	  (*it).setVectoring(Vector<int>(-1,-1));
-	}
-      return;
+	(*it).setVectoring(Vector<int>(-1,-1));
     }
+  return;
 }
 
 Vector<int> Citylist::calculateCenterOfTerritory (Player *p)
