@@ -27,8 +27,9 @@
 
 #include <libintl.h>
 
-#define LORDSAWAR_SAVEGAME_VERSION "0.1.0"
-#define LORDSAWAR_CONFIG_VERSION "0.1.0"
+#define LORDSAWAR_SAVEGAME_VERSION "0.1.1"
+#define LORDSAWAR_CONFIG_VERSION "0.1.1"
+#define LORDSAWAR_RECENTLY_PLAYED_VERSION "0.1.1"
 #define ENABLE_NLS 1 // Very important toactivate localization
 #define _(string) gettext(string) // Macro for the gettext
 #define __(astring) std::string(gettext(astring.c_str()))
@@ -72,8 +73,6 @@ const unsigned int PRODUCTION_SHIELD_TYPES = 7;
 const unsigned int MOVE_BONUS_WIDTH = 32;
 const unsigned int MOVE_BONUS_HEIGHT = 20;
 const unsigned int MOVE_BONUS_TYPES = 6;
-const unsigned int SELECTOR_FRAMES = 6;
-const unsigned int SMALL_SELECTOR_FRAMES = 4;
 
 // from www.boost.org - derivation from this class makes the derived class
 // noncopyable
@@ -93,9 +92,6 @@ const unsigned short LORDSAWAR_PORT = 14998;
 #define EASY_PLAYER_TYPE _("Easy")
 #define HARD_PLAYER_TYPE _("Hard")
 #define NO_PLAYER_TYPE _("Off")
-
-#define PLAYER_MOVING _("moving")
-#define PLAYER_WATCHING _("watching")
-#define PLAYER_NOT_HERE _("not here")
+#define NETWORKED_PLAYER_TYPE _("Network")
 
 #endif // DEFINITIONS_H
