@@ -38,41 +38,43 @@
 
 #include "main-window.h"
 
-#include "../gui/gtksdl.h"
-#include "../gui/image-helpers.h"
-#include "../gui/input-helpers.h"
-#include "../gui/error-utils.h"
+#include "gtksdl.h"
+#include "image-helpers.h"
+#include "input-helpers.h"
+#include "error-utils.h"
 
-#include "../ucompose.hpp"
-#include "../tileset.h"
-#include "../GameMap.h"
-#include "../defs.h"
-#include "../sound.h"
-#include "../File.h"
-#include "../GraphicsCache.h"
-#include "../smallmap.h"
-#include "../GameScenario.h"
-#include "../CreateScenarioRandomize.h"
-#include "../armysetlist.h"
-#include "../Itemlist.h"
-#include "../playerlist.h"
-#include "../shieldsetlist.h"
-#include "../ai_dummy.h"
+#include "ucompose.hpp"
+#include "tileset.h"
+#include "tilesetlist.h"
+#include "GameMap.h"
+#include "defs.h"
+#include "sound.h"
+#include "File.h"
+#include "GraphicsCache.h"
+#include "GraphicsLoader.h"
+#include "smallmap.h"
+#include "GameScenario.h"
+#include "CreateScenarioRandomize.h"
+#include "armysetlist.h"
+#include "Itemlist.h"
+#include "playerlist.h"
+#include "shieldsetlist.h"
+#include "ai_dummy.h"
 
-#include "../stack.h"
-#include "../city.h"
-#include "../ruin.h"
-#include "../signpost.h"
-#include "../temple.h"
-#include "../citylist.h"
-#include "../templelist.h"
-#include "../ruinlist.h"
-#include "../signpostlist.h"
-#include "../roadlist.h"
-#include "../bridgelist.h"
-#include "../portlist.h"
-#include "../MapGenerator.h"
-#include "../counter.h"
+#include "stack.h"
+#include "city.h"
+#include "ruin.h"
+#include "signpost.h"
+#include "temple.h"
+#include "citylist.h"
+#include "templelist.h"
+#include "ruinlist.h"
+#include "signpostlist.h"
+#include "roadlist.h"
+#include "bridgelist.h"
+#include "portlist.h"
+#include "MapGenerator.h"
+#include "counter.h"
 
 #include "glade-helpers.h"
 #include "editorbigmap.h"
@@ -176,6 +178,8 @@ MainWindow::MainWindow()
     
     xml->connect_clicked("fullscreen_menuitem", 
 			 sigc::mem_fun(this, &MainWindow::on_fullscreen_activated));
+    xml->connect_clicked("toggle_tile_graphics_menuitem", 
+			 sigc::mem_fun(this, &MainWindow::on_tile_graphics_toggled));
     xml->get_widget("fullscreen_menuitem", fullscreen_menuitem);
     xml->connect_clicked("smooth_map_menuitem", 
 			 sigc::mem_fun(this, 
@@ -585,6 +589,9 @@ bool MainWindow::on_map_mouse_motion_event(GdkEventMotion *e)
 void MainWindow::on_sdl_surface_changed()
 {
     if (!sdl_inited) {
+	GraphicsLoader::instantiatePixmaps(Armysetlist::getInstance());
+	GraphicsLoader::instantiatePixmaps(Tilesetlist::getInstance());
+	GraphicsLoader::instantiatePixmaps(Shieldsetlist::getInstance());
 	sdl_inited = true;
 	sdl_initialized.emit();
     }
@@ -730,6 +737,12 @@ void MainWindow::on_fullscreen_activated()
     else
 	window->unfullscreen();
 }
+			 
+void MainWindow::on_tile_graphics_toggled()
+{
+  bigmap->toggleViewStylesOrTypes();
+  bigmap->draw();
+}
 
 void MainWindow::remove_tile_style_buttons()
 {
@@ -1146,7 +1159,7 @@ void MainWindow::on_help_about_activated()
   dialog->set_transient_for(*window.get());
 
   dialog->set_version(PACKAGE_VERSION);
-  SDL_Surface *logo = File::getMiscPicture("castle_icon.png");
+  SDL_Surface *logo = GraphicsLoader::getMiscPicture("castle_icon.png");
   dialog->set_logo(to_pixbuf(logo));
   dialog->show_all();
   dialog->run();
