@@ -34,7 +34,7 @@
 #include <gtkmm/radiobutton.h>
 #include <gtkmm/tooltips.h>
 
-#include "../map-tip-position.h"
+#include "map-tip-position.h"
 #include "editorbigmap.h"
 #include "RenamableLocation.h"
 
@@ -44,7 +44,7 @@ class SmallMap;
 class GameScenario;
 class CreateScenarioRandomize;
 
-#include "../UniquelyIdentified.h"
+#include "UniquelyIdentified.h"
 
 //! Scenario editor.  Edits a Scenario.
 class MainWindow: public sigc::trackable
@@ -109,6 +109,7 @@ class MainWindow: public sigc::trackable
     void on_edit_rewards_activated();
 
     void on_fullscreen_activated();
+    void on_tile_graphics_toggled();
 
     void on_random_all_cities_activated();
     void on_random_unnamed_cities_activated();
