@@ -29,8 +29,8 @@
 #include <gtkmm/widget.h>
 #include <gtkmm/scale.h>
 
-#include "../Tile.h"
-#include "../game-parameters.h"
+#include "Tile.h"
+#include "game-parameters.h"
 
 //! Scenario editor.  Edit parameters to make a new map.
 class NewMapDialog
