@@ -41,20 +41,23 @@
 #include "tileset-window.h"
 #include "tileset-info-dialog.h"
 #include "tile-preview-dialog.h"
+#include "tileset-selector-editor-dialog.h"
 
-#include "../gui/gtksdl.h"
-#include "../gui/image-helpers.h"
-#include "../gui/input-helpers.h"
-#include "../gui/error-utils.h"
+#include "gtksdl.h"
+#include "image-helpers.h"
+#include "input-helpers.h"
+#include "error-utils.h"
 
-#include "../defs.h"
-#include "../Configuration.h"
-#include "../tilesetlist.h"
-#include "../Tile.h"
-#include "../File.h"
-#include "../overviewmap.h"
+#include "defs.h"
+#include "Configuration.h"
+#include "tilesetlist.h"
+#include "Tile.h"
+#include "File.h"
+#include "overviewmap.h"
+#include "GraphicsCache.h"
+#include "GraphicsLoader.h"
 
-#include "../ucompose.hpp"
+#include "ucompose.hpp"
 
 #include "glade-helpers.h"
 
@@ -123,6 +126,9 @@ TileSetWindow::TileSetWindow()
     xml->get_widget("edit_tileset_info_menuitem", edit_tileset_info_menuitem);
     edit_tileset_info_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &TileSetWindow::on_edit_tileset_info_activated));
+    xml->get_widget("army_unit_selector_menuitem", army_unit_selector_menuitem);
+    army_unit_selector_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &TileSetWindow::on_army_unit_selector_activated));
     xml->get_widget("preview_tile_menuitem", preview_tile_menuitem);
     preview_tile_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &TileSetWindow::on_preview_tile_activated));
@@ -211,11 +217,13 @@ TileSetWindow::update_tileset_menuitems()
     {
       save_tileset_as_menuitem->set_sensitive(false);
       save_tileset_menuitem->set_sensitive(false);
+      army_unit_selector_menuitem->set_sensitive(false);
     }
   else
     {
       save_tileset_as_menuitem->set_sensitive(true);
       save_tileset_menuitem->set_sensitive(true);
+      army_unit_selector_menuitem->set_sensitive(true);
     }
 }
 
@@ -307,7 +315,7 @@ void TileSetWindow::fill_tilestyleset_info(TileStyleSet *t)
   if (height)
     {
       d_tileset->setTileSize(height);
-      t->instantiatePixmaps(t->getSubDir(), height);
+      GraphicsLoader::instantiatePixmaps(t, d_tileset->getTileSize());
     }
   inhibit_image_change = true;
   image_filechooser_button->set_filename(n);
@@ -509,7 +517,7 @@ void TileSetWindow::on_load_tileset_activated()
       //hackus horribilium
       std::string back = "../../../../../../../../../../../../../../../../";
       d_tileset->setSubDir(back + dir);
-      d_tileset->instantiatePixmaps();
+      GraphicsLoader::instantiatePixmaps(d_tileset);
       for (Tileset::iterator i = d_tileset->begin(); i != d_tileset->end(); ++i)
 	{
 	  Gtk::TreeIter l = tiles_list->append();
@@ -597,7 +605,7 @@ void TileSetWindow::on_help_about_activated()
   dialog->set_transient_for(*window.get());
 
   dialog->set_version(PACKAGE_VERSION);
-  SDL_Surface *logo = File::getMiscPicture("tileset_icon.png");
+  SDL_Surface *logo = GraphicsLoader::getMiscPicture("tileset_icon.png");
   dialog->set_logo(to_pixbuf(logo));
   dialog->show_all();
   dialog->run();
@@ -608,6 +616,7 @@ void TileSetWindow::on_help_about_activated()
 void TileSetWindow::on_sdl_surface_changed()
 {
   if (!sdl_inited) {
+    GraphicsLoader::instantiatePixmaps(Tilesetlist::getInstance());
     sdl_inited = true;
     sdl_initialized.emit();
   }
@@ -794,7 +803,7 @@ void TileSetWindow::fill_colours(Tile *tile)
       c.set_red(sdl.r * 255); c.set_green(sdl.g * 255); c.set_blue(sdl.b * 255);
       tile_smallmap_second_colorbutton->set_color(c);
       break;
-    case Tile::RANDOMIZED: case Tile::TABLECLOTH:
+    case Tile::RANDOMIZED: case Tile::TABLECLOTH: case Tile::DIAGONAL: case Tile::CROSSHATCH:
       tile_smallmap_second_colorbutton->set_sensitive(true);
       tile_smallmap_third_colorbutton->set_sensitive(true);
       sdl = tile->getColor();
@@ -1061,7 +1070,7 @@ void TileSetWindow::on_image_chosen()
 void TileSetWindow::on_refresh_clicked()
 {
   TileStyleSet *set = get_selected_tilestyleset ();
-  set->instantiatePixmaps(set->getSubDir(), d_tileset->getTileSize());
+  GraphicsLoader::instantiatePixmaps(set, d_tileset->getTileSize());
 }
 
 void TileSetWindow::on_preview_tile_activated()
@@ -1073,3 +1082,9 @@ void TileSetWindow::on_preview_tile_activated()
       d.run();
     }
 }
+      
+void TileSetWindow::on_army_unit_selector_activated()
+{
+  TilesetSelectorEditorDialog d(d_tileset);
+  d.run();
+}
