@@ -32,7 +32,9 @@
 #include "sound.h"
 #include "GraphicsCache.h"
 #include "GameScenario.h"
-#include "NextTurn.h"
+#include "NextTurnNetworked.h"
+#include "NextTurnHotseat.h"
+#include "NextTurnPbm.h"
 
 #include "gamebigmap.h"
 #include "smallmap.h"
@@ -40,6 +42,7 @@
 #include "army.h"
 #include "fight.h"
 #include "hero.h"
+#include "heroproto.h"
 #include "stacklist.h"
 #include "citylist.h"
 #include "ruinlist.h"
@@ -61,6 +64,8 @@
 #include "FogMap.h"
 #include "GameMap.h"
 #include "history.h"
+#include "pbm-game-server.h"
+#include "LocationBox.h"
 
 #include "herotemplates.h"
 
@@ -101,8 +106,7 @@ void Game::addPlayer(Player *p)
 	(p->advice_asked.connect
 	 (sigc::mem_fun(advice_asked, &sigc::signal<void, float>::emit)));
     }
-	
-      
+    
   connections[p->getId()].push_back
     (p->schangingStatus.connect 
      (sigc::mem_fun(this, &Game::update_sidebar_stats)));
@@ -125,25 +129,20 @@ void Game::addPlayer(Player *p)
       (ruinfight_finished, &sigc::signal<void, Fight::Result>::emit)));
   connections[p->getId()].push_back
     (p->cityfight_finished.connect (sigc::mem_fun(*this, &Game::on_city_fight_finished))); 
+  if (p->getType() == Player::NETWORKED && p == Playerlist::getActiveplayer())
+    lock_inputs();
+  if (p->getType() == Player::HUMAN && p == Playerlist::getActiveplayer())
+    unlock_inputs();
 }
 
-#define NETWORK_TESTING 1
-
 #include "game-server.h"
 
-GameServer *game_server = 0;
-
-Game::Game(GameScenario* gameScenario)
-    : d_gameScenario(gameScenario) 
+Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
+    : d_gameScenario(gameScenario), d_nextTurn(nextTurn)
 {
     current_game = this;
     input_locked = false;
 
-#if NETWORK_TESTING
-    game_server = new GameServer();
-    game_server->start();
-#endif
-    
     // init the bigmap
     bigmap.reset(new GameBigMap
 		 (GameScenario::s_intense_combat, 
@@ -189,12 +188,10 @@ Game::Game(GameScenario* gameScenario)
 	Player *p = *i;
 	addPlayer(p);
     }
-    pl->splayerDead.connect(sigc::mem_fun(this, &Game::on_player_died));
+    if (gameScenario->getPlayMode() == GameScenario::HOTSEAT)
+      pl->splayerDead.connect(sigc::mem_fun(this, &Game::on_player_died));
     pl->ssurrender.connect(sigc::mem_fun(this, &Game::on_surrender_offered));
 
-    //set up a NextTurn object
-    d_nextTurn = new NextTurn(d_gameScenario->getTurnmode(),
-			      d_gameScenario->s_random_turns);
     d_nextTurn->splayerStart.connect(
 	sigc::mem_fun(this, &Game::init_turn_for_player));
     d_nextTurn->snextRound.connect(
@@ -204,6 +201,7 @@ Game::Game(GameScenario* gameScenario)
     d_nextTurn->supdating.connect(
 	sigc::mem_fun(this, &Game::redraw));
             
+
     center_view_on_city();
     update_control_panel();
 
@@ -239,6 +237,9 @@ void Game::end_turn()
     lock_inputs();
 
     d_nextTurn->endTurn();
+    if (d_gameScenario->getPlayMode() == GameScenario::NETWORKED &&
+	Playerlist::getActiveplayer()->getType() == Player::NETWORKED)
+      remote_next_player_turn.emit();
 }
 
 void Game::update_stack_info()
@@ -949,6 +950,11 @@ SmallMap &Game::get_smallmap()
 void Game::startGame()
 {
   debug ("start_game()");
+      
+  center_view_on_city();
+  update_sidebar_stats();
+  update_control_panel();
+  update_stack_info();
   lock_inputs();
 
   d_nextTurn->start();
@@ -958,8 +964,12 @@ void Game::startGame()
 
 void Game::loadGame()
 {
+  //if pbm and human
+  //if pbm and not human, then bail
+
   Player *player = Playerlist::getActiveplayer();
-  if (player->getType() == Player::HUMAN)
+
+  if (player->getType() == Player::HUMAN && d_gameScenario->getPlayMode() == GameScenario::HOTSEAT)
     {
       //human players want access to the controls and an info box
       unlock_inputs();
@@ -969,10 +979,30 @@ void Game::loadGame()
       update_control_panel();
       update_stack_info();
       game_loaded.emit(player);
+      if (player->getType() == Player::HUMAN)
+	d_nextTurn->setContinuingTurn();
+    }
+  else if (player->getType() == Player::HUMAN && 
+	   d_gameScenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
+    {
+      if (player->hasAlreadyInitializedTurn())
+	{
+	  unlock_inputs();
+	  player->getStacklist()->setActivestack(0);
+	  center_view_on_city();
+	  update_sidebar_stats();
+	  update_control_panel();
+	  update_stack_info();
+	  game_loaded.emit(player);
+	  player->loadPbmGame();
+	  d_nextTurn->setContinuingTurn();
+	}
+      else
+	{
+	  lock_inputs();
+	}
     }
 
-  if (player->getType() == Player::HUMAN)
-    d_nextTurn->setContinuingTurn();
   d_nextTurn->start();
 #if 0
   else
@@ -1084,10 +1114,10 @@ void Game::on_fight_started(Fight &fight)
   
   //FIXME: zoom the map here if we're attacking an observable human, 
   //from an unobservable computer player
-  bigmap->setFighting(true);
+  bigmap->setFighting(LocationBox(fight.getAttackers().front()->getPos()));
   bigmap->draw();
   fight_started.emit(fight);
-  bigmap->setFighting(false);
+  bigmap->setFighting(LocationBox(Vector<int>(-1,-1)));
   bigmap->draw();
 }
 
@@ -1212,7 +1242,7 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
 	  Stack *o = neu->getStacklist()->getObjectAt(city->getPos());
 	  if (o)
 	    {
-	      int army_type = o->getStrongestArmy()->getType();
+	      int army_type = o->getStrongestArmy()->getTypeId();
 	      for (int i = 0; i < city->getMaxNoOfProductionBases(); i++)
 		{
 		  if (city->getArmytype(i) == army_type)
@@ -1228,10 +1258,19 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
   return;
 }
     
-bool Game::recruitHero(Hero *hero, City *city, int gold)
+bool Game::recruitHero(HeroProto *hero, City *city, int gold)
 {
-  bool retval = hero_offers_service (city->getOwner(), hero, city, gold);
+  bool retval = hero_offers_service.emit (city->getOwner(), hero, city, gold);
   if (d_gameScenario->getRound() == 1)
     city_visited.emit(city);
   return retval;
 }
+    
+bool Game::saveTurnFile(std::string turnfile)
+{
+  bool broken;
+  //trigger the GameServer to spit out a set of networkactions and networkhistory events for the active player, into a file.
+  if (d_gameScenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
+    PbmGameServer::getInstance()->endTurn(turnfile, broken);
+  return true;
+}
