@@ -31,18 +31,19 @@
 #include "glade-helpers.h"
 #include "image-helpers.h"
 #include "input-helpers.h"
-#include "../ucompose.hpp"
-#include "../defs.h"
-#include "../army.h"
-#include "../player.h"
-#include "../city.h"
-#include "../GraphicsCache.h"
-#include "../armysetlist.h"
+#include "ucompose.hpp"
+#include "defs.h"
+#include "army.h"
+#include "armyprodbase.h"
+#include "player.h"
+#include "city.h"
+#include "GraphicsCache.h"
+#include "armysetlist.h"
 #include "buy-production-dialog.h"
 #include "destination-dialog.h"
-#include "../GameMap.h"
-#include "../citylist.h"
-#include "../playerlist.h"
+#include "GameMap.h"
+#include "citylist.h"
+#include "playerlist.h"
 
 CityWindow::CityWindow(City *c, bool razing_possible, 
 		       bool see_opponents_production)
@@ -55,6 +56,9 @@ CityWindow::CityWindow(City *c, bool razing_possible,
     Gtk::Dialog *d = 0;
     xml->get_widget("dialog", d);
     dialog.reset(d);
+    decorate(dialog.get());
+    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+    set_title(c->getName());
     
     xml->get_widget("map_image", map_image);
 
@@ -151,7 +155,7 @@ void CityWindow::run()
 void CityWindow::fill_in_city_info()
 {
     
-    dialog->set_title(city->getName());
+    set_title(city->getName());
 
     // fill in status label
     Glib::ustring s;
@@ -269,7 +273,7 @@ void CityWindow::fill_in_production_info()
     }
     else
     {
-        const Army* a = city->getProductionBase(slot);
+        const ArmyProdBase * a = city->getProductionBase(slot);
 
 	// fill in first column
 	s1 += a->getName();
@@ -279,11 +283,11 @@ void CityWindow::fill_in_production_info()
 	s1 += String::ucompose(_("Time: %1"), a->getProduction());
 	s1 += "\n";
 	s1 += String::ucompose(_("Strength: %1"),
-			      a->getStat(Army::STRENGTH, false));
+			      a->getStrength());
 	
 	// fill in second column
 	s2 += "\n";
-	s2 += String::ucompose(_("Moves: %1"), a->getStat(Army::MOVES, false));
+	s2 += String::ucompose(_("Moves: %1"), a->getMaxMoves());
 	s2 += "\n";
 	s2 += String::ucompose(_("Cost: %1"), a->getUpkeep());
 
@@ -295,7 +299,7 @@ void CityWindow::fill_in_production_info()
             s3 += String::ucompose(_(", then to %1"), 
                                    dest ? dest->getName() : "Standard");
           }
-      pic = to_pixbuf(gc->getArmyPic(as, a->getType(), player, NULL));
+      pic = to_pixbuf(gc->getArmyPic(as, a->getTypeId(), player, NULL));
     }
     
     current_image->set(pic);
@@ -351,7 +355,7 @@ bool CityWindow::on_production_button_event(GdkEventButton *e, Gtk::ToggleButton
 	}
 	assert(slot != -1);
 
-	const Army *army = city->getProductionBase(slot);
+	const ArmyProdBase *army = city->getProductionBase(slot);
 
 	if (army)
 	    army_info_tip.reset(new ArmyInfoTip(toggle, army));
@@ -431,6 +435,9 @@ void CityWindow::on_rename_clicked ()
     Gtk::Dialog *d;
     renamexml->get_widget("dialog", d);
     renamedialog.reset(d);
+    Decorated decorator;
+    decorator.decorate(renamedialog.get());
+    decorator.window_closed.connect(sigc::mem_fun(renamedialog.get(), &Gtk::Dialog::hide));
     renamedialog->set_transient_for(*dialog.get());
     
     Glib::ustring s = _("Rename City");
@@ -440,7 +447,7 @@ void CityWindow::on_rename_clicked ()
     Gtk::Entry *e;
     renamexml->get_widget("name_entry", e);
 
-    d->set_title(s);
+    decorator.set_title(s);
     s = _("Type the new name for this city:");
     l->set_text(s);
 
@@ -466,6 +473,9 @@ void CityWindow::on_raze_clicked ()
     Gtk::Dialog *d;
     razexml->get_widget("dialog", d);
     razedialog.reset(d);
+    Decorated decorator;
+    decorator.decorate(razedialog.get());
+    decorator.window_closed.connect(sigc::mem_fun(razedialog.get(), &Gtk::Dialog::hide));
     razedialog->set_transient_for(*dialog.get());
     
     Glib::ustring s = _("Raze City");
@@ -473,7 +483,7 @@ void CityWindow::on_raze_clicked ()
     Gtk::Label *l;
     razexml->get_widget("label", l);
 
-    d->set_title(s);
+    decorator.set_title(s);
     s = String::ucompose(_("Are you sure that you want to raze %1?"), 
                            city->getName());
     s += "\n";
