@@ -25,20 +25,22 @@
 #include <gtkmm/filefilter.h>
 
 #include "game-preferences-dialog.h"
+#include "cycle-button.h"
 
 #include "glade-helpers.h"
 #include "image-helpers.h"
-#include "../ucompose.hpp"
-#include "../defs.h"
-#include "../File.h"
-#include "../xmlhelper.h"
-#include "../armysetlist.h"
-#include "../shieldsetlist.h"
-#include "../GameScenario.h"
-#include "../GraphicsCache.h"
-#include "../tilesetlist.h"
-#include "../citysetlist.h"
-#include "../player.h"
+#include "ucompose.hpp"
+#include "defs.h"
+#include "File.h"
+#include "xmlhelper.h"
+#include "armysetlist.h"
+#include "shieldsetlist.h"
+#include "GameScenario.h"
+#include "GraphicsCache.h"
+#include "GraphicsLoader.h"
+#include "tilesetlist.h"
+#include "citysetlist.h"
+#include "player.h"
 
 static bool inhibit_difficulty_combobox = false;
 
@@ -50,6 +52,8 @@ void GamePreferencesDialog::init()
     Gtk::Dialog *d = 0;
     xml->get_widget("dialog", d);
     dialog.reset(d);
+    decorate(dialog.get());
+    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
 
     xml->get_widget("start_game_button", start_game_button);
     xml->get_widget("difficulty_label", difficulty_label);
@@ -94,6 +98,8 @@ void GamePreferencesDialog::init()
     xml->get_widget("difficulty_combobox", difficulty_combobox);
 
     xml->get_widget("players_vbox", players_vbox);
+    xml->get_widget("game_name_label", game_name_label);
+    xml->get_widget("game_name_entry", game_name_entry);
 
     // add default players
     default_player_names.push_back("The Sirians");
@@ -234,14 +240,24 @@ void GamePreferencesDialog::init()
   return;
 }
 
-GamePreferencesDialog::GamePreferencesDialog()
+GamePreferencesDialog::GamePreferencesDialog(GameScenario::PlayMode play_mode)
 {
+  mode = play_mode;
   init();
+  if (mode != GameScenario::NETWORKED)
+    {
+      delete game_name_label;
+      delete game_name_entry;
+    }
+
 }
 
 GamePreferencesDialog::GamePreferencesDialog(std::string filename)
 {
+  mode = GameScenario::HOTSEAT;
   init ();
+  delete game_name_label;
+  delete game_name_entry;
   load_map_radio->set_active(true);
   load_map_filechooser->set_filename(filename);
   load_map_radio->set_sensitive(false);
@@ -265,11 +281,16 @@ void GamePreferencesDialog::hide()
   dialog->hide();
 }
 
-bool GamePreferencesDialog::run()
+bool GamePreferencesDialog::run(std::string nickname)
 {
-  Shieldsetlist::getInstance()->instantiatePixmaps();
     dialog->show_all();
-  update_shields();
+    if (mode == GameScenario::NETWORKED)
+      {
+	std::string text = nickname;
+	text += "'s game";
+	game_name_entry->set_text(text);
+      }
+    update_shields();
     int response = dialog->run();
     if (response == 0)
       return true;
@@ -297,15 +318,16 @@ void GamePreferencesDialog::add_player(const Glib::ustring &type,
   //okay, add a new hbox, with a combo and an entry in it
   //add it to players_vbox
   Gtk::HBox *player_hbox = new Gtk::HBox();
-  Gtk::ComboBoxText *player_type = new Gtk::ComboBoxText();
-  player_type->signal_changed().connect
+  std::list<Gtk::Button*> states;
+  states.push_back(manage(new Gtk::Button(HUMAN_PLAYER_TYPE)));
+  states.push_back(manage(new Gtk::Button(EASY_PLAYER_TYPE)));
+  states.push_back(manage(new Gtk::Button(HARD_PLAYER_TYPE)));
+  states.push_back(manage(new Gtk::Button(NO_PLAYER_TYPE)));
+  CycleButton *player_type = new CycleButton(states);
+  player_type->signal_changed.connect
       (sigc::mem_fun(this, &GamePreferencesDialog::on_player_type_changed));
   Gtk::Entry *player_name = new Gtk::Entry();
   player_name->set_text(name);
-  player_type->append_text(HUMAN_PLAYER_TYPE);
-  player_type->append_text(EASY_PLAYER_TYPE);
-  player_type->append_text(HARD_PLAYER_TYPE);
-  player_type->append_text(NO_PLAYER_TYPE);
 
   if (type == HUMAN_PLAYER_TYPE)
     player_type->set_active(0);
@@ -320,7 +342,7 @@ void GamePreferencesDialog::add_player(const Glib::ustring &type,
   player_types.push_back(player_type);
   player_names.push_back(player_name);
   player_hbox->pack_start(*manage(player_name), Gtk::PACK_SHRINK, 10);
-  player_hbox->add(*manage(player_type));
+  player_hbox->add(*manage(player_type->get_widget()));
   players_vbox->add(*manage(player_hbox));
 }
 
@@ -340,7 +362,7 @@ void GamePreferencesDialog::on_random_map_toggled()
     
     if (random_map)
       {
-	std::list<Gtk::ComboBoxText *>::iterator c = player_types.begin();
+	std::list<CycleButton *>::iterator c = player_types.begin();
 	std::list<Gtk::Entry *>::iterator e = player_names.begin();
 	for (; c != player_types.end(); c++, e++)
 	  {
@@ -354,7 +376,7 @@ void GamePreferencesDialog::on_random_map_toggled()
 	if (load_map_filechooser->get_filename().empty() == false)
 	  {
 	    //disable all names, and types
-	    std::list<Gtk::ComboBoxText *>::iterator c = player_types.begin();
+	    std::list<CycleButton *>::iterator c = player_types.begin();
 	    std::list<Gtk::Entry *>::iterator e = player_names.begin();
 	    for (; c != player_types.end(); c++, e++)
 	      {
@@ -453,6 +475,7 @@ void GamePreferencesDialog::update_shields()
 {
   if (dialog->is_realized() == false)
     return;
+  GraphicsLoader::instantiatePixmaps(Shieldsetlist::getInstance());
   //get rid of the old shields
   player_shields.clear();
 
@@ -486,7 +509,7 @@ void GamePreferencesDialog::update_shields()
 void GamePreferencesDialog::on_player_type_changed()
 {
   Uint32 offcount = 0;
-    std::list<Gtk::ComboBoxText *>::iterator c = player_types.begin();
+    std::list<CycleButton *>::iterator c = player_types.begin();
     for (; c != player_types.end(); c++)
       {
 	if (player_type_to_enum((*c)->get_active_text()) ==
@@ -503,7 +526,7 @@ void GamePreferencesDialog::on_player_type_changed()
 void GamePreferencesDialog::update_difficulty_rating()
 {
     GameParameters g;
-    std::list<Gtk::ComboBoxText *>::iterator c = player_types.begin();
+    std::list<CycleButton *>::iterator c = player_types.begin();
     for (; c != player_types.end(); c++)
       {
 	GameParameters::Player p;
@@ -623,7 +646,7 @@ void GamePreferencesDialog::on_start_game_clicked()
 	g.map_path = load_map_filechooser->get_filename();
 
     int id = 0;
-    std::list<Gtk::ComboBoxText *>::iterator c = player_types.begin();
+    std::list<CycleButton *>::iterator c = player_types.begin();
     std::list<Gtk::Entry *>::iterator e = player_names.begin();
     for (; c != player_types.end(); c++, e++, id++)
       {
@@ -663,6 +686,11 @@ void GamePreferencesDialog::on_start_game_clicked()
 
     g.difficulty = GameScenario::calculate_difficulty_rating(g);
 
+    if (mode == GameScenario::NETWORKED)
+      g.name = game_name_entry->get_text();
+    else
+      g.name = _("Autogenerated");
+
     //don't start if the armyset size differs from the terrain set size.
     Uint32 army_tilesize = al->getTileSize(al->getArmysetId(g.army_theme));
     Uint32 terrain_size = tl->getTileset(g.tile_theme)->getTileSize();
@@ -767,7 +795,7 @@ void GamePreferencesDialog::on_difficulty_changed()
     {
       if (type_num)
 	{
-	  std::list<Gtk::ComboBoxText *>::iterator c = player_types.begin();
+	  std::list<CycleButton *>::iterator c = player_types.begin();
 	  for (; c != player_types.end(); c++)
 	    {
 	      if ((*c)->get_active_row_number() != 3)
@@ -895,3 +923,7 @@ bool GamePreferencesDialog::scan_players(std::string tag, XML_Helper* helper)
     return true;
 }
 
+void GamePreferencesDialog::set_title(std::string text)
+{
+  Decorated::set_title(text);
+}
