@@ -34,33 +34,42 @@
 #include <gtkmm/filechooserbutton.h>
 #include <gtkmm/widget.h>
 #include <gtkmm/scale.h>
+#include <gtkmm/entry.h>
 
-#include "../game-parameters.h"
+#include "game-parameters.h"
 #include <SDL.h>
 #include "game-options-dialog.h"
+#include "GameScenario.h"
 
+#include "decorated.h"
 class XML_Helper;
+class CycleButton;
 
 // dialog for choosing parameters for starting a new game
-class GamePreferencesDialog
+class GamePreferencesDialog: public Decorated
 {
  public:
-    GamePreferencesDialog();
+    GamePreferencesDialog(GameScenario::PlayMode mode);
     GamePreferencesDialog(std::string filename);
     ~GamePreferencesDialog();
 
+    void set_title(std::string title);
+
     void set_parent_window(Gtk::Window &parent);
 
     sigc::signal<void, GameParameters> game_started;
     
-    bool run();
+    bool run(std::string nickname = "guest");
     void hide();
     
  private:
     void init();
     std::auto_ptr<Gtk::Dialog> dialog;
+    GameScenario::PlayMode mode;
 
     Gtk::Button *start_game_button;
+    Gtk::Label *game_name_label;
+    Gtk::Entry *game_name_entry;
     Gtk::ComboBoxText *tile_theme_combobox;
     Gtk::ComboBoxText *army_theme_combobox;
     Gtk::ComboBoxText *shield_theme_combobox;
@@ -97,7 +106,7 @@ class GamePreferencesDialog
     player_name_seq default_player_names;
     player_name_seq::iterator current_player_name;
     
-    std::list<Gtk::ComboBoxText *> player_types;
+    std::list<CycleButton*> player_types;
     std::list<Gtk::Entry *> player_names;
     std::list<Gtk::Image *> player_shields;
 
