@@ -43,6 +43,7 @@
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/alignment.h>
+#include <glib.h>
 
 #include "game-window.h"
 
@@ -52,6 +53,8 @@
 #include "input-helpers.h"
 #include "error-utils.h"
 
+#include "driver.h"
+
 #include "fight-window.h"
 #include "city-window.h"
 #include "army-gains-level-dialog.h"
@@ -78,43 +81,53 @@
 #include "stack-info-dialog.h"
 #include "timed-message-dialog.h"
 
-#include "../ucompose.hpp"
-#include "../defs.h"
-#include "../sound.h"
-#include "../File.h"
-#include "../game.h"
-#include "../gamebigmap.h"
-#include "../smallmap.h"
-#include "../GameScenarioOptions.h"
-#include "../army.h"
-#include "../ruin.h"
-#include "../path.h"
-#include "../player.h"
-#include "../stacklist.h"
-#include "../signpostlist.h"
-#include "../playerlist.h"
-#include "../citylist.h"
-#include "../hero.h"
-#include "../temple.h"
-#include "../city.h"
-#include "../Quest.h"
-#include "../QuestsManager.h"
-#include "../stack.h"
-#include "../GraphicsCache.h"
-#include "../QuestsManager.h"
-#include "../Quest.h"
-#include "../QCitySack.h"
-#include "../QCityRaze.h"
-#include "../QCityOccupy.h"
-#include "../QPillageGold.h"
-#include "../counter.h"
-#include "../armysetlist.h"
-#include "../CreateScenario.h"
-#include "../reward.h"
-#include "../Configuration.h"
-#include "../GameMap.h"
-#include "../Item.h"
-#include "../shieldsetlist.h"
+#include "ucompose.hpp"
+#include "defs.h"
+#include "sound.h"
+#include "File.h"
+#include "game.h"
+#include "gamebigmap.h"
+#include "smallmap.h"
+#include "GameScenarioOptions.h"
+#include "army.h"
+#include "ruin.h"
+#include "path.h"
+#include "player.h"
+#include "stacklist.h"
+#include "signpostlist.h"
+#include "playerlist.h"
+#include "citylist.h"
+#include "hero.h"
+#include "heroproto.h"
+#include "temple.h"
+#include "city.h"
+#include "Quest.h"
+#include "QuestsManager.h"
+#include "stack.h"
+#include "GraphicsCache.h"
+#include "GraphicsLoader.h"
+#include "QuestsManager.h"
+#include "Quest.h"
+#include "QCitySack.h"
+#include "QCityRaze.h"
+#include "QCityOccupy.h"
+#include "QPillageGold.h"
+#include "counter.h"
+#include "armysetlist.h"
+#include "tilesetlist.h"
+#include "CreateScenario.h"
+#include "reward.h"
+#include "Configuration.h"
+#include "GameMap.h"
+#include "Item.h"
+#include "shieldsetlist.h"
+#include "game-server.h"
+#include "game-client.h"
+#include "NextTurnHotseat.h"
+#include "NextTurnPbm.h"
+#include "NextTurnNetworked.h"
+#include "pbm-game-server.h"
+#include "network_player.h"
 
 
 GameWindow::GameWindow()
@@ -127,10 +140,15 @@ GameWindow::GameWindow()
     Gtk::Window *w = 0;
     xml->get_widget("window", w);
     window.reset(w);
+    decorate(window.get(), File::getMiscFile("various/back.bmp"));
+    
+    window_closed.connect
+      (sigc::mem_fun(*this, &GameWindow::on_quit_activated));
 
-      w->signal_delete_event().connect
-       (sigc::mem_fun(*this, &GameWindow::on_delete_event));
+    w->signal_delete_event().connect
+      (sigc::mem_fun(*this, &GameWindow::on_delete_event));
 
+    xml->get_widget("menubar", menubar);
     xml->get_widget("sdl_container", sdl_container);
     xml->get_widget("stack_info_box", stack_info_box);
     xml->get_widget("stack_info_container", stack_info_container);
@@ -227,12 +245,16 @@ GameWindow::GameWindow()
     se_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[7])));
 
     // connect callbacks for the menu
-    xml->connect_clicked("load_game_menuitem",
-			 sigc::mem_fun(*this, &GameWindow::on_load_game_activated));
-    xml->connect_clicked("save_game_menuitem",
-			 sigc::mem_fun(*this, &GameWindow::on_save_game_activated));
-    xml->connect_clicked("save_game_as_menuitem", 
-			 sigc::mem_fun(*this, &GameWindow::on_save_game_as_activated));
+    xml->get_widget("load_game_menuitem", load_game_menuitem);
+    load_game_menuitem->signal_activate().connect
+      (sigc::mem_fun(*this, &GameWindow::on_load_game_activated));
+    xml->get_widget("save_game_menuitem", save_game_menuitem);
+    save_game_menuitem->signal_activate().connect
+      (sigc::mem_fun(*this, &GameWindow::on_save_game_activated));
+    xml->get_widget("save_game_as_menuitem", save_game_as_menuitem);
+    save_game_as_menuitem->signal_activate().connect
+      (sigc::mem_fun(*this, &GameWindow::on_save_game_as_activated));
+
     xml->connect_clicked("quit_menuitem", 
 			 sigc::mem_fun(*this, &GameWindow::on_quit_activated));
     xml->connect_clicked("army_report_menuitem",
@@ -253,6 +275,9 @@ GameWindow::GameWindow()
     xml->connect_clicked("preferences_menuitem", 
 			 sigc::mem_fun(*this, &GameWindow::on_preferences_activated));
 
+    xml->get_widget("show_lobby_menuitem", show_lobby_menuitem);
+    show_lobby_menuitem->signal_activate().connect
+      (sigc::mem_fun(*this, &GameWindow::on_show_lobby_activated));
     xml->get_widget("end_turn_menuitem", end_turn_menuitem);
     xml->get_widget("move_all_menuitem", move_all_menuitem);
     xml->get_widget("disband_menuitem", disband_menuitem);
@@ -287,7 +312,33 @@ GameWindow::GameWindow()
 			 sigc::mem_fun(*this, &GameWindow::on_triumphs_activated));
     xml->connect_clicked("help_about_menuitem",
 			 sigc::mem_fun(*this, &GameWindow::on_help_about_activated));
+    xml->connect_clicked("online_help_menuitem",
+			 sigc::mem_fun(*this, &GameWindow::on_online_help_activated));
     d_quick_fights = false;
+
+    if (Configuration::s_decorated == true)
+      {
+	//colour the menubar
+	Glib::RefPtr<Gtk::Style> copy;
+	Glib::RefPtr<Gdk::Pixbuf> back;
+	Glib::RefPtr<Gdk::Pixmap> pixmap;
+	Glib::RefPtr<Gdk::Bitmap> bitmap;
+	copy = menubar->get_style()->copy();
+	back = Gdk::Pixbuf::create_from_file
+	  (File::getMiscFile("various/back.bmp"));
+	pixmap = Gdk::Pixmap::create
+	  (window->get_window(), back->get_width(), back->get_height());
+	back->composite_color(back, 0, 0, 
+			      back->get_width(), back->get_height(), 
+			      0.0, 0.0, 1.0, 1.0, Gdk::INTERP_NEAREST, 127, 
+			      0, 0, 64, 0, 0);
+	back->render_pixmap_and_mask(pixmap, bitmap, 10);
+	copy->set_bg_pixmap(Gtk::STATE_NORMAL, pixmap);
+	menubar->set_style(copy);
+      }
+
+
+
 }
 
 GameWindow::~GameWindow()
@@ -367,144 +418,56 @@ void GameWindow::init(int width, int height)
 		     G_CALLBACK(surface_attached_helper), this);
     
     sdl_container->add(*sdl_widget);
+
+	    
+
 }
 
-std::string
-GameWindow::create_and_dump_scenario(const std::string &file, const GameParameters &g)
+void GameWindow::new_network_game(GameScenario *game_scenario, NextTurn *next_turn)
 {
-    CreateScenario creator (g.map.width, g.map.height);
-
-    // then fill the other players
-    int c = 0;
-    int army_id = Armysetlist::getInstance()->getArmysetId(g.army_theme);
-    Shieldsetlist *ssl = Shieldsetlist::getInstance();
-    for (std::vector<GameParameters::Player>::const_iterator
-	     i = g.players.begin(), end = g.players.end();
-	 i != end; ++i, ++c) {
-	
-	if (i->type == GameParameters::Player::OFF)
-	{
-            fl_counter->getNextId();
-	    continue;
-	}
-	
-	Player::Type type;
-	if (i->type == GameParameters::Player::EASY)
-	    type = Player::AI_FAST;
-	else if (i->type == GameParameters::Player::HARD)
-	    type = Player::AI_SMART;
-	else
-	    type = Player::HUMAN;
-
-	creator.addPlayer(i->name, army_id, ssl->getColor(g.shield_theme, 
-							  c), type);
-    }
-
-    // the neutral player must come last so it has the highest id among players
-    creator.addNeutral(_("Neutral"), army_id, 
-		       ssl->getColor(g.shield_theme, MAX_PLAYERS),
-		       Player::AI_DUMMY);
-
-    // now fill in some map information
-    creator.setMapTiles(g.tile_theme);
-    creator.setShieldset(g.shield_theme);
-    creator.setCityset(g.city_theme);
-    creator.setNoCities(g.map.cities);
-    creator.setNoRuins(g.map.ruins);
-    creator.setNoTemples(4);
-
-    // terrain: the scenario generator also accepts input with a sum of
-    // more than 100%, so the thing is rather easy here
-    creator.setPercentages(g.map.grass, g.map.water, g.map.forest, g.map.swamp,
-			   g.map.hills, g.map.mountains);
-
-    int area = g.map.width * g.map.height;
-    creator.setNoSignposts(int(area * (g.map.grass / 100.0) * 0.0030));
-
-    // and tell it the turn mode
-    if (g.process_armies == GameParameters::PROCESS_ARMIES_AT_PLAYERS_TURN)
-        creator.setTurnmode(true);
-    else
-	creator.setTurnmode(false);
-	
-    // now create the map and dump the created map
-    std::string path = File::getSavePath();
-    path += file;
-    
-    //GenerationProgressWindow gpw;
-    //gpw.show_all();
-    //gpw.setGenerator(creator.getGenerator());
-    creator.create(g);
-    creator.dump(path);
-    
-    return path;
+  if (GameServer::getInstance()->isListening() == true)
+    GameServer::getInstance()->round_begins.connect(sigc::mem_fun(this, &GameWindow::on_remote_next_player_turn));
+  else
+      GameClient::getInstance()->round_begins.connect(sigc::mem_fun(this, &GameWindow::on_remote_next_player_turn));
+  bool success = false;
+  //stop_game();
+  success = setup_game(game_scenario, next_turn);
+  if (!success)
+    return;
+  setup_signals(game_scenario);
+  game->redraw();
+  game->startGame();
 }
 
-    
-void GameWindow::new_game(GameParameters g)
+void GameWindow::continue_network_game(NextTurn *next_turn)
 {
-    if (g.map_path.empty()) {
-	// construct new random scenario if we're not going to load the game
-	std::string path = create_and_dump_scenario("random.map", g);
-	g.map_path = path;
-    }
-
-    stop_game();
-
-    bool broken = false;
-    GameScenario* game_scenario = new GameScenario(g.map_path, broken);
-
-    if (broken)
-      {
-	on_message_requested("Corrupted saved game file.");
-	game_ended.emit();
-	return;
-      }
-
-    if (game_scenario->getRound() == 0)
-      {
-	Playerlist::getInstance()->syncPlayers(g.players);
-	game_scenario->setupFog(g.hidden_map);
-	game_scenario->setupCities(g.quick_start);
-	game_scenario->setupDiplomacy(g.diplomacy);
-	game_scenario->nextRound();
-      }
-
-
-    bool success = setup_game(game_scenario);
-    if (!success)
-      return;
-    setup_signals();
-    game->startGame();
-    //FIXME: we don't get here until the game ends.
-    //is that right?
+  next_turn->start();
 }
 
-void GameWindow::load_game(std::string file_path)
+void GameWindow::new_game(GameScenario *game_scenario, NextTurn *next_turn)
 {
-    current_save_filename = file_path;
+  bool success = false;
+  stop_game();
+  success = setup_game(game_scenario, next_turn);
+  if (!success)
+    return;
+  setup_signals(game_scenario);
+  game->startGame();
+  //we don't get here until the game ends.
+}
 
-    stop_game();
+void GameWindow::load_game(GameScenario *game_scenario, NextTurn *next_turn)
+{
+  bool success = false;
+  success = setup_game(game_scenario, next_turn);
+  if (!success)
+    return;
 
-    bool broken = false;
-    GameScenario* game_scenario = new GameScenario(file_path, broken);
-
-    if (broken)
-      {
-	on_message_requested("Corrupted saved game file.");
-	game_ended.emit();
-	return;
-      }
-
-    bool success = setup_game(game_scenario);
-    if (!success)
-      return;
-    setup_signals();
-    game->loadGame();
-    //FIXME: we don't get here until the game ends.
-    //is that right?
-    if (Playerlist::getInstance()->countPlayersAlive())
-      game->redraw();
+  setup_signals(game_scenario);
+  game->loadGame();
+  //we don't get here until the game ends.
+  if (Playerlist::getInstance()->countPlayersAlive())
+    game->redraw();
 }
 
 void GameWindow::setup_button(Gtk::Button *button,
@@ -525,7 +488,7 @@ void GameWindow::setup_menuitem(Gtk::MenuItem *item,
     (game_signal.connect(sigc::mem_fun(item, &Gtk::Widget::set_sensitive)));
 }
 
-void GameWindow::setup_signals()
+void GameWindow::setup_signals(GameScenario *game_scenario)
 {
   // get rid of the connections that might be still around from last time
   std::list<sigc::connection>::iterator it = connections.begin();
@@ -570,6 +533,13 @@ void GameWindow::setup_signals()
   setup_button(move_all_button,
 	       sigc::mem_fun(game.get(), &Game::move_all_stacks),
 	       game->can_move_all_stacks);
+  setup_button(end_turn_button,
+	       sigc::mem_fun(game.get(), &Game::end_turn),
+	       game->can_end_turn);
+  if (game_scenario->getPlayMode() ==  GameScenario::PLAY_BY_MAIL)
+    setup_button(end_turn_button,
+		 sigc::mem_fun(*this, &GameWindow::end_turn_play_by_mail),
+		 game->can_end_turn);
   setup_button(nw_keypad_button,
 	       sigc::mem_fun(game.get(), &Game::move_selected_stack_northwest),
 	       game->can_move_selected_stack);
@@ -597,12 +567,25 @@ void GameWindow::setup_signals()
   setup_menuitem(move_all_menuitem,
 		 sigc::mem_fun(game.get(), &Game::move_all_stacks),
 		 game->can_move_all_stacks);
-  setup_button(end_turn_button,
-	       sigc::mem_fun(game.get(), &Game::end_turn),
-	       game->can_end_turn);
   setup_menuitem(end_turn_menuitem,
 		 sigc::mem_fun(game.get(), &Game::end_turn),
 		 game->can_end_turn);
+  if (game_scenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
+    setup_menuitem(end_turn_menuitem,
+		   sigc::mem_fun(*this, &GameWindow::end_turn_play_by_mail),
+		   game->can_end_turn);
+  if (game_scenario->getPlayMode() == GameScenario::NETWORKED)
+    {
+      load_game_menuitem->set_sensitive(false);
+      if (GameServer::getInstance()->isListening() == false)
+	{
+	  save_game_menuitem->set_sensitive(false);
+	  save_game_as_menuitem->set_sensitive(false);
+	}
+    }
+  else
+    show_lobby_menuitem->set_sensitive(false);
+
   setup_menuitem(disband_menuitem,
 		 sigc::mem_fun(*this, &GameWindow::on_disband_activated),
 		 game->can_disband_stack);
@@ -757,6 +740,9 @@ void GameWindow::setup_signals()
 	 (sigc::mem_fun(*this, &GameWindow::on_bigmap_cursor_changed)));
     }
 
+  connections.push_back
+    (game->remote_next_player_turn.connect
+     (sigc::mem_fun(*this, &GameWindow::on_remote_next_player_turn)));
 }
 
 void GameWindow::show_city_production_report (bool destitute)
@@ -778,6 +764,37 @@ void GameWindow::change_diplomacy_button_image (bool proposals_present)
     diplomacy_button->property_image() = new Gtk::Image(d_button_images[0]);
 }
 
+void GameWindow::end_turn_play_by_mail ()
+{
+  //prompt to save the turn file!
+  Gtk::FileChooserDialog chooser(*window.get(), _("Save your Turn file and mail it back"),
+				 Gtk::FILE_CHOOSER_ACTION_SAVE);
+  Gtk::FileFilter trn_filter;
+  trn_filter.add_pattern("*.trn");
+  chooser.set_filter(trn_filter);
+  chooser.set_current_folder(Glib::get_home_dir());
+
+  chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  chooser.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
+  chooser.set_default_response(Gtk::RESPONSE_ACCEPT);
+
+  chooser.show_all();
+  int res = chooser.run();
+  chooser.hide();
+
+  if (res == Gtk::RESPONSE_ACCEPT)
+    {
+      std::string filename = chooser.get_filename();
+
+      game->saveTurnFile(filename);
+      TimedMessageDialog dialog
+	(*window.get(), ("Now send the turn file back to the game master."), 0);
+      dialog.run();
+      dialog.hide();
+    }
+  game_ended.emit();
+}
+
 void GameWindow::update_diplomacy_button (bool sensitive)
 {
   if (Playerlist::getActiveplayer()->getType() != Player::HUMAN)
@@ -793,13 +810,21 @@ void GameWindow::update_diplomacy_button (bool sensitive)
   diplomacy_button->set_sensitive(sensitive);
 }
 
-bool GameWindow::setup_game(GameScenario *game_scenario)
+bool GameWindow::setup_game(GameScenario *game_scenario, NextTurn *nextTurn)
 {
+  Playerlist *pl = Playerlist::getInstance();
+  Armysetlist *al = Armysetlist::getInstance();
+  for (Playerlist::iterator i = pl->begin(); i != pl->end(); i++)
+    if ((*i)->getType() != Player::NETWORKED)
+      GraphicsLoader::instantiatePixmaps(al->getArmyset((*i)->getArmyset()));
+
+  GraphicsLoader::instantiatePixmaps(GameMap::getInstance()->getTileset());
+  GraphicsLoader::instantiatePixmaps(GameMap::getInstance()->getShieldset());
 
   Sound::getInstance()->haltMusic(false);
   Sound::getInstance()->enableBackground();
 
-  game.reset(new Game(game_scenario));
+  game.reset(new Game(game_scenario, nextTurn));
 
   show_shield_turn();
   return true;
@@ -897,7 +922,6 @@ void GameWindow::on_sdl_surface_changed()
 
 void GameWindow::on_load_game_activated()
 {
-  //return;
   Gtk::FileChooserDialog chooser(*window.get(), _("Choose Game to Load"));
   Gtk::FileFilter sav_filter;
   sav_filter.add_pattern("*.sav");
@@ -915,7 +939,33 @@ void GameWindow::on_load_game_activated()
   if (res == Gtk::RESPONSE_ACCEPT)
     {
       std::string filename = chooser.get_filename();
-      load_game(filename);
+      current_save_filename = filename;
+      stop_game();
+    
+      bool broken = false;
+      GameScenario* game_scenario = new GameScenario(filename, broken);
+
+      if (broken)
+	{
+	  on_message_requested(_("Corrupted saved game file."));
+	  game_ended.emit();
+	  return;
+	}
+      if (game_scenario->getPlayMode() == GameScenario::HOTSEAT)
+	load_game(game_scenario, 
+		  new NextTurnHotseat(game_scenario->getTurnmode(),
+				      game_scenario->s_random_turns));
+      else if (game_scenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
+	{
+	  PbmGameServer::getInstance()->start();
+	  load_game(game_scenario, 
+		    new NextTurnPbm(game_scenario->getTurnmode(),
+				    game_scenario->s_random_turns));
+	}
+      else if (game_scenario->getPlayMode() == GameScenario::NETWORKED)
+	load_game(game_scenario, 
+		  new NextTurnNetworked(game_scenario->getTurnmode(),
+					game_scenario->s_random_turns));
     }
 }
 
@@ -974,6 +1024,10 @@ void GameWindow::on_quit_activated()
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+
   dialog->set_transient_for(*window.get());
 
   int response = dialog->run();
@@ -1031,9 +1085,12 @@ void GameWindow::on_signpost_activated()
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(_("Signpost"));
+  decorator.set_title(_("Signpost"));
 
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
   if (!stack)
@@ -1085,9 +1142,12 @@ void GameWindow::on_disband_activated()
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(_("Disband"));
+  decorator.set_title(_("Disband"));
 
   Gtk::Label *l;
   xml->get_widget("label", l);
@@ -1114,6 +1174,9 @@ void GameWindow::on_resignation_completed()
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   dialog->show_all();
@@ -1136,9 +1199,12 @@ void GameWindow::on_resign_activated()
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(_("Resign"));
+  decorator.set_title(_("Resign"));
 
   Gtk::Label *l;
   xml->get_widget("label", l);
@@ -1344,10 +1410,13 @@ void GameWindow::on_help_about_activated()
   Gtk::AboutDialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   dialog->set_version(PACKAGE_VERSION);
-  SDL_Surface *logo = File::getMiscPicture("castle_icon.png");
+  SDL_Surface *logo = GraphicsLoader::getMiscPicture("castle_icon.png");
   dialog->set_logo(to_pixbuf(logo));
   dialog->show_all();
   dialog->run();
@@ -1398,12 +1467,15 @@ void GameWindow::on_game_over(Player *winner)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   Gtk::Image *image;
   xml->get_widget("image", image);
 
-  SDL_Surface *win = File::getMiscPicture("win.png", false);
+  SDL_Surface *win = GraphicsLoader::getMiscPicture("win.png", false);
 
   image->property_pixbuf() = to_pixbuf(win);
 
@@ -1498,8 +1570,7 @@ bool GameWindow::on_army_button_event(GdkEventButton *e,
   if (event.button == MouseButtonEvent::RIGHT_BUTTON
       && event.state == MouseButtonEvent::PRESSED) {
 
-    army_info_tip.reset(new ArmyInfoTip(toggle, army, 
-					ArmyInfoTip::ARMY_INSTANCE));
+    army_info_tip.reset(new ArmyInfoTip(toggle, army));
 
     return true;
   }
@@ -1554,11 +1625,22 @@ void GameWindow::ensure_one_army_button_active()
 
 void GameWindow::on_sidebar_stats_changed(SidebarStats s)
 {
-  cities_stats_label->set_text(String::ucompose("%1", s.cities));
-  gold_stats_label->set_text(String::ucompose("%1", s.gold));
-  income_stats_label->set_text(String::ucompose("%1", s.income));
-  upkeep_stats_label->set_text(String::ucompose("%1", s.upkeep));
-  turn_label->set_text(String::ucompose("Turn %1", s.turns));
+  if (Configuration::s_decorated)
+    {
+      cities_stats_label->set_markup(String::ucompose("<b>%1</b>", s.cities));
+      gold_stats_label->set_markup(String::ucompose("<b>%1</b>", s.gold));
+      income_stats_label->set_markup(String::ucompose("<b>%1</b>", s.income));
+      upkeep_stats_label->set_markup(String::ucompose("<b>%1</b>", s.upkeep));
+      turn_label->set_markup(String::ucompose("<b>Turn %1</b>", s.turns));
+    }
+  else
+    {
+      cities_stats_label->set_markup(String::ucompose("%1", s.cities));
+      gold_stats_label->set_markup(String::ucompose("%1", s.gold));
+      income_stats_label->set_markup(String::ucompose("%1", s.income));
+      upkeep_stats_label->set_markup(String::ucompose("%1", s.upkeep));
+      turn_label->set_markup(String::ucompose("Turn %1", s.turns));
+    }
 }
 
 void GameWindow::on_smallmap_changed(SDL_Surface *map)
@@ -1615,6 +1697,7 @@ void GameWindow::fill_in_group_info (Stack *s)
 
 void GameWindow::show_stack(Stack *s)
 {
+  GraphicsCache *gc = GraphicsCache::getInstance();
   s->sortForViewing (true);
   stats_box->hide();
 
@@ -1626,7 +1709,7 @@ void GameWindow::show_stack(Stack *s)
       Gtk::VBox *toggle_box = manage(new Gtk::VBox);
 
       // image
-      toggle_box->add(*manage(new Gtk::Image(to_pixbuf(army->getPixmap()))));
+      toggle_box->add(*manage(new Gtk::Image(to_pixbuf(gc->getArmyPic(army)))));
       // number of moves
       Glib::ustring moves_str = String::ucompose("%1", army->getMoves());
       toggle_box->add(*manage(new Gtk::Label(moves_str,
@@ -1761,9 +1844,12 @@ void GameWindow::on_ruin_searched(Ruin *ruin, Stack *stack, Reward *reward)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(ruin->getName());
+  decorator.set_title(ruin->getName());
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -1810,9 +1896,12 @@ void GameWindow::on_ruinfight_started(Stack *attackers, Stack *defenders)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(_("Searching"));
+  decorator.set_title(_("Searching"));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -1836,12 +1925,15 @@ void GameWindow::on_ruinfight_finished(Fight::Result result)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   if (result == Fight::ATTACKER_WON)
-    dialog->set_title(_("Hero Victorious"));
+    decorator.set_title(_("Hero Victorious"));
   else
-    dialog->set_title(_("Hero Defeated"));
+    decorator.set_title(_("Hero Defeated"));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -1886,9 +1978,12 @@ void GameWindow::on_hero_brings_allies (int numAllies)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(_("Hero brings allies!"));
+  decorator.set_title(_("Hero brings allies!"));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -1903,7 +1998,7 @@ void GameWindow::on_hero_brings_allies (int numAllies)
   dialog->hide();
 }
 
-bool GameWindow::on_hero_offers_service(Player *player, Hero *hero, City *city, int gold)
+bool GameWindow::on_hero_offers_service(Player *player, HeroProto *hero, City *city, int gold)
 {
   HeroOfferDialog d(player, hero, city, gold);
   d.set_parent_window(*window.get());
@@ -1946,6 +2041,9 @@ bool GameWindow::on_stack_considers_treachery (Player *me, Stack *stack,
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -1986,9 +2084,12 @@ bool GameWindow::on_temple_searched(bool hasHero, Temple *temple, int blessCount
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(temple->getName());
+  decorator.set_title(temple->getName());
 
   Gtk::Label *l;
   Gtk::Button *close_button;
@@ -2132,6 +2233,9 @@ CityDefeatedAction GameWindow::on_city_defeated(City *city, int gold)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   Gtk::Image *image;
@@ -2237,9 +2341,12 @@ void GameWindow::on_city_looted (City *city, int gold)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(String::ucompose(_("%1 Looted"), city->getName()));
+  decorator.set_title(String::ucompose(_("%1 Looted"), city->getName()));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -2268,9 +2375,12 @@ void GameWindow::on_city_pillaged(City *city, int gold, int pillaged_army_type)
   Gtk::Image *pillaged_army_type_image;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(String::ucompose(_("Pillaged %1"), city->getName()));
+  decorator.set_title(String::ucompose(_("Pillaged %1"), city->getName()));
 
   Gtk::Label *pillaged_army_type_cost_label;
   xml->get_widget("pillaged_army_type_cost_label", pillaged_army_type_cost_label);
@@ -2320,9 +2430,12 @@ void GameWindow::on_city_sacked(City *city, int gold, std::list<Uint32> sacked_t
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(String::ucompose(_("Sacked %1"), city->getName()));
+  decorator.set_title(String::ucompose(_("Sacked %1"), city->getName()));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -2377,7 +2490,7 @@ void GameWindow::on_city_sacked(City *city, int gold, std::list<Uint32> sacked_t
 	}
       pic = to_pixbuf(gc->getArmyPic(as, *it, player, NULL));
       sack_image->set(pic);
-      const Army *a = 
+      const ArmyProto *a = 
 	Armysetlist::getInstance()->getArmy (player->getArmyset(), *it);
       s = String::ucompose("%1 gp", a->getProductionCost() / 2);
       sack_label->set_text(s);
@@ -2418,9 +2531,12 @@ void GameWindow::on_city_razed (City *city)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(String::ucompose(_("Razed %1"), city->getName()));
+  decorator.set_title(String::ucompose(_("Razed %1"), city->getName()));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -2479,6 +2595,21 @@ void GameWindow::show_shield_turn()
     shield_image[i]->clear();
 }
 
+void GameWindow::on_remote_next_player_turn()
+{
+  printf ("updating turn indicator now!\n");
+  std::auto_ptr<Gtk::Dialog> dialog;
+
+  while (g_main_context_iteration(NULL, FALSE)); //doEvents
+
+  if (Playerlist::isFinished()) //closed window while ai player moves
+    return;
+  d_quick_fights = false;
+  show_shield_turn();
+  turn_label->set_markup(String::ucompose("Turn %1", 
+					  GameScenarioOptions::s_round));
+}
+
 void GameWindow::on_next_player_turn(Player *player, unsigned int turn_number)
 {
   std::auto_ptr<Gtk::Dialog> dialog;
@@ -2502,6 +2633,9 @@ void GameWindow::on_next_player_turn(Player *player, unsigned int turn_number)
       dialog.reset(d);
       dialog->set_transient_for(*window.get());
 
+      Decorated decorator;
+      decorator.decorate(dialog.get());
+      decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
       Gtk::Image *image;
       xml->get_widget("image", image);
       image->property_file() = File::getMiscFile("various/ship.png");
@@ -2517,11 +2651,12 @@ void GameWindow::on_next_player_turn(Player *player, unsigned int turn_number)
       dialog->hide();
       show();
     }
-	
+
 }
 
 void GameWindow::on_medal_awarded_to_army(Army *army)
 {
+  GraphicsCache *gc = GraphicsCache::getInstance();
   std::auto_ptr<Gtk::Dialog> dialog;
 
   Glib::RefPtr<Gnome::Glade::Xml> xml
@@ -2530,11 +2665,14 @@ void GameWindow::on_medal_awarded_to_army(Army *army)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   Gtk::Image *image;
   xml->get_widget("image", image);
-  image->property_pixbuf() = to_pixbuf(army->getPixmap());
+  image->property_pixbuf() = to_pixbuf(gc->getArmyPic(army));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -2572,6 +2710,9 @@ void GameWindow::on_game_loaded(Player *player)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   Gtk::Label *label;
@@ -2603,6 +2744,9 @@ void GameWindow::on_quest_expired(Quest *quest)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
   Gtk::Label *label;
@@ -2661,9 +2805,12 @@ void GameWindow::on_advice_asked(float percent)
   Gtk::Dialog *d;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  Decorated decorator;
+  decorator.decorate(dialog.get());
+  decorator.window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
   dialog->set_transient_for(*window.get());
 
-  dialog->set_title(_("Advisor!"));
+  decorator.set_title(_("Advisor!"));
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -2728,3 +2875,101 @@ void GameWindow::on_advice_asked(float percent)
   return;
 }
 
+void GameWindow::on_show_lobby_activated()
+{
+  show_lobby.emit();
+}
+
+//taken from go-file.c of gnucash 2.0.4
+static char *
+check_program (char const *prog)
+{
+  if (NULL == prog)
+    return NULL;
+  if (g_path_is_absolute (prog)) {
+    if (!g_file_test (prog, G_FILE_TEST_IS_EXECUTABLE))
+      return NULL;
+  } else if (!g_find_program_in_path (prog))
+    return NULL;
+  return g_strdup (prog);
+}
+
+//taken from go-file.c of gnucash 2.0.4
+GError *
+go_url_show (gchar const *url)
+{
+  GError *err = NULL;
+  char *browser = NULL;
+  char *clean_url = NULL;
+
+  /* 1) Check BROWSER env var */
+  browser = check_program (getenv ("BROWSER"));
+
+  if (browser == NULL) {
+    static char const * const browsers[] = {
+      "sensible-browser",	/* debian */
+      "htmlview", /* fedora */
+      "firefox",
+      "epiphany",
+      "mozilla-firebird",
+      "mozilla",
+      "netscape",
+      "konqueror",
+      "xterm -e w3m",
+      "xterm -e lynx",
+      "xterm -e links"
+    };
+    unsigned i;
+    for (i = 0 ; i < G_N_ELEMENTS (browsers) ; i++)
+      if (NULL != (browser = check_program (browsers[i])))
+	break;
+  }
+
+  if (browser != NULL) {
+    gint    argc;
+    gchar **argv = NULL;
+    char   *cmd_line = g_strconcat (browser, " %1", NULL);
+
+    if (g_shell_parse_argv (cmd_line, &argc, &argv, &err)) {
+      /* check for '%1' in an argument and substitute the url
+       * 			 * otherwise append it */
+      gint i;
+      char *tmp;
+
+      for (i = 1 ; i < argc ; i++)
+	if (NULL != (tmp = strstr (argv[i], "%1"))) {
+	  *tmp = '\0';
+	  tmp = g_strconcat (argv[i],
+			     (clean_url != NULL) ? (char const *)clean_url : url,
+			     tmp+2, NULL);
+	  g_free (argv[i]);
+	  argv[i] = tmp;
+	  break;
+	}
+
+      /* there was actually a %1, drop the one we added */
+      if (i != argc-1) {
+	g_free (argv[argc-1]);
+	argv[argc-1] = NULL;
+      }
+      g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+		     NULL, NULL, NULL, &err);
+      g_strfreev (argv);
+    }
+    g_free (cmd_line);
+  }
+  g_free (browser);
+  g_free (clean_url);
+  return err;
+}
+
+void GameWindow::on_online_help_activated()
+{
+  go_url_show ("http://www.nongnu.org/lordsawar/manual/" PACKAGE_VERSION "/lordsawar.html");
+  return;
+}
+
+void GameWindow::on_player_replaced(Player *p)
+{
+  game->addPlayer(p);
+}
