@@ -28,14 +28,16 @@
 
 #include "glade-helpers.h"
 #include "image-helpers.h"
-#include "../ucompose.hpp"
-#include "../defs.h"
-#include "../hero.h"
-#include "../Item.h"
-#include "../GameMap.h"
+#include "ucompose.hpp"
+#include "defs.h"
+#include "hero.h"
+#include "Item.h"
+#include "GameMap.h"
+#include "GraphicsCache.h"
 
 HeroDialog::HeroDialog(Hero *h, Vector<int> p)
 {
+    GraphicsCache *gc = GraphicsCache::getInstance();
     hero = h;
     pos = p;
     
@@ -46,7 +48,9 @@ HeroDialog::HeroDialog(Hero *h, Vector<int> p)
     Gtk::Dialog *d = 0;
     xml->get_widget("dialog", d);
     dialog.reset(d);
-    d->set_title(hero->getName());
+    decorate(dialog.get());
+    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+    set_title(hero->getName());
 
     Gtk::Label *hero_label;
     xml->get_widget("hero_label", hero_label);
@@ -54,7 +58,7 @@ HeroDialog::HeroDialog(Hero *h, Vector<int> p)
 
     Gtk::Image *hero_image;
     xml->get_widget("hero_image", hero_image);
-    hero_image->property_pixbuf() = to_pixbuf(hero->getPixmap());
+    hero_image->property_pixbuf() = to_pixbuf(gc->getArmyPic(hero));
 
     xml->get_widget("info_label1", info_label1);
     xml->get_widget("info_label2", info_label2);
@@ -83,7 +87,7 @@ HeroDialog::HeroDialog(Hero *h, Vector<int> p)
 
 #if 0
     // debug code
-    #include "../Itemlist.h"
+    #include "Itemlist.h"
     static bool first = true;
 
     if (first)
