@@ -28,14 +28,14 @@
 #include "glade-helpers.h"
 #include "image-helpers.h"
 #include "input-helpers.h"
-#include "../ucompose.hpp"
-#include "../defs.h"
-#include "../GameMap.h"
-#include "../File.h"
-#include "../sound.h"
-#include "../city.h"
+#include "ucompose.hpp"
+#include "defs.h"
+#include "GameMap.h"
+#include "File.h"
+#include "sound.h"
+#include "city.h"
 
-HeroOfferDialog::HeroOfferDialog(Player *player, Hero *h, City *c, int gold)
+HeroOfferDialog::HeroOfferDialog(Player *player, HeroProto *h, City *c, int gold)
 {
     city = c;
     hero = h;
@@ -48,6 +48,9 @@ HeroOfferDialog::HeroOfferDialog(Player *player, Hero *h, City *c, int gold)
     xml->get_widget("dialog", d);
     dialog.reset(d);
 
+    decorate(dialog.get());
+    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+
     xml->get_widget("map_image", map_image);
 
     heromap.reset(new HeroMap(city));
@@ -57,7 +60,7 @@ HeroOfferDialog::HeroOfferDialog(Player *player, Hero *h, City *c, int gold)
     Gtk::EventBox *map_eventbox;
     xml->get_widget("map_eventbox", map_eventbox);
 
-    dialog->set_title(String::ucompose(_("Hero offer for %1"),
+    set_title(String::ucompose(_("Hero offer for %1"),
 				       player->getName()));
 
     xml->get_widget("hero_image", hero_image);
@@ -65,7 +68,7 @@ HeroOfferDialog::HeroOfferDialog(Player *player, Hero *h, City *c, int gold)
     male_radiobutton->signal_clicked().connect(
 	sigc::mem_fun(this, &HeroOfferDialog::on_male_toggled));
     
-    male_radiobutton->set_active(hero->getGender() == Army::MALE);
+    male_radiobutton->set_active(hero->getGender() == Hero::MALE);
     on_male_toggled();
     
     xml->get_widget("name", name_entry);
