@@ -21,7 +21,7 @@
 
 #include <sigc++/trackable.h>
 
-#include "../defs.h"
+#include "defs.h"
 
 // initialize the GUI and run the main loop; only one instance is ever
 // constructed so Main::instance is a convenience for retrieving it
@@ -38,7 +38,11 @@ class Main: public sigc::trackable, public noncopyable
     void stop_main_loop();
     bool iterate_main_loop();
 
+    bool start_stress_test;
+    int start_robots;
     bool start_test_scenario;
+    std::string load_filename;
+    std::string turn_filename;
     
  private:
     struct Impl;
