@@ -27,16 +27,17 @@
 #include "image-helpers.h"
 #include "input-helpers.h"
 #include "bar-chart.h"
-#include "../ucompose.hpp"
-#include "../defs.h"
-#include "../File.h"
-#include "../GameMap.h"
-#include "../playerlist.h"
-#include "../citylist.h"
-#include "../stacklist.h"
-#include "../action.h"
-#include "../GraphicsCache.h"
-#include "../armysetlist.h"
+#include "ucompose.hpp"
+#include "defs.h"
+#include "File.h"
+#include "GameMap.h"
+#include "playerlist.h"
+#include "citylist.h"
+#include "stacklist.h"
+#include "action.h"
+#include "GraphicsCache.h"
+#include "armyprodbase.h"
+#include "armysetlist.h"
 
 ReportDialog::ReportDialog(Player *player, ReportType type)
 {
@@ -47,6 +48,8 @@ ReportDialog::ReportDialog(Player *player, ReportType type)
   Gtk::Dialog *d = 0;
   xml->get_widget("dialog", d);
   dialog.reset(d);
+  decorate(dialog.get());
+  window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
 
   xml->get_widget("map_image", map_image);
   citymap.reset(new CityMap());
@@ -189,19 +192,19 @@ void ReportDialog::fill_in_info()
   switch (report_notebook->get_current_page())
     {
     case ARMY:
-      dialog->set_title(_("Army Report"));
+      set_title(_("Army Report"));
       break;
     case CITY: 
-      dialog->set_title(_("City Report"));
+      set_title(_("City Report"));
       break;
     case GOLD: 
-      dialog->set_title(_("Gold Report"));
+      set_title(_("Gold Report"));
       break;
     case PRODUCTION:
-      dialog->set_title(_("Production Report"));
+      set_title(_("Production Report"));
       break;
     case WINNING:
-      dialog->set_title(_("Winning Report"));
+      set_title(_("Winning Report"));
       break;
     }
 }
@@ -352,7 +355,7 @@ void ReportDialog::addProduction(const Action *action)
     {
       const Action_Produce *act;
       act = dynamic_cast<const Action_Produce*>(action);
-      army_type = act->getArmyType();
+      army_type = act->getArmy()->getTypeId();
       Citylist::iterator cit = Citylist::getInstance()->begin();
       for (; cit != Citylist::getInstance()->end(); ++cit)
 	if ((*cit).getId() == act->getCityId())
@@ -367,7 +370,7 @@ void ReportDialog::addProduction(const Action *action)
     {
       const Action_ProduceVectored *act;
       act = dynamic_cast<const Action_ProduceVectored*>(action);
-      army_type = act->getArmyType();
+      army_type = act->getArmy()->getTypeId();
       Vector<int> pos = act->getDestination();
       City *c = Citylist::getInstance()->getObjectAt(pos);
       s+="...";
@@ -385,7 +388,7 @@ void ReportDialog::addProduction(const Action *action)
       s = c->getName();
       s += " stops production!";
     }
-  const Army *a;
+  const ArmyProto *a;
   a = Armysetlist::getInstance()->getArmy(p->getArmyset(), army_type);
   Gtk::TreeIter i = armies_list->append();
   (*i)[armies_columns.image] = to_pixbuf(gc->getArmyPic(p->getArmyset(),
