@@ -27,9 +27,10 @@
 #include <gtkmm/container.h>
 #include <gtkmm/widget.h>
 
-#include "../game-parameters.h"
+#include "game-parameters.h"
 
 
+#include "decorated.h"
 /** The opening window of the game
   * 
   * This is the first window to pop up, where the user selects whether to start
@@ -38,7 +39,7 @@
   * The splash screen is also responsible for launching dialogs in response to
   * the menu choices.
   */
-class SplashWindow: public sigc::trackable
+class SplashWindow: public Decorated
 {
  public:
     SplashWindow();
@@ -50,7 +51,9 @@ class SplashWindow: public sigc::trackable
     Gtk::Window *get_window() {return window.get();}
 
     sigc::signal<void> sdl_initialized;
-    sigc::signal<void, std::string, bool> new_network_game_requested;
+    sigc::signal<void, std::string, unsigned short, std::string> new_remote_network_game_requested;
+    sigc::signal<void, GameParameters, int, std::string > new_hosted_network_game_requested;
+    sigc::signal<void, GameParameters> new_pbm_game_requested;
     sigc::signal<void, GameParameters> new_game_requested;
     sigc::signal<void, std::string> load_requested;
     sigc::signal<void> quit_requested;
@@ -63,10 +66,13 @@ class SplashWindow: public sigc::trackable
     Gtk::Container *sdl_container;
     Gtk::Widget *sdl_widget;
     bool on_delete_event(GdkEventAny *e);
+  
+    std::string network_game_nickname;
     
     void on_new_campaign_clicked();
     void on_new_game_clicked();
     void on_new_network_game_clicked();
+    void on_new_pbm_game_clicked();
     void on_load_game_clicked();
     void on_load_scenario_clicked();
     void on_preferences_clicked();
@@ -75,6 +81,7 @@ class SplashWindow: public sigc::trackable
 	
     void on_game_started(GameParameters g);
     void on_network_game_created(GameParameters g);
+    void on_pbm_game_created(GameParameters g);
     void on_network_game_selected(std::string ip, unsigned short port);
 
 #if 0
