@@ -24,11 +24,12 @@
 #include <gtkmm/dialog.h>
 #include <gtkmm/notebook.h>
 
-#include "../player.h"
+#include "player.h"
 
+#include "decorated.h"
 //
 //
-class TriumphsDialog: public sigc::trackable
+class TriumphsDialog: public Decorated
 {
  public:
     TriumphsDialog(Player *player);
