@@ -21,6 +21,7 @@
 
 #include "history.h"
 #include "hero.h"
+#include "heroproto.h"
 #include "city.h"
 
 using namespace std;
@@ -29,7 +30,7 @@ using namespace std;
 //#define debug(x)
 
 History::History(Type type)
-:Ownable((Player *) 0), d_type(type)
+:d_type(type)
 {
 }
 
@@ -39,8 +40,9 @@ History::~History()
 
 History* History::handle_load(XML_Helper* helper)
 {
-  Uint32 t;
-  helper->getData(t, "type");
+  std::string type_str;
+  helper->getData(type_str, "type");
+  History::Type t = historyTypeFromString(type_str);
 
   switch (t)
     {
@@ -80,6 +82,8 @@ History* History::handle_load(XML_Helper* helper)
       return (new History_DiplomacyTreachery(helper));
     case HERO_FINDS_ALLIES:
       return (new History_HeroFindsAllies(helper));
+    case END_TURN:
+      return (new History_EndTurn(helper));
     }
 
   return 0;
@@ -153,11 +157,43 @@ History* History::copy(const History* a)
       return 
 	(new History_HeroFindsAllies
           (*dynamic_cast<const History_HeroFindsAllies*>(a)));
+    case END_TURN:
+      return 
+	(new History_EndTurn(*dynamic_cast<const History_EndTurn*>(a)));
     }
 
   return 0;
 }
 
+History::History (XML_Helper *helper)
+{
+  std::string type_str;
+  helper->getData(type_str, "type");
+  d_type = historyTypeFromString(type_str);
+}
+
+bool History::save(XML_Helper* helper) const
+{
+    bool retval = true;
+
+    retval &= helper->openTag("history");
+    retval &= saveContents(helper);
+    retval &= helper->closeTag();
+
+    return retval;
+}
+
+bool History::saveContents(XML_Helper* helper) const
+{
+    bool retval = true;
+
+    std::string type_str = historyTypeToString(History::Type(d_type));
+    retval &= helper->saveData("type", type_str);
+    retval &= doSave(helper);
+
+    return retval;
+}
+
 //-----------------------------------------------------------------------------
 //History_StartTurn
 
@@ -172,7 +208,7 @@ History_StartTurn::History_StartTurn(const History_StartTurn &history)
 }
 
 History_StartTurn::History_StartTurn(XML_Helper* helper)
-:History(History::START_TURN)
+:History(helper)
 {
 }
 
@@ -189,14 +225,10 @@ std::string History_StartTurn::dump() const
   return s.str();
 }
 
-bool History_StartTurn::save(XML_Helper* helper) const
+bool History_StartTurn::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
-  retval &= helper->closeTag();
-
   return retval;
 }
 
@@ -219,7 +251,7 @@ History_FoundSage::History_FoundSage(const History_FoundSage &history)
 }
 
 History_FoundSage::History_FoundSage(XML_Helper* helper)
-:History(History::FOUND_SAGE)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
 }
@@ -237,14 +269,11 @@ std::string History_FoundSage::dump() const
   return s.str();
 }
 
-bool History_FoundSage::save(XML_Helper* helper) const
+bool History_FoundSage::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -269,7 +298,7 @@ History_GoldTotal::History_GoldTotal(const History_GoldTotal &history)
 }
 
 History_GoldTotal::History_GoldTotal(XML_Helper* helper)
-:History(History::GOLD_TOTAL)
+:History(helper)
 {
   helper->getData(d_gold, "gold");
 }
@@ -287,14 +316,11 @@ std::string History_GoldTotal::dump() const
   return s.str();
 }
 
-bool History_GoldTotal::save(XML_Helper* helper) const
+bool History_GoldTotal::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("gold", d_gold);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -319,7 +345,7 @@ History_HeroEmerges::History_HeroEmerges(const History_HeroEmerges &history)
 }
 
 History_HeroEmerges::History_HeroEmerges(XML_Helper* helper)
-:History(History::HERO_EMERGES)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
   helper->getData(d_city, "city");
@@ -338,20 +364,17 @@ std::string History_HeroEmerges::dump() const
   return s.str();
 }
 
-bool History_HeroEmerges::save(XML_Helper* helper) const
+bool History_HeroEmerges::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
   retval &= helper->saveData("city", d_city);
-  retval &= helper->closeTag();
 
   return retval;
 }
 
-bool History_HeroEmerges::fillData(Hero *hero, City *city)
+bool History_HeroEmerges::fillData(HeroProto *hero, City *city)
 {
   d_hero = hero->getName();
   d_city = city->getName();
@@ -372,7 +395,7 @@ History_CityWon::History_CityWon(const History_CityWon &history)
 }
 
 History_CityWon::History_CityWon(XML_Helper* helper)
-:History(History::CITY_WON)
+:History(helper)
 {
   helper->getData(d_city, "city");
 }
@@ -391,14 +414,11 @@ std::string History_CityWon::dump() const
   return s.str();
 }
 
-bool History_CityWon::save(XML_Helper* helper) const
+bool History_CityWon::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("city", d_city);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -423,7 +443,7 @@ History_HeroCityWon::History_HeroCityWon(const History_HeroCityWon &history)
 }
 
 History_HeroCityWon::History_HeroCityWon(XML_Helper* helper)
-:History(History::HERO_CITY_WON)
+:History(helper)
 {
   helper->getData(d_city, "city");
   helper->getData(d_hero, "hero");
@@ -444,15 +464,12 @@ std::string History_HeroCityWon::dump() const
   return s.str();
 }
 
-bool History_HeroCityWon::save(XML_Helper* helper) const
+bool History_HeroCityWon::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("city", d_city);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -478,7 +495,7 @@ History_CityRazed::History_CityRazed(const History_CityRazed &history)
 }
 
 History_CityRazed::History_CityRazed(XML_Helper* helper)
-:History(History::CITY_RAZED)
+:History(helper)
 {
   helper->getData(d_city, "city");
 }
@@ -496,14 +513,11 @@ std::string History_CityRazed::dump() const
   return s.str();
 }
 
-bool History_CityRazed::save(XML_Helper* helper) const
+bool History_CityRazed::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("city", d_city);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -528,7 +542,7 @@ History_HeroQuestStarted::History_HeroQuestStarted(const History_HeroQuestStarte
 }
 
 History_HeroQuestStarted::History_HeroQuestStarted(XML_Helper* helper)
-:History(History::HERO_QUEST_STARTED)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
 }
@@ -546,14 +560,11 @@ std::string History_HeroQuestStarted::dump() const
   return s.str();
 }
 
-bool History_HeroQuestStarted::save(XML_Helper* helper) const
+bool History_HeroQuestStarted::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -578,7 +589,7 @@ History_HeroQuestCompleted::History_HeroQuestCompleted(const History_HeroQuestCo
 }
 
 History_HeroQuestCompleted::History_HeroQuestCompleted(XML_Helper* helper)
-:History(History::HERO_QUEST_COMPLETED)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
 }
@@ -596,14 +607,11 @@ std::string History_HeroQuestCompleted::dump() const
   return s.str();
 }
 
-bool History_HeroQuestCompleted::save(XML_Helper* helper) const
+bool History_HeroQuestCompleted::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -628,7 +636,7 @@ History_HeroKilledInCity::History_HeroKilledInCity(const History_HeroKilledInCit
 }
 
 History_HeroKilledInCity::History_HeroKilledInCity(XML_Helper* helper)
-:History(History::HERO_KILLED_IN_CITY)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
   helper->getData(d_city, "city");
@@ -647,15 +655,12 @@ std::string History_HeroKilledInCity::dump() const
   return s.str();
 }
 
-bool History_HeroKilledInCity::save(XML_Helper* helper) const
+bool History_HeroKilledInCity::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
   retval &= helper->saveData("city", d_city);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -681,7 +686,7 @@ History_HeroKilledInBattle::History_HeroKilledInBattle(const History_HeroKilledI
 }
 
 History_HeroKilledInBattle::History_HeroKilledInBattle(XML_Helper* helper)
-:History(History::HERO_KILLED_IN_BATTLE)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
 }
@@ -699,14 +704,11 @@ std::string History_HeroKilledInBattle::dump() const
   return s.str();
 }
 
-bool History_HeroKilledInBattle::save(XML_Helper* helper) const
+bool History_HeroKilledInBattle::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -731,7 +733,7 @@ History_HeroKilledSearching::History_HeroKilledSearching(const History_HeroKille
 }
 
 History_HeroKilledSearching::History_HeroKilledSearching(XML_Helper* helper)
-:History(History::HERO_KILLED_SEARCHING)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
 }
@@ -749,14 +751,11 @@ std::string History_HeroKilledSearching::dump() const
   return s.str();
 }
 
-bool History_HeroKilledSearching::save(XML_Helper* helper) const
+bool History_HeroKilledSearching::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -781,7 +780,7 @@ History_Score::History_Score(const History_Score &history)
 }
 
 History_Score::History_Score(XML_Helper* helper)
-:History(History::SCORE)
+:History(helper)
 {
   helper->getData(d_score, "score");
 }
@@ -799,14 +798,11 @@ std::string History_Score::dump() const
   return s.str();
 }
 
-bool History_Score::save(XML_Helper* helper) const
+bool History_Score::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("score", d_score);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -831,7 +827,7 @@ History_PlayerVanquished::History_PlayerVanquished(const History_PlayerVanquishe
 }
 
 History_PlayerVanquished::History_PlayerVanquished(XML_Helper* helper)
-:History(History::PLAYER_VANQUISHED)
+:History(helper)
 {
 }
 
@@ -848,14 +844,10 @@ std::string History_PlayerVanquished::dump() const
   return s.str();
 }
 
-bool History_PlayerVanquished::save(XML_Helper* helper) const
+bool History_PlayerVanquished::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
-  retval &= helper->closeTag();
-
   return retval;
 }
 
@@ -873,7 +865,7 @@ History_DiplomacyPeace::History_DiplomacyPeace(const History_DiplomacyPeace &his
 }
 
 History_DiplomacyPeace::History_DiplomacyPeace(XML_Helper* helper)
-:History(History::DIPLOMATIC_PEACE)
+:History(helper)
 {
   helper->getData(d_opponent_id, "opponent_id");
 }
@@ -892,14 +884,11 @@ std::string History_DiplomacyPeace::dump() const
   return s.str();
 }
 
-bool History_DiplomacyPeace::save(XML_Helper* helper) const
+bool History_DiplomacyPeace::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("opponent_id", d_opponent_id);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -924,7 +913,7 @@ History_DiplomacyWar::History_DiplomacyWar(const History_DiplomacyWar &history)
 }
 
 History_DiplomacyWar::History_DiplomacyWar(XML_Helper* helper)
-:History(History::DIPLOMATIC_WAR)
+:History(helper)
 {
   helper->getData(d_opponent_id, "opponent_id");
 }
@@ -943,14 +932,11 @@ std::string History_DiplomacyWar::dump() const
   return s.str();
 }
 
-bool History_DiplomacyWar::save(XML_Helper* helper) const
+bool History_DiplomacyWar::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("opponent_id", d_opponent_id);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -975,7 +961,7 @@ History_DiplomacyTreachery::History_DiplomacyTreachery(const History_DiplomacyTr
 }
 
 History_DiplomacyTreachery::History_DiplomacyTreachery(XML_Helper* helper)
-:History(History::DIPLOMATIC_TREACHERY)
+:History(helper)
 {
   helper->getData(d_opponent_id, "opponent_id");
 }
@@ -994,14 +980,11 @@ std::string History_DiplomacyTreachery::dump() const
   return s.str();
 }
 
-bool History_DiplomacyTreachery::save(XML_Helper* helper) const
+bool History_DiplomacyTreachery::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("opponent_id", d_opponent_id);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1026,7 +1009,7 @@ History_HeroFindsAllies::History_HeroFindsAllies(const History_HeroFindsAllies &
 }
 
 History_HeroFindsAllies::History_HeroFindsAllies(XML_Helper* helper)
-:History(History::HERO_FINDS_ALLIES)
+:History(helper)
 {
   helper->getData(d_hero, "hero");
 }
@@ -1044,14 +1027,11 @@ std::string History_HeroFindsAllies::dump() const
   return s.str();
 }
 
-bool History_HeroFindsAllies::save(XML_Helper* helper) const
+bool History_HeroFindsAllies::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("history");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1062,3 +1042,136 @@ bool History_HeroFindsAllies::fillData(Hero *hero)
   return true;
 }
 
+//-----------------------------------------------------------------------------
+//History_EndTurn
+
+History_EndTurn::History_EndTurn()
+:History(History::END_TURN)
+{
+}
+
+History_EndTurn::History_EndTurn(const History_EndTurn &history)
+:History(history)
+{
+}
+
+History_EndTurn::History_EndTurn(XML_Helper* helper)
+:History(helper)
+{
+}
+
+History_EndTurn::~History_EndTurn()
+{
+}
+
+std::string History_EndTurn::dump() const
+{
+  std::stringstream s;
+
+  s <<"player ends a turn" << "\n";
+
+  return s.str();
+}
+
+bool History_EndTurn::doSave(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  return retval;
+}
+
+bool History_EndTurn::fillData()
+{
+  return true;
+}
+
+std::string History::historyTypeToString(const History::Type type)
+{
+  switch (type)
+    {
+    case History::START_TURN:
+      return "History::START_TURN";
+    case History::FOUND_SAGE:
+      return "History::FOUND_SAGE";
+    case History::GOLD_TOTAL:
+      return "History::GOLD_TOTAL";
+    case History::HERO_EMERGES:
+      return "History::HERO_EMERGES";
+    case History::CITY_WON:
+      return "History::CITY_WON";
+    case History::HERO_CITY_WON:
+      return "History::HERO_CITY_WON";
+    case History::CITY_RAZED:
+      return "History::CITY_RAZED";
+    case History::HERO_QUEST_STARTED:
+      return "History::HERO_QUEST_STARTED";
+    case History::HERO_QUEST_COMPLETED:
+      return "History::HERO_QUEST_COMPLETED";
+    case History::HERO_KILLED_IN_CITY:
+      return "History::HERO_KILLED_IN_CITY";
+    case History::HERO_KILLED_IN_BATTLE:
+      return "History::HERO_KILLED_IN_BATTLE";
+    case History::HERO_KILLED_SEARCHING:
+      return "History::HERO_KILLED_SEARCHING";
+    case History::SCORE:
+      return "History::SCORE";
+    case History::PLAYER_VANQUISHED:
+      return "History::PLAYER_VANQUISHED";
+    case History::DIPLOMATIC_PEACE:
+      return "History::DIPLOMATIC_PEACE";
+    case History::DIPLOMATIC_WAR:
+      return "History::DIPLOMATIC_WAR";
+    case History::DIPLOMATIC_TREACHERY:
+      return "History::DIPLOMATIC_TREACHERY";
+    case History::HERO_FINDS_ALLIES:
+      return "History::HERO_FINDS_ALLIES";
+    case History::END_TURN:
+      return "History::END_TURN";
+    }
+  return "History::START_TURN";
+}
+
+History::Type History::historyTypeFromString(const std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return History::Type(atoi(str.c_str()));
+  if (str == "History::START_TURN")
+    return History::START_TURN;
+  else if (str == "History::FOUND_SAGE")
+    return History::FOUND_SAGE;
+  else if (str == "History::GOLD_TOTAL")
+    return History::GOLD_TOTAL;
+  else if (str == "History::HERO_EMERGES")
+    return History::HERO_EMERGES;
+  else if (str == "History::CITY_WON")
+    return History::CITY_WON;
+  else if (str== "History::HERO_CITY_WON")
+    return History::HERO_CITY_WON;
+  else if (str == "History::CITY_RAZED")
+    return History::CITY_RAZED;
+  else if (str == "History::HERO_QUEST_STARTED")
+    return History::HERO_QUEST_STARTED;
+  else if (str == "History::HERO_QUEST_COMPLETED")
+    return History::HERO_QUEST_COMPLETED;
+  else if (str == "History::HERO_KILLED_IN_CITY")
+    return History::HERO_KILLED_IN_CITY;
+  else if (str == "History::HERO_KILLED_IN_BATTLE")
+    return History::HERO_KILLED_IN_BATTLE;
+  else if (str == "History::HERO_KILLED_SEARCHING")
+    return History::HERO_KILLED_SEARCHING;
+  else if (str == "History::SCORE")
+    return History::SCORE;
+  else if (str == "History::PLAYER_VANQUISHED")
+    return History::PLAYER_VANQUISHED;
+  else if (str == "History::DIPLOMATIC_PEACE")
+    return History::DIPLOMATIC_PEACE;
+  else if (str == "History::DIPLOMATIC_WAR")
+    return History::DIPLOMATIC_WAR;
+  else if (str == "History::DIPLOMATIC_TREACHERY")
+    return History::DIPLOMATIC_TREACHERY;
+  else if (str == "History::HERO_FINDS_ALLIES")
+    return History::HERO_FINDS_ALLIES;
+  else if (str == "History::END_TURN")
+    return History::END_TURN;
+  return History::START_TURN;
+}
