@@ -19,13 +19,13 @@
 #define HISTORY_H
 
 #include <string>
-#include "Ownable.h"
 #include "vector.h"
 #include <sigc++/trackable.h>
 
 class XML_Helper;
 
 class Hero;
+class HeroProto;
 class City;
 #include "army.h"
 
@@ -42,7 +42,7 @@ class City;
  * happened.
  *
  */
-class History: public Ownable
+class History
 {
     public:
 	//! A History can be one of the following kinds.
@@ -82,21 +82,25 @@ class History: public Ownable
 	  //! The player has been treacherous towards an opponent.
 	  DIPLOMATIC_TREACHERY = 17,
 	  //! A Hero finds some powerful allies.
-	  HERO_FINDS_ALLIES = 18
+	  HERO_FINDS_ALLIES = 18,
+	  //! The player has finished a turn.
+	  END_TURN = 19
         };
+	static std::string historyTypeToString(const History::Type type);
+	static History::Type historyTypeFromString(const std::string str);
                 
 	//! Default constructor.
         History(Type type);
 
+	//! Loading from XML constructor.
+	History (XML_Helper *helper);
+
 	//! Destructor.
         virtual ~History();
 
         //! Returns debug information. Needs to be overwritten by derivatives
         virtual std::string dump() const = 0;
 
-        //! Save function. See XML_Helper for information about saving.
-        virtual bool save(XML_Helper* helper) const = 0;
-        
         /** 
 	 * static load function (see XML_Helper)
          * 
@@ -115,7 +119,11 @@ class History: public Ownable
         //! Returns the id which identifies the type of History event.
         Type getType() const {return d_type;}
         
+	bool save(XML_Helper* helper) const;
+	bool saveContents(XML_Helper* helper) const;
+
     protected:
+        virtual bool doSave(XML_Helper* helper) const = 0;
         Type d_type;
 };
 
@@ -138,7 +146,7 @@ class History_StartTurn : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! This method doesn't need to be called for History_StartTurn.
         bool fillData();
@@ -165,7 +173,7 @@ class History_FoundSage : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Hero who found the sage.
         bool fillData(Hero *hero);
@@ -197,7 +205,7 @@ class History_GoldTotal : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the amount of gold pieces the Player has.
         bool fillData(int gold);
@@ -229,7 +237,7 @@ class History_HeroEmerges : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with pertinent data.
 	/**
@@ -239,7 +247,7 @@ class History_HeroEmerges : public History
 	 * @param hero    The Hero that has emergerd.
 	 * @param city    The City where the Hero has emerged.
 	 */
-        bool fillData(Hero *hero, City *city);
+        bool fillData(HeroProto *hero, City *city);
 
 	//! Get the name of the Hero who appeared.
 	std::string getHeroName() const {return d_hero;}
@@ -274,7 +282,7 @@ class History_CityWon : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the record with the City that the Player defeated.
         bool fillData(City *city);
@@ -306,7 +314,7 @@ class History_HeroCityWon: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with pertinent data.
 	/**
@@ -351,7 +359,7 @@ class History_CityRazed : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the City that was razed.
         bool fillData(City *city);
@@ -383,7 +391,7 @@ class History_HeroQuestStarted : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Hero who initiated a new Quest.
         bool fillData(Hero *hero);
@@ -415,7 +423,7 @@ class History_HeroQuestCompleted: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Hero who finished a Quest.
         bool fillData(Hero *hero);
@@ -447,7 +455,7 @@ class History_HeroKilledInCity : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with pertinent data.
 	/**
@@ -492,7 +500,7 @@ class History_HeroKilledInBattle: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Hero who was killed in battle.
         bool fillData(Hero *hero);
@@ -524,7 +532,7 @@ class History_HeroKilledSearching: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Hero who was killed while searching.
         bool fillData(Hero *hero);
@@ -556,7 +564,7 @@ class History_Score: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the player's score for this turn.
         bool fillData(Uint32 score);
@@ -588,7 +596,7 @@ class History_PlayerVanquished: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 };
 
@@ -611,7 +619,7 @@ class History_DiplomacyPeace : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Player who we are at peace with.
         bool fillData(Player *opponent);
@@ -643,7 +651,7 @@ class History_DiplomacyWar: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Player who we are at war with.
         bool fillData(Player *opponent);
@@ -675,7 +683,7 @@ class History_DiplomacyTreachery: public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the Player we performed treachery on.
         bool fillData(Player *opponent);
@@ -707,7 +715,7 @@ class History_HeroFindsAllies : public History
         std::string dump() const;
 
 	//! Save the historical event to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+        virtual bool doSave(XML_Helper* helper) const;
 
 	//! Populate the event with the name of the Hero who found allies.
         bool fillData(Hero *hero);
@@ -720,4 +728,31 @@ class History_HeroFindsAllies : public History
 	std::string d_hero;
 };
 
+//-----------------------------------------------------------------------------
+//! A permanent record of a player ending a turn.
+class History_EndTurn : public History
+{
+    public:
+	//! Default constructor.
+	History_EndTurn();
+	//! Copy constructor.
+	History_EndTurn(const History_EndTurn &history);
+	//! Load the historical event from an opened saved-game file.
+	History_EndTurn(XML_Helper* helper);
+	//! Destructor.
+	~History_EndTurn();
+
+	//! Return some debug information about this historical event.
+	std::string dump() const;
+
+	//! Save the historical event to an opened saved-game file.
+        virtual bool doSave(XML_Helper* helper) const;
+
+	//! This method doesn't need to be called for History_EndTurn.
+	bool fillData();
+    
+    private:
+};
+
+
 #endif //HISTORY_H
