@@ -37,7 +37,9 @@
 #include "path.h"
 #include "GameMap.h"
 #include "army.h"
+#include "armyprodbase.h"
 #include "hero.h"
+#include "heroproto.h"
 #include "action.h"
 #include "MoveResult.h"
 #include "Configuration.h"
@@ -47,6 +49,7 @@
 #include "game-parameters.h"
 #include "signpost.h"
 #include "history.h"
+#include "vectoredunit.h"
 
 using namespace std;
 
@@ -110,6 +113,8 @@ void NetworkPlayer::levelArmy(Army* a)
 
 void NetworkPlayer::decodeActions(std::list<Action *> actions)
 {
+  if (isDead())
+    return;
   std::list<Action*>::iterator it = actions.begin();
   pruneActionlist();
   for (; it != actions.end(); it++)
@@ -120,6 +125,7 @@ void NetworkPlayer::decodeActions(std::list<Action *> actions)
 
 void NetworkPlayer::decodeAction(const Action *a)
 {
+  d_actions.push_back(Action::copy(a));
   switch(a->getType())
     {
     case Action::STACK_MOVE:
@@ -204,6 +210,9 @@ void NetworkPlayer::decodeAction(const Action *a)
     case Action::CITY_DESTITUTE:
       return decodeActionCityTooPoorToProduce
 	(dynamic_cast<const Action_CityTooPoorToProduce*>(a));
+    case Action::INIT_TURN:
+      return decodeActionInitTurn
+	(dynamic_cast<const Action_InitTurn*>(a));
     }
 
   return;
@@ -237,25 +246,64 @@ Item *findItemById(const std::list<Item *> &l, Uint32 id)
 void NetworkPlayer::decodeActionMove(const Action_Move *action)
 {
   Stack *stack = d_stacklist->getStackById(action->getStackId());
+  if (stack == NULL)
+    {
+      printf ("couldn't find stack with id %d\n", action->getStackId());
+      printf ("is there a stack near the ending position?\n");
+      for (int x = -1; x <= 1; x++)
+	for (int y = -1; y <= 1; y++)
+	  {
+	    Vector<int> dest = action->getEndingPosition() + Vector<int>(x,y);
+	    Stack *s = d_stacklist->getOwnObjectAt(dest);
+	    printf ("stack at position %d,%d is %p\n", dest.x, dest.y, s);
+	    if (s)
+	      printf ("stack id is %d\n", s->getId());
+	  }
+    }
+  assert (stack != NULL);
   stack->moveToDest(action->getEndingPosition());
+  stack->sortForViewing(true);
   supdatingStack.emit(stack);
 }
 
 void NetworkPlayer::decodeActionSplit(const Action_Split *action)
 {
-  Stack *stack = d_stacklist->getStackById(action->getStackId());
+  Uint32 stack_id = action->getStackId();
+  Stack *stack = d_stacklist->getStackById(stack_id);
+  assert (stack != NULL);
   
+  for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
+    (*it)->setGrouped(true);
   for (unsigned int i = 0; i < MAX_STACK_SIZE; ++i) {
     Uint32 army_id = action->getGroupedArmyId(i);
     if (army_id == 0)
       continue;
 
     Army *army = stack->getArmyById(army_id);
+    if (army == NULL)
+      {
+	printf ("couldn't find army with id %d in stack %d\n", army_id, stack_id);
+	printf ("available IDs are...\n");
+	for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
+	  printf ("%d ", (*it)->getId());
+	printf ("\n");
+	exit(0);
+      }
+    assert (army != NULL);
     army->setGrouped(false);
   }
 
-  //fixme: the new stack should have an Id of getNewStackId()
-  doStackSplit(stack);
+  Stack *new_stack = doStackSplit(stack);
+  assert (new_stack != NULL);
+  if (new_stack->getId() != action->getNewStackId())
+    {
+      printf ("created stack with id %d, but expected %d\n", new_stack->getId(),
+	      action->getNewStackId());
+    }
+  assert (new_stack->getId() == action->getNewStackId());
+  new_stack->sortForViewing(true);
+  stack->sortForViewing(true);
+
 }
 
 void NetworkPlayer::decodeActionFight(const Action_Fight *action)
@@ -271,11 +319,17 @@ void NetworkPlayer::decodeActionFight(const Action_Fight *action)
          end = defender_army_ids.end(); i != end; ++i)
     defenders.push_back(findStackById(*i));
 
+  Stack *attack = &*attackers.front();
+  Stack *defend = &*defenders.front();
   Fight fight(attackers, defenders, action->getBattleHistory());
-  fight.battleFromHistory();
+  Fight::Result result = fight.battleFromHistory();
   fight_started.emit(fight);
 
   cleanupAfterFight(attackers, defenders);
+  if (result == Fight::ATTACKER_WON)
+    {
+      printf ("there are %d attackers left in %d at %d,%d\n", attack->size(),attack->getId(), attack->getPos().x, attack->getPos().y);
+    }
 }
 
 void NetworkPlayer::decodeActionJoin(const Action_Join *action)
@@ -283,7 +337,11 @@ void NetworkPlayer::decodeActionJoin(const Action_Join *action)
   Stack *receiver = d_stacklist->getStackById(action->getReceivingStackId());
   Stack *joining = d_stacklist->getStackById(action->getJoiningStackId());
 
+  assert (receiver != NULL);
+  assert (joining != NULL);
   doStackJoin(receiver, joining, false);
+  receiver->sortForViewing(true);
+  supdatingStack.emit(0);
 }
 
 void NetworkPlayer::decodeActionRuin(const Action_Ruin *action)
@@ -418,19 +476,24 @@ void NetworkPlayer::decodeActionQuest(const Action_Quest *action)
 void NetworkPlayer::decodeActionEquip(const Action_Equip *action)
 {
   Stack *stack = d_stacklist->getArmyStackById(action->getHeroId());
+  if (stack == NULL)
+    {
+      printf ("couldn't find hero with id %d\n", action->getHeroId());
+    }
+  assert (stack != NULL);
   Hero *hero = dynamic_cast<Hero *>(stack->getArmyById(action->getHeroId()));
   Item *item = 0;
 
   switch (action->getToBackpackOrToGround())
   {
   case Action_Equip::BACKPACK:
-    item = findItemById(GameMap::getInstance()->getTile(stack->getPos())->getItems(), action->getItemId());
-    doHeroPickupItem(hero, item, stack->getPos());
+    item = findItemById(GameMap::getInstance()->getTile(action->getItemPos())->getItems(), action->getItemId());
+    doHeroPickupItem(hero, item, action->getItemPos());
     break;
 
   case Action_Equip::GROUND:
     item = findItemById(hero->getBackpack(), action->getItemId());
-    doHeroDropItem(hero, item, stack->getPos());
+    doHeroDropItem(hero, item, action->getItemPos());
     break;
   }
 }
@@ -441,12 +504,20 @@ void NetworkPlayer::decodeActionLevel(const Action_Level *action)
   Army *army = stack->getArmyById(action->getArmyId());
 
   doLevelArmy(army, Army::Stat(action->getStatToIncrease()));
+  printf ("army is hero? %d\n", army->isHero());
+  printf ("new level is %d\n", army->getLevel());
 }
 
 void NetworkPlayer::decodeActionDisband(const Action_Disband *action)
 {
   Stack *stack = d_stacklist->getStackById(action->getStackId());
-  doStackDisband(stack);
+  if (stack == NULL)
+    {
+      printf ("couldn't find stack with id %d\n", action->getStackId());
+    }
+  assert (stack != NULL);
+  bool found = doStackDisband(stack);
+  assert (found == true);
 }
 
 void NetworkPlayer::decodeActionModifySignpost(const Action_ModifySignpost *act)
@@ -488,35 +559,30 @@ void NetworkPlayer::decodeActionPlant(const Action_Plant *action)
 
 void NetworkPlayer::decodeActionProduce(const Action_Produce *action)
 {
-  //Note: we don't use this to produce new armies in cities
-  //That happens during the city->nextTurn method.
-  //It is enough that we record the changes in city production, and then
-  //let the city->nextTurn method do it's job.
-  //This action is used for the Production Report instead, see
-  //gui/report-dialog.cpp
- 
-  //there's no real need to send this action back to the server because
-  //only the current player can see her own production report.
-
-  //if we wanted to decode it, we'd do this:
-  //City *city = Citylist::getInstance()->getById(action->getCityId());
-  //doCityProducesArmy(city);
+  //if it was vectored, we just wait for the Action_ProduceVectored later on.
+  if (action->getVectored() == true)
+    {
+      printf ("produced unit but it's vectored.\n");
+    return;
+    }
+  ArmyProdBase *a = action->getArmy();
+  City *c = Citylist::getInstance()->getById(action->getCityId());
+  Army *army = new Army (*a, this);
+  Stack *s = c->addArmy(army);
+  printf ("created army id %d, in stack %d of size %d\n", army->getId(), s->getId(), s->size());
+  s->sortForViewing(true);
 }
 
 void NetworkPlayer::decodeActionProduceVectored(const Action_ProduceVectored *action)
 {
-  //Note: we don't use this to produce new vectored army units on the map 
-  //That happens during the vectoredunit->nextTurn method.
-  //It is enough that we record the changes in city vectoring, and then
-  //let the vectoredunitlist->nextTurn method do it's job.
-  //This action is used for the Production Report instead, see
-  //gui/report-dialog.cpp
- 
-  //there's no real need to send this action back to the server because
-  //only the current player can see her own production report.
-
-  //we couldn't decode this action if we wanted to, because there isn't
-  //enough data to make it happen.
+  //create a vectored unit.
+  VectoredUnit v(action->getOrigination(), action->getDestination(),
+		 action->getArmy(), 0, this);
+  Army *army = doVectoredUnitArrives(&v);
+  printf ("army is %p\n", army);
+  Stack *s = d_stacklist->getArmyStackById(army->getId());
+  s->sortForViewing(true);
+  printf ("created vectored army id %d, in stack %d of size %d\n", army->getId(), s->getId(), s->size());
 }
 
 void NetworkPlayer::decodeActionDiplomacyState(const Action_DiplomacyState *action)
@@ -539,6 +605,7 @@ void NetworkPlayer::decodeActionDiplomacyScore(const Action_DiplomacyScore *acti
 
 void NetworkPlayer::decodeActionEndTurn(const Action_EndTurn *action)
 {
+  printf ("ending turn!!\n");
   ending_turn.emit();
 }
 
@@ -546,19 +613,23 @@ void NetworkPlayer::decodeActionConquerCity(const Action_ConquerCity *action)
 {
   City *city = Citylist::getInstance()->getById(action->getCityId());
   Stack *stack = d_stacklist->getStackById(action->getStackId());
-  conquerCity(city, stack);
+  doConquerCity(city, stack);
 }
 
 void NetworkPlayer::decodeActionRecruitHero(const Action_RecruitHero *action)
 {
 
   City *city = Citylist::getInstance()->getById(action->getCityId());
-  Army *ally = 0;
+  ArmyProto *ally = 0;
   if (action->getNumAllies())
     ally = Armysetlist::getInstance()->getArmy(getArmyset(),
                                                action->getAllyArmyType());
-  doRecruitHero(action->getHero(), city, action->getCost(), 
-		action->getNumAllies(), ally);
+  Hero *hero = doRecruitHero(action->getHero(), city, action->getCost(), 
+			     action->getNumAllies(), ally);
+  printf ("created hero with id %d, in stack %d\n", hero->getId(),
+	 d_stacklist->getArmyStackById(hero->getId())->getId());
+  //hero->syncNewId();
+  d_stacklist->getArmyStackById(hero->getId())->sortForViewing(true);
 }
 
 void NetworkPlayer::decodeActionRenamePlayer(const Action_RenamePlayer *action)
@@ -571,4 +642,8 @@ void NetworkPlayer::decodeActionCityTooPoorToProduce(const Action_CityTooPoorToP
   //this action is only used for reporting purposes.
 }
 
+void NetworkPlayer::decodeActionInitTurn(const Action_InitTurn*action)
+{
+}
+
 // End of file
