@@ -44,6 +44,7 @@
 #include "counter.h"
 #include "army.h"
 #include "hero.h"
+#include "heroproto.h"
 #include "Configuration.h"
 #include "GameScenarioOptions.h"
 #include "action.h"
@@ -58,6 +59,7 @@
 #include "signpost.h"
 #include "vectoredunit.h"
 #include "ucompose.hpp"
+#include "armyprodbase.h"
 
 using namespace std;
 
@@ -154,7 +156,9 @@ Player::Player(XML_Helper* helper)
     helper->getData(d_gold, "gold");
     helper->getData(d_dead, "dead");
     helper->getData(d_immortal, "immortal");
-    helper->getData(d_type, "type");
+    std::string type_str;
+    helper->getData(type_str, "type");
+    d_type = playerTypeFromString(type_str);
     helper->getData(d_upkeep, "upkeep");
     helper->getData(d_income, "income");
 
@@ -284,6 +288,8 @@ void Player::initTurn()
   clearActionlist();
   History_StartTurn* item = new History_StartTurn();
   addHistory(item);
+  Action_InitTurn* action = new Action_InitTurn();
+  addAction(action);
 }
 
 void Player::setColor(SDL_Color c)
@@ -420,12 +426,16 @@ bool Player::deleteStack(Stack* stack)
 }
 
 void Player::kill()
+{
+  doKill();
+}
+
+void Player::doKill()
 {
     if (d_immortal)
         // ignore it
         return;
-    
-                        
+
     History_PlayerVanquished* item;
     item = new History_PlayerVanquished();
     addHistory(item);
@@ -438,8 +448,8 @@ void Player::kill()
     // single cities
     Citylist* cl = Citylist::getInstance();
     for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
-        if ((*it).getOwner() == this)
-            (*it).setOwner(Playerlist::getInstance()->getNeutral());
+        if ((*it).getOwner() == this && (*it).isBurnt() == false)
+            Playerlist::getInstance()->getNeutral()->takeCityInPossession(&*it);
 
     d_diplomatic_rank = 0;
     d_diplomatic_title = std::string("");
@@ -463,7 +473,8 @@ bool Player::save(XML_Helper* helper) const
     retval &= helper->saveData("gold", d_gold);
     retval &= helper->saveData("dead", d_dead);
     retval &= helper->saveData("immortal", d_immortal);
-    retval &= helper->saveData("type", d_type);
+    std::string type_str = playerTypeToString(Player::Type(d_type));
+    retval &= helper->saveData("type", type_str);
     debug("type of " << d_name << " is " << d_type)
     retval &= helper->saveData("upkeep", d_upkeep);
     retval &= helper->saveData("income", d_income);
@@ -553,10 +564,9 @@ bool Player::save(XML_Helper* helper) const
 Player* Player::loadPlayer(XML_Helper* helper)
 {
     Type type;
-    int t;
-    
-    helper->getData(t, "type");
-    type = static_cast<Type>(t);
+    std::string type_str;
+    helper->getData(type_str, "type");
+    type = playerTypeFromString(type_str);
 
     switch (type)
     {
@@ -702,9 +712,7 @@ Stack *Player::doStackSplit(Stack* s)
 
     Army* ungrouped = s->getFirstUngroupedArmy();
     if (!ungrouped)        //no armies to split
-    {
         return 0;
-    }
 
     bool all_ungrouped = true;    //the whole stack would be split
     for (Stack::iterator it = s->begin(); it != s->end(); it++)
@@ -715,9 +723,7 @@ Stack *Player::doStackSplit(Stack* s)
         }
     }
     if (all_ungrouped)
-    {
         return 0;
-    }
 
     Stack* new_stack = new Stack(this, s->getPos());
 
@@ -986,7 +992,6 @@ MoveResult *Player::stackMove(Stack* s, Vector<int> dest, bool follow)
 	      d_stacklist->getActivestack()->sortForViewing(false);
 	      
 	    supdatingStack.emit(0);
-	    SDL_Delay(Configuration::s_displaySpeedDelay);
     
             MoveResult *moveResult = new MoveResult(moved);
             moveResult->setStepCount(stepCount);
@@ -1032,7 +1037,6 @@ MoveResult *Player::stackMove(Stack* s, Vector<int> dest, bool follow)
             stepCount++;
 
         supdatingStack.emit(0);
-        SDL_Delay(Configuration::s_displaySpeedDelay);
     
         MoveResult *moveResult = new MoveResult(true);
         moveResult->setStepCount(stepCount);
@@ -1486,6 +1490,7 @@ int Player::lootCity(City *city)
 void Player::doConquerCity(City *city, Stack *stack)
 {
   int gold = lootCity(city);
+  takeCityInPossession(city);
   sinvadingCity.emit(city, gold);
   
   History_CityWon *item = new History_CityWon();
@@ -1506,7 +1511,7 @@ void Player::conquerCity(City *city, Stack *stack)
   Action_ConquerCity *action = new Action_ConquerCity();
   action->fillData(city, stack);
   addAction(action);
-                
+
   doConquerCity(city, stack);
 }
 
@@ -1524,7 +1529,7 @@ void Player::takeCityInPossession(City* c)
 
 void Player::doCityOccupy(City *c)
 {
-  takeCityInPossession(c);
+  assert (c->getOwner() == this);
   
   soccupyingCity.emit(c, getActivestack());
   QuestsManager::getInstance()->cityOccupied(c, getActivestack());
@@ -1557,7 +1562,7 @@ void Player::doCityPillage(City *c, int& gold, int* pillaged_army_type)
       int slot = -1;
       for (i = 0; i < c->getNoOfProductionBases(); i++)
 	{
-	  const Army *a = c->getProductionBase(i);
+	  const ArmyProdBase *a = c->getProductionBase(i);
 	  if (a != NULL)
 	    {
 	      if (a->getProductionCost() == 0)
@@ -1574,9 +1579,9 @@ void Player::doCityPillage(City *c, int& gold, int* pillaged_army_type)
 	}
       if (slot > -1)
 	{
-	  const Army *a = c->getProductionBase(slot);
+	  const ArmyProdBase *a = c->getProductionBase(slot);
 	  if (pillaged_army_type)
-	    *pillaged_army_type = a->getType();
+	    *pillaged_army_type = a->getTypeId();
 	  if (a->getProductionCost() == 0)
 	    gold += 1500;
 	  else
@@ -1592,7 +1597,7 @@ void Player::doCityPillage(City *c, int& gold, int* pillaged_army_type)
       QuestsManager::getInstance()->cityPillaged(c, s, gold);
     }
 
-  takeCityInPossession(c);
+  //takeCityInPossession(c);
 }
 
 void Player::cityPillage(City* c, int& gold, int* pillaged_army_type)
@@ -1614,7 +1619,7 @@ void Player::doCitySack(City* c, int& gold, std::list<Uint32> *sacked_types)
 
   if (c->getNoOfProductionBases() > 1)
     {
-      const Army *a;
+      const ArmyProdBase *a;
       int i, max = 0;
       for (i = 0; i < c->getNoOfProductionBases(); i++)
 	{
@@ -1629,7 +1634,7 @@ void Player::doCitySack(City* c, int& gold, std::list<Uint32> *sacked_types)
 	  a = c->getProductionBase(i);
 	  if (a != NULL)
 	    {
-	      sacked_types->push_back(a->getType());
+	      sacked_types->push_back(a->getTypeId());
 	      if (a->getProductionCost() == 0)
 		gold += 1500;
 	      else
@@ -1645,7 +1650,7 @@ void Player::doCitySack(City* c, int& gold, std::list<Uint32> *sacked_types)
   Stack *s = getActivestack();
   ssackingCity.emit(c, s, gold, *sacked_types);
   QuestsManager::getInstance()->citySacked(c, s, gold);
-  takeCityInPossession(c);
+  //takeCityInPossession(c);
 }
 
 void Player::citySack(City* c, int& gold, std::list<Uint32> *sacked_types)
@@ -1691,7 +1696,7 @@ void Player::doCityBuyProduction(City* c, int slot, int type)
   Uint32 as = c->getOwner()->getArmyset();
 
   c->removeProductionBase(slot);
-  c->addProductionBase(slot, new Army(*al->getArmy(as, type)));
+  c->addProductionBase(slot, new ArmyProdBase(*al->getArmy(as, type)));
 
   // and do the rest of the neccessary actions
   withdrawGold(al->getArmy(as, type)->getProductionCost());
@@ -1752,7 +1757,7 @@ void Player::doGiveReward(Stack *s, Reward *reward)
       break;
     case Reward::ALLIES:
         {
-          const Army *a = dynamic_cast<Reward_Allies*>(reward)->getArmy();
+          const ArmyProto *a = dynamic_cast<Reward_Allies*>(reward)->getArmy();
 
           Reward_Allies::addAllies(s->getOwner(), s->getPos(), a,
       			     dynamic_cast<Reward_Allies*>(reward)->getNoOfAllies());
@@ -1775,7 +1780,7 @@ void Player::doGiveReward(Stack *s, Reward *reward)
     case Reward::MAP:
         {
           Reward_Map *map = dynamic_cast<Reward_Map*>(reward);
-          d_fogmap->alterFogRectangle(map->getPos(), 
+          d_fogmap->alterFogRectangle(map->getLocation()->getPos(), 
       				map->getHeight(), map->getWidth(), 
       				FogMap::OPEN);
         }
@@ -1797,11 +1802,12 @@ bool Player::giveReward(Stack *s, Reward *reward)
   return true;
 }
 
-void Player::doStackDisband(Stack* s)
+bool Player::doStackDisband(Stack* s)
 {
     getStacklist()->setActivestack(0);
-    getStacklist()->deleteStack(s);
+    bool found = getStacklist()->deleteStack(s);
     supdatingStack.emit(0);
+    return found;
 }
 
 bool Player::stackDisband(Stack* s)
@@ -1814,9 +1820,7 @@ bool Player::stackDisband(Stack* s)
     item->fillData(s);
     addAction(item);
 
-    doStackDisband(s);
-    
-    return true;
+    return doStackDisband(s);
 }
 
 void Player::doHeroDropItem(Hero *h, Item *i, Vector<int> pos)
@@ -1830,7 +1834,7 @@ bool Player::heroDropItem(Hero *h, Item *i, Vector<int> pos)
   doHeroDropItem(h, i, pos);
   
   Action_Equip* item = new Action_Equip();
-  item->fillData(h, i, Action_Equip::GROUND);
+  item->fillData(h, i, Action_Equip::GROUND, pos);
   addAction(item);
   
   return true;
@@ -1845,10 +1849,19 @@ bool Player::heroDropAllItems(Hero *h, Vector<int> pos)
   return true;
 }
 
+bool Player::doHeroDropAllItems(Hero *h, Vector<int> pos)
+{
+  std::list<Item*> backpack = h->getBackpack();
+  for (std::list<Item*>::iterator i = backpack.begin(), end = backpack.end();
+       i != end; ++i)
+    doHeroDropItem(h, *i, pos);
+  return true;
+}
 void Player::doHeroPickupItem(Hero *h, Item *i, Vector<int> pos)
 {
-  GameMap::getInstance()->getTile(pos)->removeItem(i);
-  h->addToBackpack(i, 0);
+  bool found = GameMap::getInstance()->getTile(pos)->removeItem(i);
+  if (found)
+    h->addToBackpack(i);
 }
 
 bool Player::heroPickupItem(Hero *h, Item *i, Vector<int> pos)
@@ -1856,7 +1869,7 @@ bool Player::heroPickupItem(Hero *h, Item *i, Vector<int> pos)
   doHeroPickupItem(h, i, pos);
   
   Action_Equip* item = new Action_Equip();
-  item->fillData(h, i, Action_Equip::BACKPACK);
+  item->fillData(h, i, Action_Equip::BACKPACK, pos);
   addAction(item);
   
   return true;
@@ -2091,7 +2104,7 @@ double Player::removeDeadArmies(std::list<Stack*>& stacks,
                         item = new History_HeroKilledSearching();
                         item->fillData(h);
 			h->getOwner()->addHistory(item);
-                        heroDropAllItems (h, (*it)->getPos());
+                        doHeroDropAllItems (h, (*it)->getPos());
                     }
                     else if (tile->getBuilding() == Maptile::CITY)
                     {
@@ -2101,7 +2114,7 @@ double Player::removeDeadArmies(std::list<Stack*>& stacks,
                         item = new History_HeroKilledInCity();
                         item->fillData(h, c);
 			h->getOwner()->addHistory(item);
-                        heroDropAllItems (h, (*it)->getPos());
+                        doHeroDropAllItems (h, (*it)->getPos());
                     }
                     else //somewhere else
                     {
@@ -2109,7 +2122,7 @@ double Player::removeDeadArmies(std::list<Stack*>& stacks,
                         item = new History_HeroKilledInBattle();
                         item->fillData(h);
 			h->getOwner()->addHistory(item);
-                        heroDropAllItems (h, (*it)->getPos());
+                        doHeroDropAllItems (h, (*it)->getPos());
                     }
                 }
                 //Add the XP bonus to the total of the battle;
@@ -2138,12 +2151,13 @@ double Player::removeDeadArmies(std::list<Stack*>& stacks,
             if (owner)
             {
                 debug("Removing this stack from the owner's stacklist");
-                owner->deleteStack(*it);
+                bool found = owner->deleteStack(*it);
+		assert (found == true);
             }
             else // there is no owner - like for the ruin's occupants
                 debug("No owner for this stack - do stacklist too");
 
-            debug("Removing from the vector too (the vetor had "
+            debug("Removing from the vector too (the vector had "
                   << stacks.size() << " elt)");
             it = stacks.erase(it);
         }
@@ -2251,7 +2265,7 @@ void Player::updateArmyValues(std::list<Stack*>& stacks, double xp_sum)
             army->setNumberHasHit(0);
             army->setNumberHasBeenHit(0);
 
-            if (army->isHero())
+            if (army->isHero() && getType() != Player::NETWORKED)
               {
                 while(army->canGainLevel())
                   {
@@ -2264,7 +2278,7 @@ void Player::updateArmyValues(std::list<Stack*>& stacks, double xp_sum)
                     //levels per time depending on the XP and level itself
 
                     debug("ADVANCING LEVEL "<< "CANGAINLEVEL== " << army->canGainLevel());
-                    army->getOwner()->levelArmy(army);
+		    army->getOwner()->levelArmy(army);
                   }
                 debug("Army new XP=" << army->getXP())
               }
@@ -2290,12 +2304,8 @@ void Player::tallyTriumph(Player *p, TriumphType type)
   d_triumph[id][type]++;
 }
 
-void Player::doRecruitHero(Hero* herotemplate, City *city, int cost, int alliesCount, const Army *ally)
+Hero* Player::doRecruitHero(HeroProto* herotemplate, City *city, int cost, int alliesCount, const ArmyProto *ally)
 {
-  History_HeroEmerges *item = new History_HeroEmerges();
-  item->fillData(herotemplate, city);
-  addHistory(item);
-
   Hero *newhero = new Hero(*herotemplate);
   newhero->setOwner(this);
   GameMap::getInstance()->addArmy(city, newhero);
@@ -2316,14 +2326,19 @@ void Player::doRecruitHero(Hero* herotemplate, City *city, int cost, int alliesC
   }
   withdrawGold(cost);
   supdatingStack.emit(0);
+  return newhero;
 }
 
-void Player::recruitHero(Hero* hero, City *city, int cost, int alliesCount, const Army *ally)
+void Player::recruitHero(HeroProto* heroproto, City *city, int cost, int alliesCount, const ArmyProto *ally)
 {
   Action_RecruitHero *action = new Action_RecruitHero();
-  action->fillData(hero, city, cost, alliesCount, ally);
+  action->fillData(heroproto, city, cost, alliesCount, ally);
   addAction(action);
-  doRecruitHero(hero, city, cost, alliesCount, ally);
+  History_HeroEmerges *item = new History_HeroEmerges();
+  item->fillData(heroproto, city);
+  addHistory(item);
+
+  Hero *hero = doRecruitHero(heroproto, city, cost, alliesCount, ally);
 }
 
 void Player::doDeclareDiplomacy (DiplomaticState state, Player *player)
@@ -2590,7 +2605,7 @@ void Player::AI_maybeBuyScout()
 	  if (c->getArmytype(i) == -1)    // no production in this slot
 	    continue;
 
-	  const Army *proto = c->getProductionBase(i);
+	  const ArmyProdBase *proto = c->getProductionBase(i);
 	  if (proto->getProduction() == 1)
 	    {
 	      one_turn_army_exists = true;
@@ -2603,8 +2618,8 @@ void Player::AI_maybeBuyScout()
 	  int free_slot = c->getFreeBasicSlot();
 	  if (free_slot == -1)
 	    free_slot = 0;
-	  Army *scout = al->getScout(getArmyset());
-	  cityBuyProduction(c, free_slot, scout->getType());
+	  ArmyProto *scout = al->getScout(getArmyset());
+	  cityBuyProduction(c, free_slot, scout->getTypeId());
 	}
     }
 }
@@ -2830,6 +2845,7 @@ bool Player::AI_maybeDisband(Stack *s, City *city, Uint32 min_defenders,
 bool Player::AI_maybeVector(City *c, Uint32 safe_mp, Uint32 min_defenders,
 			    City *target, City **vector_city)
 {
+  assert (c->getOwner() == this);
   if (vector_city)
     *vector_city = NULL;
   Citylist *cl = Citylist::getInstance();
@@ -2848,6 +2864,7 @@ bool Player::AI_maybeVector(City *c, Uint32 safe_mp, Uint32 min_defenders,
   City *near_city = cl->getNearestFriendlyVectorableCity(target->getPos());
   if (!near_city)
     return false;
+  assert (near_city->getOwner() == this);
 
   //if it's us then it's easier to just walk.
   if (near_city == c)
@@ -2860,7 +2877,7 @@ bool Player::AI_maybeVector(City *c, Uint32 safe_mp, Uint32 min_defenders,
   //can i just walk there faster?
 
   //find mp from source to target city
-  const Army *proto = c->getProductionBase(c->getActiveProductionSlot());
+  const ArmyProdBase *proto = c->getActiveProductionBase();
   Uint32 mp_from_source_city = Stack::scout(c->getOwner(), c->getPos(),
 					    target->getPos(), proto);
 
@@ -2868,7 +2885,7 @@ bool Player::AI_maybeVector(City *c, Uint32 safe_mp, Uint32 min_defenders,
   Uint32 mp_from_near_city = Stack::scout(c->getOwner(), near_city->getPos(),
 					  target->getPos(), proto);
 
-  Uint32 max_moves_per_turn = proto->getStat(Army::MOVES);
+  Uint32 max_moves_per_turn = proto->getMaxMoves();
 
   double turns_to_move_from_source_city = 
     (double)mp_from_source_city / (double)max_moves_per_turn;
@@ -2954,35 +2971,42 @@ void Player::AI_setupVectoring(Uint32 safe_mp, Uint32 min_defenders,
     }
 }
 
-void Player::doCityProducesArmy(City *city)
+const Army * Player::doCityProducesArmy(City *city)
 {
-  city->armyArrives();
+  return city->armyArrives();
 }
 
 bool Player::cityProducesArmy(City *city)
 {
   Action_Produce *item = new Action_Produce();
-  doCityProducesArmy(city);
-  const Army *army = city->getProductionBase(city->getActiveProductionSlot());
+  const Army *army = doCityProducesArmy(city);
+  const ArmyProdBase *source_army = city->getActiveProductionBase();
   if (city->getVectoring() == Vector<int>(-1, -1))
-    item->fillData(army, city, false);
+    item->fillData(source_army, city, false);
   else
-    item->fillData(army, city, true);
+    item->fillData(source_army, city, true);
   addAction(item);
   return true;
 }
 	
-void Player::doVectoredUnitArrives(VectoredUnit *unit)
+Army* Player::doVectoredUnitArrives(VectoredUnit *unit)
 {
-  unit->armyArrives();
+  Army *army = unit->armyArrives();
+  return army;
 }
 
 bool Player::vectoredUnitArrives(VectoredUnit *unit)
 {
   Action_ProduceVectored *item = new Action_ProduceVectored();
-  item->fillData(unit->getArmy()->getType(), unit->getDestination());
+  item->fillData(unit->getArmy(), unit->getDestination(), unit->getPos());
   addAction(item);
-  doVectoredUnitArrives(unit);
+  Army *army = doVectoredUnitArrives(unit);
+  if (!army)
+    {
+    printf("whooops... this vectored unit failed to show up.\n");
+    exit (1);
+    }
+
   return true;
 }
 
@@ -3000,11 +3024,10 @@ std::list<Action *> Player::getReportableActions()
   return actions;
 }
 
-	
 void Player::cityTooPoorToProduce(City *city, int slot)
 {
   cityChangeProduction(city, -1);
-  const Army *a = city->getProductionBase(slot);
+  const ArmyProdBase *a = city->getProductionBase(slot);
   Action_CityTooPoorToProduce *action = new Action_CityTooPoorToProduce();
   action->fillData(city, a);
   addAction(action);
@@ -3064,4 +3087,86 @@ void Player::pruneActionlist(std::list<Action*> actions)
 
 }
 
+std::string Player::playerTypeToString(const Player::Type type)
+{
+  switch (type)
+    {
+    case Player::HUMAN:
+      return "Player::HUMAN";
+    case Player::AI_FAST:
+      return "Player::AI_FAST";
+    case Player::AI_DUMMY:
+      return "Player::AI_DUMMY";
+    case Player::AI_SMART:
+      return "Player::AI_SMART";
+    case Player::NETWORKED:
+      return "Player::NETWORKED";
+    }
+  return "Player::HUMAN";
+}
+
+Player::Type Player::playerTypeFromString(const std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return Player::Type(atoi(str.c_str()));
+  if (str == "Player::HUMAN")
+    return Player::HUMAN;
+  else if (str == "Player::AI_FAST")
+    return Player::AI_FAST;
+  else if (str == "Player::AI_DUMMY")
+    return Player::AI_DUMMY;
+  else if (str == "Player::AI_SMART")
+    return Player::AI_SMART;
+  else if (str == "Player::NETWORKED")
+    return Player::NETWORKED;
+  return Player::HUMAN;
+}
+
+bool Player::hasAlreadyInitializedTurn() const
+{
+    for (list<Action*>::const_iterator it = d_actions.begin();
+        it != d_actions.end(); it++)
+      if ((*it)->getType() == Action::INIT_TURN)
+	return true;
+    return false;
+}
+
+bool Player::hasAlreadyEndedTurn() const
+{
+    for (list<Action*>::const_iterator it = d_actions.begin();
+        it != d_actions.end(); it++)
+      if ((*it)->getType() == Action::END_TURN)
+	return true;
+    return false;
+}
+
+std::list<History*> Player::getHistoryForThisTurn() const
+{
+  std::list<History*> history;
+  for (list<History*>::const_reverse_iterator it = d_history.rbegin();
+       it != d_history.rend(); it++)
+    {
+      history.push_front(*it);
+      if ((*it)->getType() == History::START_TURN)
+	break;
+    }
+  return history;
+}
+
+void Player::loadPbmGame() 
+{
+  for (list<Action*>::const_iterator it = d_actions.begin();
+       it != d_actions.end(); it++)
+    {
+      NetworkAction *copy = new NetworkAction(*it, this);
+      acting.emit(copy);
+    }
+  std::list<History*> history = getHistoryForThisTurn();
+  for (list<History*>::const_iterator it = history.begin();
+       it != history.end(); it++)
+    {
+      NetworkHistory *copy = new NetworkHistory(*it, this);
+      history_written.emit(copy);
+    }
+}
 // End of file
