@@ -35,11 +35,13 @@
 #include "vector.h"
 #include "fight.h"
 #include "army.h"
+#include "armyproto.h"
 #include "reward.h"
 
 class Stacklist;
 class XML_Helper;
 class Hero;
+class HeroProto;
 class Action;
 class NetworkAction;
 class History;
@@ -125,6 +127,8 @@ class Player: public sigc::trackable
 	  //! Remote player.  See the NetworkPlayer class.
 	  NETWORKED = 8
 	};
+	static std::string playerTypeToString(const Player::Type type);
+	static Player::Type playerTypeFromString(const std::string str);
 
 	//! Every player has a diplomatic state with every other player.
 	enum DiplomaticState {
@@ -530,7 +534,7 @@ class Player: public sigc::trackable
 	 * 
          * @note Only change the name and gender attributes of the Hero.
          */
-        void recruitHero(Hero* herotemplate, City *city, int cost, int alliesCount, const Army *ally);
+        void recruitHero(HeroProto* hero, City *city, int cost, int alliesCount, const ArmyProto *ally);
 
 	void rename (std::string name);
         /** 
@@ -1148,7 +1152,7 @@ class Player: public sigc::trackable
 	 * @return True if we're accepting a hero, false if not.
 	 */
         //! Emitted whenever a hero is recruited.
-        sigc::signal<bool, Hero*, City *, int> srecruitingHero;
+        sigc::signal<bool, HeroProto*, City *, int> srecruitingHero;
 
 	/**
 	 * @param army   The army that has gained a level.
@@ -1292,6 +1296,9 @@ class Player: public sigc::trackable
 	//! is it safe to vector from the given city?
 	static bool safeFromAttack(City *c, Uint32 safe_mp, Uint32 min_defenders);
 	void addHistory(History *history);
+	bool hasAlreadyInitializedTurn() const;
+	bool hasAlreadyEndedTurn() const;
+	void loadPbmGame();
     protected:
         // do some fight cleaning up, setting
         void cleanupAfterFight(std::list<Stack*> &attackers,
@@ -1390,9 +1397,10 @@ class Player: public sigc::trackable
         void doCityChangeProduction(City *c, int slot);
         void doGiveReward(Stack *s, Reward *reward);
         void doHeroDropItem(Hero *hero, Item *item, Vector<int> pos);
+	bool doHeroDropAllItems(Hero *h, Vector<int> pos);
         void doHeroPickupItem(Hero *hero, Item *item, Vector<int> pos);
         void doLevelArmy(Army *army, Army::Stat stat);
-        void doStackDisband(Stack *stack);
+        bool doStackDisband(Stack *stack);
         void doSignpostChange(Signpost *signpost, std::string message);
         void doCityRename(City *c, std::string name);
         void doVectorFromCity(City * c, Vector<int> dest);
@@ -1402,10 +1410,11 @@ class Player: public sigc::trackable
         void doDeclareDiplomacy (DiplomaticState state, Player *player);
         void doProposeDiplomacy (DiplomaticProposal proposal, Player *player);
         void doConquerCity(City *city, Stack *stack);
-        void doRecruitHero(Hero* herotemplate, City *city, int cost, int alliesCount, const Army *ally);
+        Hero* doRecruitHero(HeroProto* hero, City *city, int cost, int alliesCount, const ArmyProto *ally);
         void doRename(std::string name);
-	void doCityProducesArmy(City *city);
-	void doVectoredUnitArrives(VectoredUnit *unit);
+	void doKill();
+	const Army *doCityProducesArmy(City *city);
+	Army *doVectoredUnitArrives(VectoredUnit *unit);
 
 	void AI_maybeBuyScout();
 
@@ -1457,6 +1466,7 @@ class Player: public sigc::trackable
 
 	void pruneActionlist();
 	static void pruneActionlist(std::list<Action*> actions);
+	std::list<History*> getHistoryForThisTurn() const;
 	    
     private:
         //! Loads the subdata of a player (actions and stacklist)
