@@ -25,6 +25,7 @@
 #include <sigc++/functors/mem_fun.h>
 
 #include "playerlist.h"
+#include "armysetlist.h"
 
 #include "citylist.h"
 #include "ruinlist.h"
@@ -38,6 +39,7 @@
 #include "real_player.h"
 #include "ai_smart.h"
 #include "ai_fast.h"
+#include "network_player.h"
 #include "GameMap.h"
 
 using namespace std;
@@ -541,7 +543,7 @@ bool Playerlist::randomly(const Player *lhs, const Player *rhs)
 {
   Playerlist *pl = Playerlist::getInstance();
   if (lhs == pl->getNeutral())
-    return true;
+    return false;
   if (rand() % 2 == 0)
     return true;
   else
@@ -699,3 +701,103 @@ void Playerlist::syncPlayers(std::vector<GameParameters::Player> players)
     syncPlayer(*i);
 
 }
+	
+void Playerlist::turnHumansIntoNetworkPlayers()
+{
+  for (iterator i = begin(); i != end(); i++)
+    {
+      if ((*i)->getType() == Player::HUMAN)
+	{
+	  NetworkPlayer *new_p = new NetworkPlayer(**i);
+	  swap((*i), new_p);
+	  //delete *i; fixme
+	  i = begin();
+	  continue;
+	}
+    }
+}
+
+void Playerlist::turnHumansInto(Player::Type type, int number_of_players)
+{
+  int count = 0;
+  for (iterator i = begin(); i != end(); i++)
+    {
+      if (count >= number_of_players && number_of_players > 0)
+	break;
+      if ((*i)->getType() == Player::HUMAN)
+	{
+	  switch (type)
+	    {
+	    case Player::AI_FAST:
+		{
+		  AI_Fast *new_p = new AI_Fast(**i);
+		  swap((*i), new_p);
+		  //delete *i; fixme
+		  i = begin();
+		  count++;
+		  continue;
+		}
+	      break;
+	    case Player::AI_SMART:
+		{
+		  AI_Smart *new_p = new AI_Smart(**i);
+		  swap((*i), new_p);
+		  //delete *i; fixme
+		  i = begin();
+		  count++;
+		  continue;
+		}
+	      break;
+	    case Player::NETWORKED:
+		{
+		  NetworkPlayer *new_p = new NetworkPlayer(**i);
+		  swap((*i), new_p);
+		  //delete *i; fixme
+		  i = begin();
+		  count++;
+		  continue;
+		}
+	      break;
+	    case Player::HUMAN:
+	      break;
+	    }
+	}
+    }
+}
+
+std::list<Uint32> given_turn_order;
+bool Playerlist::inGivenOrder(const Player *lhs, const Player *rhs)  
+{
+  if (given_turn_order.size() == 0)
+    return true;
+
+  int count = 0;
+  for(std::list<Uint32>::iterator it = given_turn_order.begin(); 
+      it != given_turn_order.end(); it++)
+    {
+      count++;
+      if (lhs->getId() == (*it))
+	break;
+    }
+  int lhs_rank = count;
+  count = 0;
+  for(std::list<Uint32>::iterator it = given_turn_order.begin(); 
+      it != given_turn_order.end(); it++)
+    {
+      count++;
+      if (rhs->getId() == (*it))
+	break;
+    }
+  int rhs_rank = count;
+  return lhs_rank > rhs_rank;
+}
+
+
+void Playerlist::reorder(std::list<Uint32> order)
+{
+  given_turn_order = order;
+  sort(inGivenOrder);
+  given_turn_order.clear();
+  d_activeplayer = getFirstLiving();
+}
+
