@@ -223,6 +223,10 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
         //! Emitted when a surrender is offered.
         sigc::signal<void, Player*> ssurrender;
     
+	void turnHumansIntoNetworkPlayers();
+	void turnHumansInto(Player::Type type, int num_players = -1);
+	void reorder(std::list<Uint32> order);
+
     protected:
 	//! Default constructor.
         Playerlist();
@@ -238,6 +242,9 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	//! Comparison function to assist in sorting the list of players.
 	static bool randomly(const Player *lhs, const Player *rhs);
 
+	//! Comparison function to assist in sorting the list of players.
+	static bool inGivenOrder(const Player *lhs, const Player *rhs);
+
 	//! Comparison function to assist in sorting the list of players.
 	static bool inOrderOfId(const Player *lhs, const Player *rhs);
 
