@@ -28,6 +28,7 @@
 
 #include "real_player.h"
 #include "action.h"
+#include "history.h"
 #include "playerlist.h"
 #include "stacklist.h"
 #include "citylist.h"
@@ -82,6 +83,8 @@ bool RealPlayer::startTurn()
 
 void RealPlayer::endTurn()
 {
+  History *history = new History_EndTurn();
+  addHistory(history);
   pruneActionlist();
   Action *action = new Action_EndTurn;
   addAction(action);
@@ -126,14 +129,14 @@ bool RealPlayer::maybeRecruitHero ()
   if (this == Playerlist::getInstance()->getNeutral())
     return false;
   
-  City *city;
+  City *city = NULL;
   int gold_needed = 0;
   if (Citylist::getInstance()->countCities(this) == 0)
     return false;
   //give the player a hero if it's the first round.
   //otherwise we get a hero based on chance
   //a hero costs a random number of gold pieces
-  if (GameScenarioOptions::s_round == 1)
+  if (GameScenarioOptions::s_round == 1 && d_stacklist->countHeroes() == 0)
     gold_needed = 0;
   else
     {
@@ -151,17 +154,9 @@ bool RealPlayer::maybeRecruitHero ()
   if (((((rand() % 6) == 0) && (gold_needed < getGold())) 
        || gold_needed == 0))
     {
-      Hero *herotemplate =HeroTemplates::getInstance()->getRandomHero(getId());
-#if 0
-      const Armysetlist* al = Armysetlist::getInstance();
-      const Army *heroType = al->getArmy (p->getArmyset(), 
-					  templateHero->getType());
-      Hero* newhero = new Hero(*heroType, templateHero->getName(), p);
-      newhero->setGender(Army::Gender(templateHero->getGender()));
-#endif
+      HeroProto *heroproto=HeroTemplates::getInstance()->getRandomHero(getId());
       if (gold_needed == 0)
 	{
-	  //city = Citylist::getInstance()->getFirstCity(this);
 	  //we do it this way because maybe quickstart is on.
 	  Citylist* cl = Citylist::getInstance();
 	  for (Citylist::iterator it = cl->begin(); it != cl->end(); ++it)
@@ -171,6 +166,8 @@ bool RealPlayer::maybeRecruitHero ()
 		city = &*it;
 		break;
 	      }
+	  if (!city) //no capital cities
+	    city = Citylist::getInstance()->getFirstCity(this);
 	}
       else
 	{
@@ -186,8 +183,8 @@ bool RealPlayer::maybeRecruitHero ()
 
       if (srecruitingHero.empty())
         accepted = true;
-      else
-        accepted = srecruitingHero.emit(herotemplate, city, gold_needed);
+      else if (city)
+        accepted = srecruitingHero.emit(heroproto, city, gold_needed);
 
       if (accepted) {
         /* now maybe add a few allies */
@@ -201,7 +198,7 @@ bool RealPlayer::maybeRecruitHero ()
         else
           alliesCount = 0;
 
-        const Army *ally = 0;
+        const ArmyProto *ally = 0;
         if (alliesCount > 0)
         {
           ally = Reward_Allies::randomArmyAlly();
@@ -209,7 +206,7 @@ bool RealPlayer::maybeRecruitHero ()
             alliesCount = 0;
         }
         
-        recruitHero(herotemplate, city, gold_needed, alliesCount, ally);
+        recruitHero(heroproto, city, gold_needed, alliesCount, ally);
       }
     }
   return accepted;
