@@ -39,25 +39,17 @@ Reward::Reward(Type type, std::string name)
 
 Reward::Reward(XML_Helper *helper)
 {
-  Uint32 t;
-  helper->getData(t, "type");
-  d_type = static_cast<Reward::Type>(t);
+  std::string type_str;
+  helper->getData(type_str, "type");
+  d_type = rewardTypeFromString(type_str);
   helper->getData(d_name, "name");
 }
-        
+
 Reward::Reward (const Reward& orig)
 	:d_type(orig.d_type), d_name(orig.d_name)
 {
 }
 
-bool Reward::save(XML_Helper* helper) const
-{
-  bool retval = true;
-  retval &= helper->saveData("type", d_type);
-  retval &= helper->saveData("name", d_name);
-  return retval;
-}
-
 Reward::~Reward()
 {
 }
@@ -65,7 +57,9 @@ Reward::~Reward()
 Reward* Reward::handle_load(XML_Helper* helper)
 {
     Uint32 t;
-    helper->getData(t, "type");
+    std::string type_str;
+    helper->getData(type_str, "type");
+    t = rewardTypeFromString(type_str);
 
     switch (t)
     {
@@ -100,11 +94,13 @@ Reward_Gold::Reward_Gold (const Reward_Gold & orig)
 {
 }
 
-bool Reward_Gold::save(XML_Helper* helper) const
+bool Reward_Gold::save(XML_Helper* helper)
 {
   bool retval = true;
   retval &= helper->openTag("reward");
-  retval &= Reward::save(helper);
+  std::string type_str = rewardTypeToString(Reward::Type(d_type));
+  retval &= helper->saveData("type", type_str);
+  retval &= helper->saveData("name", d_name);
   retval &= helper->saveData("gold", d_gold);
   retval &= helper->closeTag();
   return retval;
@@ -128,10 +124,10 @@ Reward_Allies::Reward_Allies(Uint32 army_type, Uint32 army_set, Uint32 count)
   d_army  = al->getArmy (army_set, army_type);
 }
 
-Reward_Allies::Reward_Allies(const Army *army, Uint32 count)
+Reward_Allies::Reward_Allies(const ArmyProto *army, Uint32 count)
     :Reward(Reward::ALLIES), d_count(count)
 {
-  d_army_type = army->getType();
+  d_army_type = army->getTypeId();
   d_army_set = army->getArmyset();
   d_army = army;
 }
@@ -152,11 +148,13 @@ Reward_Allies::Reward_Allies (const Reward_Allies& orig)
 {
 }
 
-bool Reward_Allies::save(XML_Helper* helper) const
+bool Reward_Allies::save(XML_Helper* helper)
 {
   bool retval = true;
   retval &= helper->openTag("reward");
-  retval &= Reward::save(helper);
+  std::string type_str = rewardTypeToString(Reward::Type(d_type));
+  retval &= helper->saveData("type", type_str);
+  retval &= helper->saveData("name", d_name);
   retval &= helper->saveData("num_allies", d_count);
   retval &= helper->saveData("ally_type", d_army_type);
   retval &= helper->saveData("ally_armyset", d_army_set);
@@ -169,18 +167,18 @@ const Uint32 Reward_Allies::getRandomAmountOfAllies()
   return (rand() % MAX_STACK_SIZE) + 1;
 }
 
-const Army* Reward_Allies::randomArmyAlly()
+const ArmyProto* Reward_Allies::randomArmyAlly()
 {
   Uint32 allytype;
   // list all the army types that can be allies.
-  std::vector<const Army*> allytypes;
+  std::vector<const ArmyProto*> allytypes;
   Armysetlist *al = Armysetlist::getInstance();
   Player *p = Playerlist::getInstance()->getActiveplayer();
   if (!p)
     p = Playerlist::getInstance()->getNeutral();
   for (unsigned int j = 0; j < al->getSize(p->getArmyset()); j++)
     {
-      const Army *a = al->getArmy (p->getArmyset(), j);
+      const ArmyProto *a = al->getArmy (p->getArmyset(), j);
       if (a->getAwardable())
         allytypes.push_back(a);
     }
@@ -191,7 +189,7 @@ const Army* Reward_Allies::randomArmyAlly()
   return allytypes[allytype];
 }
 
-bool Reward_Allies::addAllies(Player *p, Vector<int> pos, const Army *army, Uint32 alliesCount)
+bool Reward_Allies::addAllies(Player *p, Vector<int> pos, const ArmyProto *army, Uint32 alliesCount)
 {
   for (unsigned int i = 0; i < alliesCount; i++)
     {
@@ -247,11 +245,13 @@ Reward_Item::Reward_Item (const Reward_Item& orig)
 {
 }
 
-bool Reward_Item::save(XML_Helper* helper) const
+bool Reward_Item::save(XML_Helper* helper)
 {
   bool retval = true;
   retval &= helper->openTag("reward");
-  retval &= Reward::save(helper);
+  std::string type_str = rewardTypeToString(Reward::Type(d_type));
+  retval &= helper->saveData("type", type_str);
+  retval &= helper->saveData("name", d_name);
   retval &= d_item->save(helper);
   retval &= helper->closeTag();
   return retval;
@@ -262,7 +262,7 @@ Item *Reward_Item::getRandomItem()
   Itemlist *il = Itemlist::getInstance();
   Itemlist::iterator it = il->begin();
   std::advance(it, rand() % il->size());
-  Item *i = it->second;
+  ItemProto *i = it->second;
   return new Item(*i);
 }
 
@@ -291,11 +291,13 @@ Reward_Ruin::Reward_Ruin (const Reward_Ruin& orig)
 	:Reward(orig), d_ruin_pos(orig.d_ruin_pos)
 {
 }
-bool Reward_Ruin::save(XML_Helper* helper) const
+bool Reward_Ruin::save(XML_Helper* helper)
 {
   bool retval = true;
   retval &= helper->openTag("reward");
-  retval &= Reward::save(helper);
+  std::string type_str = rewardTypeToString(Reward::Type(d_type));
+  retval &= helper->saveData("type", type_str);
+  retval &= helper->saveData("name", d_name);
   retval &= helper->saveData("x", getRuin()->getPos().x);
   retval &= helper->saveData("y", getRuin()->getPos().y);
   retval &= helper->closeTag();
@@ -345,35 +347,55 @@ Reward_Ruin::~Reward_Ruin()
 
 Reward_Map::Reward_Map(Vector<int> pos, std::string name, 
 		       Uint32 height, Uint32 width)
-    :Reward(Reward::MAP, name), Location(pos), d_height(height), d_width(width)
+    :Reward(Reward::MAP, name), d_height(height), d_width(width)
 {
+  d_loc = new LocationBox(pos);
+  d_map_name = name;
 }
 
-Reward_Map::Reward_Map(XML_Helper* helper)
-    :Reward(helper), Location(helper)
+bool Reward_Map::loadMap(std::string tag, XML_Helper* helper)
 {
-  helper->getData(d_height, "height");
-  helper->getData(d_width, "width");
+  if (tag == "map")
+    {
+      d_loc = new LocationBox(helper);
+      helper->getData(d_height, "height");
+      helper->getData(d_width, "width");
+      return true;
+    }
+    
+  return false;
+}
+
+
+Reward_Map::Reward_Map(XML_Helper* helper)
+    :Reward(helper)
+{
+  helper->registerTag("map", sigc::mem_fun(this, &Reward_Map::loadMap));
 }
 
 Reward_Map::Reward_Map (const Reward_Map& orig)
-	:Reward(orig), Location(orig),
-	d_height(orig.d_height), d_width(orig.d_width)
+	:Reward(orig), d_height(orig.d_height), d_width(orig.d_width),
+	d_map_name(orig.d_map_name)
 {
+  d_loc = new LocationBox(*orig.d_loc);
 }
 
-bool Reward_Map::save(XML_Helper* helper) const
+//FIXME: location has a name, and so does the reward!
+bool Reward_Map::save(XML_Helper* helper)
 {
   bool retval = true;
   retval &= helper->openTag("reward");
-  retval &= Reward::save(helper);
-  retval &= helper->saveData("id", getId());
-  retval &= helper->saveData("x", getPos().x);
-  retval &= helper->saveData("y", getPos().y);
-  retval &= helper->saveData("name", getName());
+  std::string type_str = rewardTypeToString(Reward::Type(d_type));
+  retval &= helper->saveData("type", type_str);
+  retval &= helper->saveData("name", d_name);
+  retval &= helper->openTag("map");
+  retval &= helper->saveData("x", d_loc->getPos().x);
+  retval &= helper->saveData("y", d_loc->getPos().y);
+  retval &= helper->saveData("map_name", d_map_name);
   retval &= helper->saveData("height", d_height);
   retval &= helper->saveData("width", d_width);
   retval &= helper->closeTag();
+  retval &= helper->closeTag();
   return retval;
 }
 
@@ -429,8 +451,8 @@ std::string Reward::getDescription()
 	{
 	  Reward_Map *m = dynamic_cast<Reward_Map *>(this);
 	  s += String::ucompose(_("Map: %1,%2 %3x%4"), 
-				  m->getPos().x,
-				  m->getPos().y,
+				  m->getLocation()->getPos().x,
+				  m->getLocation()->getPos().y,
 				  m->getHeight(),
 				  m->getWidth());
 	  return s;
@@ -438,3 +460,62 @@ std::string Reward::getDescription()
     }
   return s;
 }
+
+std::string Reward::rewardTypeToString(const Reward::Type type)
+{
+  switch (type)
+    {
+      case Reward::GOLD:
+	return "Reward::GOLD";
+	break;
+      case Reward::ALLIES:
+	return "Reward::ALLIES";
+	break;
+      case Reward::ITEM:
+	return "Reward::ITEM";
+	break;
+      case Reward::RUIN:
+	return "Reward::RUIN";
+	break;
+      case Reward::MAP:
+	return "Reward::MAP";
+	break;
+    }
+  return "Reward::GOLD";
+}
+
+Reward::Type Reward::rewardTypeFromString(const std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return Reward::Type(atoi(str.c_str()));
+  if (str == "Reward::GOLD")
+    return Reward::GOLD;
+  else if (str == "Reward::ALLIES")
+    return Reward::ALLIES;
+  else if (str == "Reward::ITEM")
+    return Reward::ITEM;
+  else if (str == "Reward::RUIN")
+    return Reward::RUIN;
+  else if (str == "Reward::MAP")
+    return Reward::MAP;
+  return Reward::GOLD;
+}
+
+Reward* Reward::copy(const Reward* r)
+{
+  switch(r->getType())
+    {
+    case  Reward::GOLD:
+      return (new Reward_Gold(*dynamic_cast<const Reward_Gold*>(r)));
+    case  Reward::ALLIES:
+      return (new Reward_Allies(*dynamic_cast<const Reward_Allies*>(r)));
+    case Reward::ITEM:
+      return (new Reward_Item(*dynamic_cast<const Reward_Item*>(r)));
+    case Reward::RUIN:
+      return (new Reward_Ruin(*dynamic_cast<const Reward_Ruin*>(r)));
+    case Reward::MAP:
+      return (new Reward_Map(*dynamic_cast<const Reward_Map*>(r)));
+    }
+
+  return 0;
+}
