@@ -45,7 +45,9 @@ Ruin::Ruin(XML_Helper* helper)
     d_owner(0), d_sage(0), d_reward(0)
 {
     Uint32 ui;
-    helper->getData(d_type, "type");
+    std::string type_str;
+    helper->getData(type_str, "type");
+    d_type = ruinTypeFromString(type_str);
     helper->getData(d_searched, "searched");
     helper->getData(d_sage, "sage");
     helper->getData(d_hidden, "hidden");
@@ -81,7 +83,8 @@ bool Ruin::save(XML_Helper* helper) const
   retval &= helper->saveData("x", getPos().x);
   retval &= helper->saveData("y", getPos().y);
   retval &= helper->saveData("name", getName());
-  retval &= helper->saveData("type", d_type);
+  std::string type_str = ruinTypeToString(Ruin::Type(d_type));
+  retval &= helper->saveData("type", type_str);
   retval &= helper->saveData("searched", d_searched);
   retval &= helper->saveData("sage", d_sage);
   retval &= helper->saveData("hidden", d_hidden);
@@ -92,18 +95,7 @@ bool Ruin::save(XML_Helper* helper) const
   if (d_occupant)
     retval &= d_occupant->save(helper);
   if (d_sage == false && d_reward)
-    {
-      if (d_reward->getType() == Reward::GOLD)
-	static_cast<Reward_Gold*>(d_reward)->save(helper);
-      else if (d_reward->getType() == Reward::ALLIES)
-	static_cast<Reward_Allies*>(d_reward)->save(helper);
-      else if (d_reward->getType() == Reward::ITEM)
-	static_cast<Reward_Item*>(d_reward)->save(helper);
-      else if (d_reward->getType() == Reward::RUIN)
-	static_cast<Reward_Ruin*>(d_reward)->save(helper);
-      else if (d_reward->getType() == Reward::MAP)
-	static_cast<Reward_Map*>(d_reward)->save(helper);
-    }
+    retval &= d_reward->save(helper);
   retval &= helper->closeTag();
 
   return retval;
@@ -113,21 +105,24 @@ bool Ruin::load(std::string tag, XML_Helper* helper)
 {
   if (tag == "reward")
     {
-      Reward *reward = new Reward(helper);
-      if (reward->getType() == Reward::GOLD)
-	d_reward = new Reward_Gold(helper);
-      else if (reward->getType() == Reward::ALLIES)
-	d_reward = new Reward_Allies(helper);
-      else if (reward->getType() == Reward::ITEM)
-	d_reward = new Reward_Item(helper);
-      else if (reward->getType() == Reward::RUIN)
-	d_reward = new Reward_Ruin(helper);
-      else if (reward->getType() == Reward::MAP)
-	d_reward = new Reward_Map(helper);
-
-      delete reward;
-
-      return true;
+	Uint32 t;
+	std::string type_str;
+	helper->getData(type_str, "type");
+	t = Reward::rewardTypeFromString(type_str);
+	switch (t)
+	  {
+	  case  Reward::GOLD:
+	    d_reward = new Reward_Gold(helper); break;
+	  case  Reward::ALLIES:
+	    d_reward = new Reward_Allies(helper); break;
+	  case Reward::ITEM:
+	    d_reward = new Reward_Item(helper); break;
+	  case Reward::RUIN:
+	    d_reward = new Reward_Ruin(helper); break;
+	  case Reward::MAP:
+	    d_reward = new Reward_Map(helper); break;
+	  }
+	return true;
     }
 
   if (tag == "stack")
@@ -151,7 +146,7 @@ void Ruin::populateWithRandomReward()
     setReward(new Reward_Gold(Reward_Gold::getRandomGoldPieces()));
   else if (num == 1)
     {
-      const Army *a = Reward_Allies::randomArmyAlly();
+      const ArmyProto *a = Reward_Allies::randomArmyAlly();
       setReward(new Reward_Allies(a, Reward_Allies::getRandomAmountOfAllies()));
     }
   else if (num == 2)
@@ -163,4 +158,29 @@ void Ruin::populateWithRandomReward()
 	setReward(new Reward_Gold(Reward_Gold::getRandomGoldPieces()));
     }
 }
+
+std::string Ruin::ruinTypeToString(const Ruin::Type type)
+{
+  switch (type)
+    {
+      case Ruin::RUIN:
+	return "Ruin::RUIN";
+	break;
+      case Ruin::STRONGHOLD:
+	return "Ruin::STRONGHOLD";
+	break;
+    }
+  return "Ruin::RUIN";
+}
+
+Ruin::Type Ruin::ruinTypeFromString(const std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return Ruin::Type(atoi(str.c_str()));
+  if (str == "Ruin::RUIN")
+    return Ruin::RUIN;
+  else if (str == "Ruin::STRONGHOLD")
+    return Ruin::STRONGHOLD;
+  return Ruin::RUIN;
+}
 // End of file
