@@ -109,12 +109,6 @@ bool Shieldsetlist::loadShieldset(std::string name)
   return true;
 }
         
-void Shieldsetlist::instantiatePixmaps()
-{
-  for (iterator it = begin(); it != end(); it++)
-    (*it)->instantiatePixmaps();
-}
-	
 SDL_Color Shieldsetlist::getColor(std::string shieldset, Uint32 owner)
 {
   Shieldset *s = getShieldset(shieldset);
