@@ -32,6 +32,7 @@
 #include "counter.h"
 #include "army.h"
 #include "citylist.h"
+#include "templelist.h"
 #include "hero.h"
 #include "GameMap.h"
 #include "vector.h"
@@ -53,6 +54,13 @@ Stack::Stack(Player* player, Vector<int> pos)
     d_path = new Path();
 }
 
+Stack::Stack(Uint32 id, Player* player, Vector<int> pos)
+    : UniquelyIdentified(id), Movable(pos), Ownable(player), 
+    d_defending(false), d_parked(false), d_deleting(false)
+{
+    d_path = new Path();
+}
+
 Stack::Stack(Stack& s)
     : UniquelyIdentified(s), Movable(s), Ownable(s), 
     d_defending(s.d_defending), d_parked(s.d_parked), d_deleting(false)
@@ -63,16 +71,14 @@ Stack::Stack(Stack& s)
     
     for (iterator sit = s.begin(); sit != s.end(); sit++)
     {
-        Army* a;
-        Army* h;
 	if ((*sit)->isHero())
 	  {
-	    h = new Hero(dynamic_cast<Hero&>(**sit));
+	    Army *h = new Hero(dynamic_cast<Hero&>(**sit));
 	    push_back(h);
 	  }
 	else
 	  {
-	    a = new Army((**sit), (*sit)->getOwner());
+	    Army *a = new Army((**sit), (*sit)->getOwner());
 	    push_back(a);
 	  }
     }
@@ -93,7 +99,8 @@ Stack::Stack(XML_Helper* helper)
 Stack::~Stack()
 {
   d_deleting = true;
-  sdying.emit(this);
+  if (d_unique)
+    sdying.emit(this);
 
   delete d_path;
   flClear();
@@ -369,7 +376,8 @@ int Stack::bless()
   int count = 0;
   for (iterator it = begin(); it != end(); it++)
     {
-      if ((*it)->bless())
+      Temple *temple = Templelist::getInstance()->getObjectAt(getPos());
+      if ((*it)->bless(temple))
 	count++;
     }
   return count;
@@ -434,6 +442,12 @@ Uint32 Stack::getMaxSight() const
   return max;
 }
 
+void Stack::payUpkeep(Player *p)
+{
+  for (iterator it = begin(); it != end(); ++it)
+      p->withdrawGold((*it)->getUpkeep());
+}
+
 void Stack::nextTurn()
 {
   Uint32 movement_multiplier = 1;
@@ -456,34 +470,12 @@ void Stack::nextTurn()
   for (const_iterator it = begin(); it != end(); it++)
     (*it)->setStat(Army::MOVES_MULTIPLIER, movement_multiplier);
 
-  //now let's see if we have any items that give us gold per city
-  for (const_iterator it = begin(); it != end(); it++)
-    if ((*it)->isHero())
-      {
-	std::list<Item*> backpack = dynamic_cast<Hero*>((*it))->getBackpack();
-	std::list<Item*>::const_iterator item;
-	for (item = backpack.begin(); item != backpack.end(); item++)
-	  {
-	    Player *p = d_owner;
-	    if ((*item)->getBonus(Item::ADD2GOLDPERCITY))
-	      p->addGold(2 * Citylist::getInstance()->countCities(p));
-	    if ((*item)->getBonus(Item::ADD3GOLDPERCITY))
-	      p->addGold(3 * Citylist::getInstance()->countCities(p));
-	    if ((*item)->getBonus(Item::ADD4GOLDPERCITY))
-	      p->addGold(4 * Citylist::getInstance()->countCities(p));
-	    if ((*item)->getBonus(Item::ADD5GOLDPERCITY))
-	      p->addGold(5 * Citylist::getInstance()->countCities(p));
-	  }
-      }
   if (d_defending == true)
     setFortified(true);
 
   for (iterator it = begin(); it != end(); ++it)
     {
       (*it)->resetMoves();
-      // TODO: should be moved in a more appropriate place => class Player
-      if (d_owner)
-	d_owner->withdrawGold((*it)->getUpkeep());
       (*it)->heal();
     }
 
@@ -678,8 +670,8 @@ bool Stack::armyCompareFightOrder (const Army *lhs, const Army *rhs)
 {
   std::list<Uint32> lhs_fight_order = lhs->getOwner()->getFightOrder();
   std::list<Uint32> rhs_fight_order = rhs->getOwner()->getFightOrder();
-  Uint32 lhs_rank = getFightOrder (lhs_fight_order, lhs->getType());
-  Uint32 rhs_rank = getFightOrder (rhs_fight_order, rhs->getType());
+  Uint32 lhs_rank = getFightOrder (lhs_fight_order, lhs->getTypeId());
+  Uint32 rhs_rank = getFightOrder (rhs_fight_order, rhs->getTypeId());
   //if (lhs_rank == rhs_rank)
     //return lhs->getId() < rhs->getId();
   return lhs_rank < rhs_rank; 
@@ -753,7 +745,7 @@ std::vector<Uint32> Stack::determineReachableArmies(Vector<int> dest)
     {
       if ((*it)->getMoves() > 0)
 	{
-	  Stack *stack = new Stack(getOwner(), getPos());
+	  Stack *stack = Stack::createNonUniqueStack(getOwner(), getPos());
 	  stack->push_back(*it);
 	  if (stack->getGroupMoves() >= 
 	      stack->getPath()->calculate(stack, dest))
@@ -774,7 +766,7 @@ std::vector<Uint32> Stack::determineReachableArmies(Vector<int> dest)
 	continue;
       if ((*it)->getMoves() > 0)
 	{
-	  Stack *stack = new Stack(getOwner(), getPos());
+	  Stack *stack = Stack::createNonUniqueStack(getOwner(), getPos());
 	  stack->push_back(*it);
 	  //also push back the rest of the known reachables
 	  std::vector<Uint32>::iterator iit = ids.begin();
@@ -807,16 +799,21 @@ Uint32 Stack::countArmiesBlessedAtTemple(Uint32 temple_id)
 }
 	
 Uint32 Stack::scout(Player *p, Vector<int> src, Vector<int> dest, 
-		    const Army *proto)
+		    const ArmyProdBase *prodbase)
 {
-  Stack *stack = new Stack(p, src);
+  Stack *stack = Stack::createNonUniqueStack(p, src);
 
-  if (!proto)
-    proto = Armysetlist::getInstance()->getScout(p->getArmyset());
-  if (!proto)
-    return 0;
+  Army *army;
+  if (!prodbase)
+    {
+      ArmyProto *proto = Armysetlist::getInstance()->getScout(p->getArmyset());
+      if (!proto)
+	return 0;
+      army = Army::createNonUniqueArmy (*proto, p);
+    }
+  else
+    army = Army::createNonUniqueArmy (*prodbase, p);
 
-  Army *army = new Army (*proto);
   if (!army)
     return 0;
   stack->push_back(army);
@@ -824,6 +821,7 @@ Uint32 Stack::scout(Player *p, Vector<int> src, Vector<int> dest,
   delete stack;
   return mp;
 }
+
 Uint32 Stack::scout(Stack *stack, Vector<int> dest)
 {
   Stack *scout_stack = new Stack(*stack);
@@ -831,4 +829,10 @@ Uint32 Stack::scout(Stack *stack, Vector<int> dest)
   delete scout_stack;
   return mp;
 }
+
+Stack* Stack::createNonUniqueStack(Player *player, Vector<int> pos)
+{
+  return new Stack(0, player, pos);
+}
+
 // End of file
