@@ -36,6 +36,7 @@
 class Player;
 class Path;
 class Army;
+class ArmyProdBase;
 class XML_Helper;
 class Temple;
 
@@ -74,6 +75,9 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 	//! Destructor.
         ~Stack();
 
+	//! Create a stack with an id that isn't unique.
+	static Stack* createNonUniqueStack(Player *player, Vector<int> pos);
+
         //! Change the loyalty of the stack.
         void setPlayer(Player* p);
 
@@ -333,12 +337,17 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 
 	//! how many movement points is it for a scout to travel this line.
 	static Uint32 scout(Player *p, Vector<int> src, Vector<int> dest, 
-			    const Army *proto = NULL);
+			    const ArmyProdBase *proto = NULL);
 
 	//! how many movement points is it for the stack to travel this line
 	static Uint32 scout(Stack *stack, Vector<int> dest);
+
+	void payUpkeep(Player *p);
     private:    
 
+
+	Stack(Uint32 id, Player* player, Vector<int> pos);
+
         //! Callback for loading the stack
         bool load(std::string tag, XML_Helper* helper);
     
