@@ -33,6 +33,7 @@
 class City;
 class Stack;
 class XML_Helper;
+class Player;
 
 //! A list of Stack objects for a single player.
 /** 
@@ -115,7 +116,7 @@ class Stacklist : public std::list<Stack*>, public sigc::trackable
         static Stack* getAmbiguity(Stack* stack);
 
         //! Searches through the all players Stacklists and deletes the stack.
-        static void deleteStack(Stack* stack);
+        static bool deleteStack(Stack* stack);
 
         /** 
 	 * Scan each tile occupied by the given city and return a list of
@@ -138,6 +139,9 @@ class Stacklist : public std::list<Stack*>, public sigc::trackable
         //! Returns the total number of armies in the list.
         unsigned int countArmies();
 
+        //! Returns the total number of heroes in the list.
+        unsigned int countHeroes();
+
         /** 
 	 * Sets the currently selected stack. The purpose of this method is 
          * to designate a stack to be the one the player is currently touching.
@@ -163,6 +167,7 @@ class Stacklist : public std::list<Stack*>, public sigc::trackable
 
         //! Callback method executed at the end of every turn.
         void nextTurn();
+	void payUpkeep(Player *p);
 
 	/**
          * @return True if any stacks in the list have enough moves for 
@@ -215,14 +220,19 @@ class Stacklist : public std::list<Stack*>, public sigc::trackable
 	 */
         //! Return the stack at position (x,y) or 0 if there is none.
         Stack* getOwnObjectAt(int x, int y);
+        Stack* getOwnObjectAt(Vector<int> point)
+	  { return getOwnObjectAt(point.x, point.y);}
 
         Stack *getStackById(Uint32 id);
         Stack *getArmyStackById(Uint32 army);
 
+	void collectTaxes(Player *p, Uint32 num_cities);
     private:
         //! Callback function for loading.
         bool load(std::string tag, XML_Helper* helper);
 
+
+	void getHeroes(std::vector<Uint32>& dst);
 	//! A pointer to the currently selected Stack.
         Stack* d_activestack;
 };
