@@ -17,7 +17,6 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <SDL_image.h>
 #include <sigc++/functors/mem_fun.h>
 
 #include "tileset.h"
@@ -34,9 +33,11 @@ using namespace std;
 #define DEFAULT_TILE_SIZE 40
 	
 Tileset::Tileset(std::string name)
-	: d_name(name), d_dir(""), d_tileSize(DEFAULT_TILE_SIZE)
+	: d_name(name), d_tileSize(DEFAULT_TILE_SIZE), d_dir("")
 {
   d_info = "";
+  d_large_selector = "misc/selector.png";
+  d_small_selector = "misc/small_selector.png";
 }
 
 Tileset::Tileset(XML_Helper *helper)
@@ -44,6 +45,8 @@ Tileset::Tileset(XML_Helper *helper)
     helper->getData(d_name, "name"); 
     helper->getData(d_info, "info");
     helper->getData(d_tileSize, "tilesize");
+    helper->getData(d_large_selector, "large_selector");
+    helper->getData(d_small_selector, "small_selector");
     helper->registerTag("tile", sigc::mem_fun((*this), &Tileset::loadTile));
     helper->registerTag("smallmap", sigc::mem_fun((*this), &Tileset::loadTile));
     helper->registerTag("tilestyle", sigc::mem_fun((*this), &Tileset::loadTile));
@@ -56,12 +59,6 @@ Tileset::~Tileset()
         delete (*this)[i];
 }
 
-void Tileset::instantiatePixmaps()
-{
-    for (Uint32 i = 0; i < size(); i++)
-      (*this)[i]->instantiatePixmaps(d_dir, d_tileSize);
-}
-
 Uint32 Tileset::getIndex(Tile::Type type) const
 {
     for (Uint32 i = 0; i < size(); i++)
@@ -136,6 +133,7 @@ bool Tileset::loadTile(string tag, XML_Helper* helper)
 	// create a new tile style set with the information we got
 	// put it on the latest tile
 	TileStyleSet* tilestyleset = new TileStyleSet(helper);
+	tilestyleset->setSubDir(getSubDir());
 	tile->push_back(tilestyleset);
 	return true;
       }
@@ -160,6 +158,8 @@ bool Tileset::save(XML_Helper *helper)
   retval &= helper->saveData("name", d_name);
   retval &= helper->saveData("info", d_info);
   retval &= helper->saveData("tilesize", d_tileSize);
+  retval &= helper->saveData("large_selector", d_large_selector);
+  retval &= helper->saveData("small_selector", d_small_selector);
   for (Tileset::iterator i = begin(); i != end(); ++i)
     retval &= (*i)->save(helper);
   retval &= helper->closeTag();
@@ -197,4 +197,12 @@ int Tileset::getFreeTileStyleId()
     }
   return -1;
 }
+
+void Tileset::setSubDir(std::string dir)
+{
+  d_dir = dir;
+  for (Tileset::iterator i = begin(); i != end(); ++i)
+    for (Tile::iterator j = (*i)->begin(); j != (*i)->end(); j++)
+      (*j)->setSubDir(dir);
+}
 // End of file
