@@ -107,9 +107,3 @@ bool Tilesetlist::loadTileset(std::string name)
 
   return true;
 }
-        
-void Tilesetlist::instantiatePixmaps()
-{
-  for (iterator it = begin(); it != end(); it++)
-    (*it)->instantiatePixmaps();
-}
