@@ -28,20 +28,28 @@
 #include "GameMap.h"
 #include "action.h"
 
-VectoredUnit::VectoredUnit(Vector<int> pos, Vector<int> dest, Army *army, int duration, Player *player)
-    :Ownable(player), Location(pos), d_destination(dest), d_army(army), 
-     d_duration(duration)
+VectoredUnit::VectoredUnit(Vector<int> pos, Vector<int> dest, ArmyProdBase *army, int duration, Player *player)
+    :Ownable(player), LocationBox(pos), d_destination(dest), 
+    d_duration(duration)
 {
+  if (army)
+    d_army = new ArmyProdBase(*army);
+  else
+    d_army = NULL;
 }
 
 VectoredUnit::VectoredUnit(const VectoredUnit& v)
-    :Ownable(v), Location(v), d_destination(v.d_destination), d_army(v.d_army), 
+    :Ownable(v), LocationBox(v), d_destination(v.d_destination),
      d_duration(v.d_duration)
 {
+  if (v.d_army)
+    d_army = new ArmyProdBase(*v.d_army);
+  else
+    d_army = NULL;
 }
 
 VectoredUnit::VectoredUnit(XML_Helper* helper)
-    :Ownable(helper), Location(helper), d_army(NULL), d_duration(0)
+    :Ownable(helper), LocationBox(helper), d_army(NULL)
 {
     helper->getData(d_duration, "duration");
     helper->getData(d_destination.x, "dest_x");
@@ -61,7 +69,6 @@ bool VectoredUnit::save(XML_Helper* helper) const
     std::string name = "";
 
     retval &= helper->openTag("vectoredunit");
-    retval &= helper->saveData("id", d_id);
     retval &= helper->saveData("x", getPos().x);
     retval &= helper->saveData("y", getPos().y);
     retval &= helper->saveData("name", name);
@@ -72,50 +79,61 @@ bool VectoredUnit::save(XML_Helper* helper) const
         retval &= helper->saveData("owner", d_owner->getId());
     else
         retval &= helper->saveData("owner", -1);
-    retval &= d_army->save(helper, Army::PRODUCTION_BASE);
+    retval &= d_army->save(helper);
     retval &= helper->closeTag();
 
     return retval;
 }
 
-bool VectoredUnit::armyArrives()
+Army *VectoredUnit::armyArrives()
 {
   Citylist *cl = Citylist::getInstance();
-  Army *a = new Army(*d_army, d_owner);
 
   City *dest;
   // drop it in the destination city!
   dest = cl->getObjectAt(d_destination);
   if (!dest)
     {
-      std::list<Item*> items;
-      items = GameMap::getInstance()->getTile(d_destination)->getItems();
-      for (std::list<Item*>::iterator it = items.begin(); 
-	   it != items.end(); it++)
+      if (d_destination == Vector<int>(-1,-1))
+	return NULL;
+      Maptile *tile = GameMap::getInstance()->getTile(d_destination);
+      if (tile)
 	{
-	  if ((*it)->getPlanted() == true &&
-	      (*it)->getPlantableOwner() == d_owner)
+	  std::list<Item*> items = tile->getItems();
+	  for (std::list<Item*>::iterator it = items.begin(); 
+	       it != items.end(); it++)
 	    {
-	      Location loc = Location(d_destination);
-	      loc.addArmy(a);
-	      break;
-	    }
+	      if ((*it)->getPlanted() == true &&
+		  (*it)->getPlantableOwner() == d_owner)
+		{
+		  //army arrives on a planted standard
+		  Army *a = new Army(*d_army, d_owner);
+		  LocationBox loc = LocationBox(d_destination);
+		  loc.addArmy(a);
+		  return a;
+		}
 
+	    }
 	}
     }
   else
     {
       if (!dest->isBurnt() && dest->getOwner() == d_owner)
-	dest->addArmy(a);
+	{
+	  //army arrives in a city
+	  Army *a = new Army(*d_army, d_owner);
+	  dest->addArmy(a);
+	  return a;
+	}
     }
-  return true;
+  return NULL;
 }
 
 bool VectoredUnit::nextTurn()
 {
   d_duration--;
   if (d_duration == 0)
-    d_owner->vectoredUnitArrives(this);
+    return d_owner->vectoredUnitArrives(this);
   return false;
 }
 
